@@ -19,6 +19,10 @@ drgndoc_substitutions = [
 ]
 
 extlinks = {
+    "contrib": (
+        "https://github.com/osandov/drgn/blob/main/contrib/%s",
+        "%s",
+    ),
     "linux": (
         "https://git.kernel.org/pub/scm/linux/kernel/git/torvalds/linux.git/tree/%s",
         "%s",
