@@ -12,6 +12,7 @@ kernel log buffer.
 from typing import Dict, List, NamedTuple, Optional, Tuple
 
 from drgn import Object, Program, cast, sizeof
+from drgn.helpers.common.prog import takes_program_or_default
 
 __all__ = (
     "get_dmesg",
@@ -158,12 +159,23 @@ def _get_printk_records_lockless(prog: Program, prb: Object) -> List[PrintkRecor
 
 
 def _get_printk_records_structured(prog: Program) -> List[PrintkRecord]:
-    printk_logp_type = prog.type("struct printk_log *")
+    try:
+        printk_logp_type = prog.type("struct printk_log *")
+    except LookupError:
+        # Before Linux kernel commit 62e32ac3505a ("printk: rename struct log
+        # to struct printk_log") (in v3.11), records were "struct log" instead
+        # of "struct printk_log". RHEL 7 kernel still uses old naming.
+        printk_logp_type = prog.type("struct log *")
+
     have_caller_id = printk_logp_type.type.has_member("caller_id")
     LOG_CONT = prog["LOG_CONT"].value_()
 
     result = []
-    log_buf = prog["log_buf"].read_()
+    # Between Linux kernel commits cbd357008604 ("bpf: verifier (add ability to
+    # receive verification log)") (in v3.18) and e7bf8249e8f1 ("bpf:
+    # encapsulate verifier log state into a structure") (in v4.15),
+    # kernel/bpf/verifier.c also contains a variable named log_buf.
+    log_buf = prog.object("log_buf", filename="printk.c").read_()
     current_idx = prog["log_first_idx"].read_()
     next_idx = prog["log_next_idx"].read_()
     seq = prog["log_first_seq"].value_()
@@ -212,6 +224,7 @@ def _get_printk_records_structured(prog: Program) -> List[PrintkRecord]:
     return result
 
 
+@takes_program_or_default
 def get_printk_records(prog: Program) -> List[PrintkRecord]:
     """Get a list of records in the kernel log buffer."""
     # Linux kernel commit 896fbe20b4e2 ("printk: use the lockless ringbuffer")
@@ -224,6 +237,7 @@ def get_printk_records(prog: Program) -> List[PrintkRecord]:
         return _get_printk_records_lockless(prog, prb)
 
 
+@takes_program_or_default
 def get_dmesg(prog: Program) -> bytes:
     """
     Get the contents of the kernel log buffer formatted like
