@@ -19,15 +19,11 @@
 #include "arch_x86_64_defs.inc"
 
 static const struct drgn_cfi_row default_dwarf_cfi_row_x86_64 = DRGN_CFI_ROW(
-	/*
-	 * The System V psABI defines the CFA as the value of rsp in the calling
-	 * frame.
-	 */
+	// The System V psABI defines the CFA as the value of rsp in the calling
+	// frame.
 	[DRGN_REGISTER_NUMBER(rsp)] = { DRGN_CFI_RULE_CFA_PLUS_OFFSET },
-	/*
-	 * Other callee-saved registers default to DW_CFA_same_value. This isn't
-	 * explicitly documented in the psABI, but it seems to be the consensus.
-	 */
+	// Other callee-saved registers default to DW_CFA_same_value. This isn't
+	// explicitly documented in the psABI, but it seems to be the consensus.
 	DRGN_CFI_SAME_VALUE_INIT(DRGN_REGISTER_NUMBER(rbx)),
 	DRGN_CFI_SAME_VALUE_INIT(DRGN_REGISTER_NUMBER(rbp)),
 	DRGN_CFI_SAME_VALUE_INIT(DRGN_REGISTER_NUMBER(r12)),
@@ -168,16 +164,14 @@ orc_to_cfi_x86_64(const struct drgn_orc_entry *orc,
 				       &rule))					\
 		return &drgn_enomem;						\
 } while (0)
-		/*
-		 * This ORC entry is for an interrupt handler before it saves
-		 * the whole pt_regs. These registers are not clobbered before
-		 * they are saved, so they should have the same value. See Linux
-		 * kernel commit 81b67439d147 ("x86/unwind/orc: Fix premature
-		 * unwind stoppage due to IRET frames").
-		 *
-		 * This probably also applies to other registers, but to stay on
-		 * the safe side we only handle registers used by ORC.
-		 */
+		// This ORC entry is for an interrupt handler before it saves
+		// the whole pt_regs. These registers are not clobbered before
+		// they are saved, so they should have the same value. See Linux
+		// kernel commit 81b67439d147 ("x86/unwind/orc: Fix premature
+		// unwind stoppage due to IRET frames").
+		//
+		// This probably also applies to other registers, but to stay on
+		// the safe side we only handle registers used by ORC.
 		SET_SAME_VALUE_RULE(r10);
 		SET_SAME_VALUE_RULE(r13);
 		SET_SAME_VALUE_RULE(rdi);
@@ -243,6 +237,44 @@ get_registers_from_frame_pointer(struct drgn_program *prog,
 	return NULL;
 }
 
+// Unwind from a call instruction, assuming that nothing else has been changed
+// since.
+static struct drgn_error *unwind_call(struct drgn_program *prog,
+				      struct drgn_register_state *regs,
+				      struct drgn_register_state **ret)
+{
+	struct drgn_error *err;
+
+	struct optional_uint64 rsp =
+		drgn_register_state_get_u64(prog, regs, rsp);
+	if (!rsp.has_value)
+		return &drgn_stop;
+
+	// Read the return address from the top of the stack.
+	uint64_t ret_addr;
+	err = drgn_program_read_u64(prog, rsp.value, false, &ret_addr);
+	if (err) {
+		if (err->code == DRGN_ERROR_FAULT) {
+			drgn_error_destroy(err);
+			err = &drgn_stop;
+		}
+		return err;
+	}
+
+	// Most of the registers are unchanged.
+	struct drgn_register_state *tmp = drgn_register_state_dup(regs);
+	if (!tmp)
+		return &drgn_enomem;
+
+	// The PC and rip are the return address we just read.
+	drgn_register_state_set_pc(prog, tmp, ret_addr);
+	drgn_register_state_set_from_u64(prog, tmp, rip, ret_addr);
+	// rsp is after the saved return address.
+	drgn_register_state_set_from_u64(prog, tmp, rsp, rsp.value + 8);
+	*ret = tmp;
+	return NULL;
+}
+
 static struct drgn_error *
 fallback_unwind_x86_64(struct drgn_program *prog,
 		       struct drgn_register_state *regs,
@@ -250,6 +282,13 @@ fallback_unwind_x86_64(struct drgn_program *prog,
 {
 	struct drgn_error *err;
 
+	// If the program counter is 0, it's likely that a NULL function pointer
+	// was called. Assume that the only thing we need to unwind is a single
+	// call instruction.
+	struct optional_uint64 pc = drgn_register_state_get_pc(regs);
+	if (pc.has_value && pc.value == 0)
+		return unwind_call(prog, regs, ret);
+
 	struct optional_uint64 rbp =
 		drgn_register_state_get_u64(prog, regs, rbp);
 	if (!rbp.has_value)
@@ -398,47 +437,44 @@ linux_kernel_get_initial_registers_x86_64(const struct drgn_object *task_obj,
 	struct drgn_error *err;
 	struct drgn_program *prog = drgn_object_program(task_obj);
 
-	struct drgn_object sp_obj;
-	drgn_object_init(&sp_obj, prog);
+	DRGN_OBJECT(sp_obj, prog);
 
 	err = drgn_object_member_dereference(&sp_obj, task_obj, "thread");
 	if (err)
-		goto out;
+		return err;
 	err = drgn_object_member(&sp_obj, &sp_obj, "sp");
 	if (err)
-		goto out;
+		return err;
 
-	/*
-	 * Since Linux kernel commit 0100301bfdf5 ("sched/x86: Rewrite the
-	 * switch_to() code") (in v4.9), sp points to a struct
-	 * inactive_task_frame, which we can use to get the callee-saved
-	 * registers. Before that, sp points to bp. As long as frame pointers
-	 * are enabled, this in turn points to the previous bp and the return
-	 * address.
-	 */
+	// Since Linux kernel commit 0100301bfdf5 ("sched/x86: Rewrite the
+	// switch_to() code") (in v4.9), sp points to a struct
+	// inactive_task_frame, which we can use to get the callee-saved
+	// registers. Before that, sp points to bp. As long as frame pointers
+	// are enabled, this in turn points to the previous bp and the return
+	// address.
 	struct drgn_qualified_type frame_type;
 	err = drgn_program_find_type(prog, "struct inactive_task_frame *", NULL,
 				     &frame_type);
 	if (!err) {
 		err = drgn_object_cast(&sp_obj, frame_type, &sp_obj);
 		if (err)
-			goto out;
+			return err;
 		err = drgn_object_dereference(&sp_obj, &sp_obj);
 		if (err)
-			goto out;
+			return err;
 		err = get_initial_registers_inactive_task_frame(&sp_obj, ret);
 	} else if (err->code == DRGN_ERROR_LOOKUP) {
 		drgn_error_destroy(err);
 		err = drgn_program_find_type(prog, "void **", NULL,
 					     &frame_type);
 		if (err)
-			goto out;
+			return err;
 		err = drgn_object_cast(&sp_obj, frame_type, &sp_obj);
 		if (err)
-			goto out;
+			return err;
 		err = drgn_object_dereference(&sp_obj, &sp_obj);
 		if (err)
-			goto out;
+			return err;
 		uint64_t frame_pointer;
 		err = drgn_object_read_unsigned(&sp_obj, &frame_pointer);
 		if (err)
@@ -450,8 +486,6 @@ linux_kernel_get_initial_registers_x86_64(const struct drgn_object *task_obj,
 						"invalid frame pointer");
 		}
 	}
-out:
-	drgn_object_deinit(&sp_obj);
 	return err;
 }
 
@@ -470,10 +504,8 @@ apply_elf_reloc_x86_64(const struct drgn_relocating_section *relocating,
 		return drgn_reloc_add32(relocating, r_offset, r_addend,
 					sym_value
 					- (relocating->addr + r_offset));
-	/*
-	 * The only difference between 32 and 32S is how overflow is checked,
-	 * which we don't do.
-	 */
+	// The only difference between 32 and 32S is how overflow is checked,
+	// which we don't do.
 	case R_X86_64_32:
 	case R_X86_64_32S:
 		return drgn_reloc_add32(relocating, r_offset, r_addend,
@@ -493,19 +525,17 @@ linux_kernel_live_direct_mapping_fallback_x86_64(struct drgn_program *prog,
 						 uint64_t *size_ret)
 {
 	struct drgn_error *err;
-	unsigned long page_offset_base_address;
 
 	*size_ret = UINT64_C(1) << 46;
+	unsigned long page_offset_base_address;
 	err = proc_kallsyms_symbol_addr("page_offset_base",
 					&page_offset_base_address);
 	if (!err) {
 		return drgn_program_read_word(prog, page_offset_base_address,
 					      false, address_ret);
 	} else if (err == &drgn_not_found) {
-		/*
-		 * This is only called for pre-4.11 kernels, so we can assume
-		 * the old location.
-		 */
+		// This is only called for pre-4.11 kernels, so we can assume
+		// the old location.
 		*address_ret = UINT64_C(0xffff880000000000);
 		return NULL;
 	} else {
@@ -530,7 +560,6 @@ linux_kernel_pgtable_iterator_create_x86_64(struct drgn_program *prog,
 	return NULL;
 }
 
-
 static void linux_kernel_pgtable_iterator_destroy_x86_64(struct pgtable_iterator *_it)
 {
 	free(container_of(_it, struct pgtable_iterator_x86_64, it));
@@ -555,14 +584,14 @@ linux_kernel_pgtable_iterator_next_x86_64(struct drgn_program *prog,
 	static const int PGTABLE_SHIFT = 9;
 	static const int PGTABLE_MASK = (1 << PGTABLE_SHIFT) - 1;
 	static const uint64_t PRESENT = 0x1;
-	static const uint64_t PSE = 0x80; /* a.k.a. huge page */
+	static const uint64_t PSE = 0x80; // a.k.a. huge page
 	static const uint64_t ADDRESS_MASK = UINT64_C(0xffffffffff000);
 	struct drgn_error *err;
 	bool bswap = drgn_platform_bswap(&prog->platform);
 	struct pgtable_iterator_x86_64 *it =
 		container_of(_it, struct pgtable_iterator_x86_64, it);
 	uint64_t virt_addr = it->it.virt_addr;
-	int levels = prog->vmcoreinfo.pgtable_l5_enabled ? 5 : 4, level;
+	int levels = prog->vmcoreinfo.pgtable_l5_enabled ? 5 : 4;
 
 	uint64_t start_non_canonical =
 		(UINT64_C(1) <<
@@ -577,16 +606,16 @@ linux_kernel_pgtable_iterator_next_x86_64(struct drgn_program *prog,
 		return NULL;
 	}
 
-	/* Find the lowest level with cached entries. */
+	// Find the lowest level with cached entries.
+	int level;
 	for (level = 0; level < levels; level++) {
 		if (it->index[level] < array_size(it->table[level]))
 			break;
 	}
-	/* For every level below that, refill the cache/return pages. */
+	// For every level below that, refill the cache/return pages.
 	for (;; level--) {
 		uint64_t table;
 		bool table_physical;
-		uint16_t index;
 		if (level == levels) {
 			table = it->it.pgtable;
 			table_physical = false;
@@ -609,12 +638,11 @@ linux_kernel_pgtable_iterator_next_x86_64(struct drgn_program *prog,
 			}
 			table_physical = true;
 		}
-		index = (virt_addr >>
-			 (PAGE_SHIFT + PGTABLE_SHIFT * (level - 1))) & PGTABLE_MASK;
-		/*
-		 * It's only marginally more expensive to read 4096 bytes than 8
-		 * bytes, so we always read to the end of the table.
-		 */
+		uint16_t index = ((virt_addr >>
+				  (PAGE_SHIFT + PGTABLE_SHIFT * (level - 1)))
+				  & PGTABLE_MASK);
+		// It's only marginally more expensive to read 4096 bytes than 8
+		// bytes, so we always read to the end of the table.
 		err = drgn_program_read_memory(prog,
 					       &it->table[level - 1][index],
 					       table + 8 * index,
