@@ -17,15 +17,23 @@
 #include <unistd.h>
 
 #include "binary_buffer.h"
+#include "cleanup.h"
 #include "debug_info.h"
 #include "elf_file.h"
 #include "error.h"
 #include "linux_kernel.h"
+#include "openmp.h"
 #include "platform.h"
 #include "program.h"
 #include "util.h"
 
-DEFINE_VECTOR_FUNCTIONS(drgn_module_vector)
+static inline Dwarf *drgn_elf_file_dwarf_key(struct drgn_elf_file * const *entry)
+{
+	return (*entry)->dwarf;
+}
+DEFINE_HASH_TABLE_FUNCTIONS(drgn_elf_file_dwarf_table, drgn_elf_file_dwarf_key,
+			    ptr_key_hash_pair, scalar_key_eq);
+DEFINE_VECTOR_FUNCTIONS(drgn_module_vector);
 
 struct drgn_module_key {
 	const void *build_id;
@@ -60,13 +68,17 @@ static inline bool drgn_module_key_eq(const struct drgn_module_key *a,
 		a->start == b->start && a->end == b->end);
 }
 DEFINE_HASH_TABLE_FUNCTIONS(drgn_module_table, drgn_module_key,
-			    drgn_module_key_hash_pair, drgn_module_key_eq)
+			    drgn_module_key_hash_pair, drgn_module_key_eq);
 
-DEFINE_HASH_SET_FUNCTIONS(c_string_set, c_string_key_hash_pair, c_string_key_eq)
+DEFINE_HASH_SET_FUNCTIONS(c_string_set, c_string_key_hash_pair,
+			  c_string_key_eq);
 
 /**
  * @c Dwfl_Callbacks::find_elf() implementation.
  *
+ * If the ELF file was reported directly, this returns it. Otherwise, it falls
+ * back to an appropriate callback.
+ *
  * Ideally we'd use @c dwfl_report_elf() instead, but that doesn't take an @c
  * Elf handle, which we need for a couple of reasons:
  *
@@ -85,55 +97,29 @@ DEFINE_HASH_SET_FUNCTIONS(c_string_set, c_string_key_hash_pair, c_string_key_eq)
 static int drgn_dwfl_find_elf(Dwfl_Module *dwfl_module, void **userdatap,
 			      const char *name, Dwarf_Addr base,
 			      char **file_name, Elf **elfp)
-{
-	struct drgn_module *module = *userdatap;
-	/*
-	 * libdwfl consumes the returned path, file descriptor, and ELF handle,
-	 * so clear the fields.
-	 */
-	*file_name = module->path;
-	int fd = module->fd;
-	*elfp = module->elf;
-	module->path = NULL;
-	module->fd = -1;
-	module->elf = NULL;
-	return fd;
-}
-
-/*
- * Uses drgn_dwfl_find_elf() if the ELF file was reported directly and falls
- * back to dwfl_linux_proc_find_elf() otherwise.
- */
-static int drgn_dwfl_linux_proc_find_elf(Dwfl_Module *dwfl_module,
-					 void **userdatap, const char *name,
-					 Dwarf_Addr base, char **file_name,
-					 Elf **elfp)
 {
 	struct drgn_module *module = *userdatap;
 	if (module->elf) {
-		return drgn_dwfl_find_elf(dwfl_module, userdatap, name, base,
-					  file_name, elfp);
+		*file_name = module->path;
+		int fd = module->fd;
+		*elfp = module->elf;
+		// libdwfl consumes the returned path, file descriptor, and ELF
+		// handle, so clear the fields.
+		module->path = NULL;
+		module->fd = -1;
+		module->elf = NULL;
+		return fd;
 	}
-	return dwfl_linux_proc_find_elf(dwfl_module, userdatap, name, base,
-					file_name, elfp);
-}
-
-/*
- * Uses drgn_dwfl_find_elf() if the ELF file was reported directly and falls
- * back to dwfl_build_id_find_elf() otherwise.
- */
-static int drgn_dwfl_build_id_find_elf(Dwfl_Module *dwfl_module,
-				       void **userdatap, const char *name,
-				       Dwarf_Addr base, char **file_name,
-				       Elf **elfp)
-{
-	struct drgn_module *module = *userdatap;
-	if (module->elf) {
-		return drgn_dwfl_find_elf(dwfl_module, userdatap, name, base,
-					  file_name, elfp);
+	if (module->prog->flags & DRGN_PROGRAM_IS_LINUX_KERNEL) {
+		*elfp = NULL;
+		return -1;
+	} else if (module->prog->flags & DRGN_PROGRAM_IS_LIVE) {
+		return dwfl_linux_proc_find_elf(dwfl_module, userdatap, name,
+						base, file_name, elfp);
+	} else {
+		return dwfl_build_id_find_elf(dwfl_module, userdatap, name,
+					      base, file_name, elfp);
 	}
-	return dwfl_build_id_find_elf(dwfl_module, userdatap, name, base,
-				      file_name, elfp);
 }
 
 /**
@@ -159,18 +145,6 @@ static const Dwfl_Callbacks drgn_dwfl_callbacks = {
 	.section_address = drgn_dwfl_section_address,
 };
 
-static const Dwfl_Callbacks drgn_linux_proc_dwfl_callbacks = {
-	.find_elf = drgn_dwfl_linux_proc_find_elf,
-	.find_debuginfo = dwfl_standard_find_debuginfo,
-	.section_address = drgn_dwfl_section_address,
-};
-
-static const Dwfl_Callbacks drgn_userspace_core_dump_dwfl_callbacks = {
-	.find_elf = drgn_dwfl_build_id_find_elf,
-	.find_debuginfo = dwfl_standard_find_debuginfo,
-	.section_address = drgn_dwfl_section_address,
-};
-
 static void drgn_module_destroy(struct drgn_module *module)
 {
 	if (module) {
@@ -181,6 +155,12 @@ static void drgn_module_destroy(struct drgn_module *module)
 		if (module->fd != -1)
 			close(module->fd);
 		free(module->path);
+		for (struct drgn_elf_file_dwarf_table_iterator it =
+		     drgn_elf_file_dwarf_table_first(&module->split_dwarf_files);
+		     it.entry;
+		     it = drgn_elf_file_dwarf_table_next(it))
+			drgn_elf_file_destroy(*it.entry);
+		drgn_elf_file_dwarf_table_deinit(&module->split_dwarf_files);
 		if (module->debug_file != module->loaded_file)
 			drgn_elf_file_destroy(module->debug_file);
 		drgn_elf_file_destroy(module->loaded_file);
@@ -321,11 +301,13 @@ drgn_debug_info_report_error(struct drgn_debug_info_load_state *load,
 	}
 	if (load->num_errors == 0 &&
 	    !string_builder_append(&load->errors,
-				   "could not get debugging information for:"))
+				   "missing some debugging symbols (see https://drgn.readthedocs.io/en/latest/getting_debugging_symbols.html):"))
 		goto err;
 	if (load->num_errors < load->max_errors) {
 		if (!string_builder_line_break(&load->errors))
 			goto err;
+		if (!string_builder_append(&load->errors, "  "))
+			goto err;
 		if (name && !string_builder_append(&load->errors, name))
 			goto err;
 		if (name && (message || err) &&
@@ -439,6 +421,7 @@ drgn_debug_info_report_module(struct drgn_debug_info_load_state *load,
 	module->path = path_key;
 	module->fd = fd;
 	module->elf = elf;
+	drgn_elf_file_dwarf_table_init(&module->split_dwarf_files);
 
 	/* path_key, fd and elf are owned by the module now. */
 
@@ -458,7 +441,7 @@ drgn_debug_info_report_module(struct drgn_debug_info_load_state *load,
 		} else if (drgn_module_table_insert_searched(&dbinfo->modules,
 							     &module, hp,
 							     NULL) < 0) {
-			load->new_modules.size--;
+			drgn_module_vector_pop(&load->new_modules);
 			drgn_module_destroy(module);
 			return &drgn_enomem;
 		}
@@ -585,11 +568,11 @@ struct drgn_mapped_file_segment {
 	uint64_t file_offset;
 };
 
-DEFINE_VECTOR(drgn_mapped_file_segment_vector, struct drgn_mapped_file_segment)
+DEFINE_VECTOR(drgn_mapped_file_segment_vector, struct drgn_mapped_file_segment);
 
 DEFINE_HASH_MAP(drgn_mapped_files, const char *,
 		struct drgn_mapped_file_segment_vector, c_string_key_hash_pair,
-		c_string_key_eq)
+		c_string_key_eq);
 
 struct userspace_core_report_state {
 	struct drgn_mapped_files files;
@@ -710,10 +693,10 @@ userspace_core_get_mapped_files(struct drgn_debug_info_load_state *load,
 		 * mappings with different memory protections even though the
 		 * protection is not included in NT_FILE. Merge them if we can.
 		 */
-		if (it.entry->value.size > 0 &&
-		    drgn_mapped_file_segments_contiguous(&it.entry->value.data[it.entry->value.size - 1],
-							 &segment))
-			it.entry->value.data[it.entry->value.size - 1].end = segment.end;
+		if (!drgn_mapped_file_segment_vector_empty(&it.entry->value)
+		    && drgn_mapped_file_segments_contiguous(drgn_mapped_file_segment_vector_last(&it.entry->value),
+							    &segment))
+			drgn_mapped_file_segment_vector_last(&it.entry->value)->end = segment.end;
 		else if (!drgn_mapped_file_segment_vector_append(&it.entry->value,
 								 &segment))
 			return &drgn_enomem;
@@ -1302,8 +1285,8 @@ userspace_core_report_mapped_files(struct drgn_debug_info_load_state *load,
 	     it.entry; it = drgn_mapped_files_next(it)) {
 		err = userspace_core_maybe_report_file(load, core,
 						       it.entry->key,
-						       it.entry->value.data,
-						       it.entry->value.size);
+						       drgn_mapped_file_segment_vector_begin(&it.entry->value),
+						       drgn_mapped_file_segment_vector_size(&it.entry->value));
 		if (err)
 			return err;
 	}
@@ -1651,7 +1634,8 @@ static struct drgn_error *relocate_elf_file(Elf *elf)
 	size_t shdrnum;
 	if (elf_getshdrnum(elf, &shdrnum))
 		return drgn_error_libelf();
-	uint64_t *sh_addrs = calloc(shdrnum, sizeof(sh_addrs[0]));
+	_cleanup_free_ uint64_t *sh_addrs =
+		calloc(shdrnum, sizeof(sh_addrs[0]));
 	if (!sh_addrs && shdrnum > 0)
 		return &drgn_enomem;
 
@@ -1659,74 +1643,54 @@ static struct drgn_error *relocate_elf_file(Elf *elf)
 	while ((scn = elf_nextscn(elf, scn))) {
 		GElf_Shdr *shdr, shdr_mem;
 		shdr = gelf_getshdr(scn, &shdr_mem);
-		if (!shdr) {
-			err = drgn_error_libelf();
-			goto out;
-		}
+		if (!shdr)
+			return drgn_error_libelf();
 		sh_addrs[elf_ndxscn(scn)] = shdr->sh_addr;
 	}
 
 	size_t shstrndx;
-	if (elf_getshdrstrndx(elf, &shstrndx)) {
-		err = drgn_error_libelf();
-		goto out;
-	}
+	if (elf_getshdrstrndx(elf, &shstrndx))
+		return drgn_error_libelf();
 
 	Elf_Scn *reloc_scn = NULL;
 	while ((reloc_scn = elf_nextscn(elf, reloc_scn))) {
 		GElf_Shdr *reloc_shdr, reloc_shdr_mem;
 		reloc_shdr = gelf_getshdr(reloc_scn, &reloc_shdr_mem);
-		if (!reloc_shdr) {
-			err = drgn_error_libelf();
-			goto out;
-		}
-		/* We don't support any architectures that use SHT_REL yet. */
-		if (reloc_shdr->sh_type != SHT_RELA)
-			continue;
+		if (!reloc_shdr)
+			return drgn_error_libelf();
 
 		int r = should_apply_relocation_section(elf, shstrndx,
 							reloc_shdr);
-		if (r < 0) {
-			err = drgn_error_libelf();
-			goto out;
-		}
+		if (r < 0)
+			return drgn_error_libelf();
 		if (r) {
 			Elf_Scn *scn = elf_getscn(elf, reloc_shdr->sh_info);
-			if (!scn) {
-				err = drgn_error_libelf();
-				goto out;
-			}
+			if (!scn)
+				return drgn_error_libelf();
 			GElf_Shdr *shdr, shdr_mem;
 			shdr = gelf_getshdr(scn, &shdr_mem);
-			if (!shdr) {
-				err = drgn_error_libelf();
-				goto out;
-			}
+			if (!shdr)
+				return drgn_error_libelf();
 			if (shdr->sh_type == SHT_NOBITS)
 				continue;
 
 			Elf_Scn *symtab_scn = elf_getscn(elf,
 							 reloc_shdr->sh_link);
-			if (!symtab_scn) {
-				err = drgn_error_libelf();
-				goto out;
-			}
+			if (!symtab_scn)
+				return drgn_error_libelf();
 			shdr = gelf_getshdr(symtab_scn, &shdr_mem);
-			if (!shdr) {
-				err = drgn_error_libelf();
-				goto out;
-			}
+			if (!shdr)
+				return drgn_error_libelf();
 			if (shdr->sh_type == SHT_NOBITS) {
-				err = drgn_error_create(DRGN_ERROR_OTHER,
-							"relocation symbol table has no data");
-				goto out;
+				return drgn_error_create(DRGN_ERROR_OTHER,
+							 "relocation symbol table has no data");
 			}
 
 			Elf_Data *data, *reloc_data, *symtab_data;
 			if ((err = read_elf_section(scn, &data)) ||
 			    (err = read_elf_section(reloc_scn, &reloc_data)) ||
 			    (err = read_elf_section(symtab_scn, &symtab_data)))
-				goto out;
+				return err;
 
 			struct drgn_relocating_section relocating = {
 				.buf = data->d_buf,
@@ -1745,24 +1709,19 @@ static struct drgn_error *relocate_elf_file(Elf *elf)
 						     shdrnum, &platform);
 			}
 			if (err)
-				goto out;
+				return err;
 
 			/*
 			 * Mark the relocation section as empty so that libdwfl
 			 * doesn't try to apply it again.
 			 */
 			reloc_shdr->sh_size = 0;
-			if (!gelf_update_shdr(reloc_scn, reloc_shdr)) {
-				err = drgn_error_libelf();
-				goto out;
-			}
+			if (!gelf_update_shdr(reloc_scn, reloc_shdr))
+				return drgn_error_libelf();
 			reloc_data->d_size = 0;
 		}
 	}
-	err = NULL;
-out:
-	free(sh_addrs);
-	return err;
+	return NULL;
 }
 
 static struct drgn_error *
@@ -1806,6 +1765,12 @@ drgn_module_find_files(struct drgn_debug_info_load_state *load,
 	const char *debug_file_path;
 	dwfl_module_info(module->dwfl_module, NULL, NULL, NULL, NULL, NULL,
 			 &loaded_file_path, &debug_file_path);
+	// If the loaded file also has debugging information, debug_file_path is
+	// NULL. (debug_file_path is also NULL if libdwfl got the debug file
+	// from debuginfod, so this isn't 100% correct, but it'll at least
+	// identify the module.)
+	if (!debug_file_path)
+		debug_file_path = loaded_file_path;
 
 	module->debug_file_bias = debug_file_bias;
 	err = drgn_elf_file_create(module, debug_file_path, dwarf_getelf(dwarf),
@@ -1930,25 +1895,26 @@ drgn_debug_info_read_module(struct drgn_debug_info_load_state *load,
 static struct drgn_error *
 drgn_debug_info_update_index(struct drgn_debug_info_load_state *load)
 {
-	if (!load->new_modules.size)
+	if (drgn_module_vector_empty(&load->new_modules))
 		return NULL;
 	struct drgn_debug_info *dbinfo = load->dbinfo;
 	if (!c_string_set_reserve(&dbinfo->module_names,
-				  c_string_set_size(&dbinfo->module_names) +
-				  load->new_modules.size))
+				  c_string_set_size(&dbinfo->module_names)
+				  + drgn_module_vector_size(&load->new_modules)))
 		return &drgn_enomem;
 
 	struct drgn_dwarf_index_state index;
 	if (!drgn_dwarf_index_state_init(&index, dbinfo))
 		return &drgn_enomem;
 	struct drgn_error *err = NULL;
-	#pragma omp parallel for schedule(dynamic)
-	for (size_t i = 0; i < load->new_modules.size; i++) {
+	#pragma omp parallel for schedule(dynamic) num_threads(drgn_num_threads)
+	for (size_t i = 0; i < drgn_module_vector_size(&load->new_modules); i++) {
 		if (err)
 			continue;
+		struct drgn_module *module =
+			*drgn_module_vector_at(&load->new_modules, i);
 		struct drgn_error *module_err =
-			drgn_debug_info_read_module(load, &index,
-						    load->new_modules.data[i]);
+			drgn_debug_info_read_module(load, &index, module);
 		if (module_err) {
 			#pragma omp critical(drgn_debug_info_update_index_error)
 			if (err)
@@ -1957,11 +1923,11 @@ drgn_debug_info_update_index(struct drgn_debug_info_load_state *load)
 				err = module_err;
 		}
 	}
-	if (!err)
-		err = drgn_dwarf_info_update_index(&index);
-	drgn_dwarf_index_state_deinit(&index);
-	if (!err)
+	if (!err) {
 		drgn_debug_info_free_modules(dbinfo, true, false);
+		err = drgn_dwarf_info_update_index(&index);
+	}
+	drgn_dwarf_index_state_deinit(&index);
 	return err;
 }
 
@@ -1974,7 +1940,7 @@ drgn_debug_info_report_flush(struct drgn_debug_info_load_state *load)
 	dwfl_report_begin_add(dbinfo->dwfl);
 	if (err)
 		return err;
-	load->new_modules.size = 0;
+	drgn_module_vector_clear(&load->new_modules);
 	return NULL;
 }
 
@@ -1985,7 +1951,7 @@ drgn_debug_info_report_finalize_errors(struct drgn_debug_info_load_state *load)
 	    (!string_builder_line_break(&load->errors) ||
 	     !string_builder_appendf(&load->errors, "... %u more",
 				     load->num_errors - load->max_errors))) {
-		free(load->errors.str);
+		string_builder_deinit(&load->errors);
 		return &drgn_enomem;
 	}
 	if (load->num_errors) {
@@ -2047,10 +2013,6 @@ struct drgn_error *drgn_debug_info_load(struct drgn_debug_info *dbinfo,
 	 * the core dump.
 	 */
 
-	/*
-	 * If this fails, it's too late to roll back. This can only fail with
-	 * enomem, so it's not a big deal.
-	 */
 	err = drgn_debug_info_report_finalize_errors(&load);
 out:
 	drgn_module_vector_deinit(&load.new_modules);
@@ -2058,7 +2020,7 @@ out:
 
 err:
 	drgn_debug_info_free_modules(dbinfo, false, false);
-	free(load.errors.str);
+	string_builder_deinit(&load.errors);
 	goto out;
 }
 
@@ -2068,43 +2030,71 @@ bool drgn_debug_info_is_indexed(struct drgn_debug_info *dbinfo,
 	return c_string_set_search(&dbinfo->module_names, &name).entry != NULL;
 }
 
-struct drgn_error *drgn_debug_info_create(struct drgn_program *prog,
-					  struct drgn_debug_info **ret)
+void drgn_debug_info_init(struct drgn_debug_info *dbinfo,
+			  struct drgn_program *prog)
 {
-	struct drgn_debug_info *dbinfo = malloc(sizeof(*dbinfo));
-	if (!dbinfo)
-		return &drgn_enomem;
 	dbinfo->prog = prog;
-	const Dwfl_Callbacks *dwfl_callbacks;
-	if (prog->flags & DRGN_PROGRAM_IS_LINUX_KERNEL)
-		dwfl_callbacks = &drgn_dwfl_callbacks;
-	else if (prog->flags & DRGN_PROGRAM_IS_LIVE)
-		dwfl_callbacks = &drgn_linux_proc_dwfl_callbacks;
-	else
-		dwfl_callbacks = &drgn_userspace_core_dump_dwfl_callbacks;
-	dbinfo->dwfl = dwfl_begin(dwfl_callbacks);
-	if (!dbinfo->dwfl) {
-		free(dbinfo);
-		return drgn_error_libdwfl();
-	}
+	dbinfo->dwfl = dwfl_begin(&drgn_dwfl_callbacks);
+	// This is temporary until we stop using libdwfl, and is extremely
+	// unlikely to fail anwyays, so don't bother propagating an error up.
+	if (!dbinfo->dwfl)
+		abort();
+	drgn_program_add_type_finder_impl(prog, &dbinfo->type_finder,
+					  drgn_debug_info_find_type, dbinfo);
+	drgn_program_add_object_finder_impl(prog, &dbinfo->object_finder,
+					    drgn_debug_info_find_object,
+					    dbinfo);
 	drgn_module_table_init(&dbinfo->modules);
 	c_string_set_init(&dbinfo->module_names);
 	drgn_dwarf_info_init(dbinfo);
-	*ret = dbinfo;
-	return NULL;
 }
 
-void drgn_debug_info_destroy(struct drgn_debug_info *dbinfo)
+void drgn_debug_info_deinit(struct drgn_debug_info *dbinfo)
 {
-	if (!dbinfo)
-		return;
 	drgn_dwarf_info_deinit(dbinfo);
 	c_string_set_deinit(&dbinfo->module_names);
 	drgn_debug_info_free_modules(dbinfo, false, true);
 	assert(drgn_module_table_empty(&dbinfo->modules));
 	drgn_module_table_deinit(&dbinfo->modules);
 	dwfl_end(dbinfo->dwfl);
-	free(dbinfo);
+}
+
+struct drgn_elf_file *drgn_module_find_dwarf_file(struct drgn_module *module,
+						  Dwarf *dwarf)
+{
+	if (!module->debug_file)
+		return NULL;
+	if (dwarf == module->debug_file->dwarf)
+		return module->debug_file;
+	struct drgn_elf_file_dwarf_table_iterator it =
+		drgn_elf_file_dwarf_table_search(&module->split_dwarf_files,
+						 &dwarf);
+	return it.entry ? *it.entry : NULL;
+}
+
+struct drgn_error *
+drgn_module_create_split_dwarf_file(struct drgn_module *module,
+				    const char *name, Dwarf *dwarf,
+				    struct drgn_elf_file **ret)
+{
+	struct drgn_error *err;
+	err = drgn_elf_file_create(module, name, dwarf_getelf(dwarf), ret);
+	if (err)
+		return err;
+	err = drgn_elf_file_precache_sections(*ret);
+	if (err) {
+		drgn_elf_file_destroy(*ret);
+		return err;
+	}
+	(*ret)->dwarf = dwarf;
+	int r = drgn_elf_file_dwarf_table_insert(&module->split_dwarf_files,
+						 ret, NULL);
+	if (r < 0) {
+		drgn_elf_file_destroy(*ret);
+		return &drgn_enomem;
+	}
+	assert(r > 0);
+	return NULL;
 }
 
 struct drgn_error *
