@@ -20,8 +20,10 @@
 #include "drgn.h"
 #include "dwarf_info.h"
 #include "hash_table.h"
+#include "object_index.h"
 #include "orc_info.h"
 #include "string_builder.h"
+#include "type.h"
 #include "vector.h"
 
 struct drgn_elf_file;
@@ -49,6 +51,8 @@ enum drgn_module_state {
 	DRGN_DEBUG_INFO_MODULE_INDEXED,
 } __attribute__((__packed__));
 
+DEFINE_HASH_TABLE_TYPE(drgn_elf_file_dwarf_table, struct drgn_elf_file *);
+
 /**
  * A module reported to a @ref drgn_debug_info.
  *
@@ -86,6 +90,8 @@ struct drgn_module {
 	 */
 	uint64_t debug_file_bias;
 
+	struct drgn_elf_file_dwarf_table split_dwarf_files;
+
 	/** DWARF debugging information. */
 	struct drgn_module_dwarf_info dwarf;
 	/** ORC unwinder information. */
@@ -120,15 +126,18 @@ struct drgn_module {
 	struct drgn_module *next;
 };
 
-DEFINE_HASH_TABLE_TYPE(drgn_module_table, struct drgn_module *)
+DEFINE_HASH_TABLE_TYPE(drgn_module_table, struct drgn_module *);
 
-DEFINE_HASH_SET_TYPE(c_string_set, const char *)
+DEFINE_HASH_SET_TYPE(c_string_set, const char *);
 
 /** Cache of debugging information. */
 struct drgn_debug_info {
 	/** Program owning this cache. */
 	struct drgn_program *prog;
 
+	struct drgn_type_finder type_finder;
+	struct drgn_object_finder object_finder;
+
 	/** DWARF frontend library handle. */
 	Dwfl *dwfl;
 	/** Modules keyed by build ID and address range. */
@@ -144,14 +153,14 @@ struct drgn_debug_info {
 	struct drgn_dwarf_info dwarf;
 };
 
-/** Create a @ref drgn_debug_info. */
-struct drgn_error *drgn_debug_info_create(struct drgn_program *prog,
-					  struct drgn_debug_info **ret);
+/** Initialize a @ref drgn_debug_info. */
+void drgn_debug_info_init(struct drgn_debug_info *dbinfo,
+			  struct drgn_program *prog);
 
-/** Destroy a @ref drgn_debug_info. */
-void drgn_debug_info_destroy(struct drgn_debug_info *dbinfo);
+/** Deinitialize a @ref drgn_debug_info. */
+void drgn_debug_info_deinit(struct drgn_debug_info *dbinfo);
 
-DEFINE_VECTOR_TYPE(drgn_module_vector, struct drgn_module *)
+DEFINE_VECTOR_TYPE(drgn_module_vector, struct drgn_module *);
 
 /** State tracked while loading debugging information. */
 struct drgn_debug_info_load_state {
@@ -243,8 +252,8 @@ drgn_debug_info_main_language(struct drgn_debug_info *dbinfo,
 			      const struct drgn_language **ret);
 
 /** @ref drgn_type_find_fn() that uses debugging information. */
-struct drgn_error *drgn_debug_info_find_type(enum drgn_type_kind kind,
-					     const char *name, size_t name_len,
+struct drgn_error *drgn_debug_info_find_type(uint64_t kinds, const char *name,
+					     size_t name_len,
 					     const char *filename, void *arg,
 					     struct drgn_qualified_type *ret);
 
@@ -255,6 +264,14 @@ drgn_debug_info_find_object(const char *name, size_t name_len,
 			    enum drgn_find_object_flags flags, void *arg,
 			    struct drgn_object *ret);
 
+struct drgn_elf_file *drgn_module_find_dwarf_file(struct drgn_module *module,
+						  Dwarf *dwarf);
+
+struct drgn_error *
+drgn_module_create_split_dwarf_file(struct drgn_module *module,
+				    const char *name, Dwarf *dwarf,
+				    struct drgn_elf_file **ret);
+
 /**
  * Get the Call Frame Information in a @ref drgn_module at a given program
  * counter.
