@@ -161,12 +161,11 @@ drgn_error_format_fault(uint64_t address, const char *format, ...)
 struct drgn_error *drgn_error_from_string_builder(enum drgn_error_code code,
 						  struct string_builder *sb)
 {
-	char *message = string_builder_null_terminate(sb);
-	if (!message) {
-		free(sb->str);
+	if (!string_builder_null_terminate(sb)) {
+		string_builder_deinit(sb);
 		return &drgn_enomem;
 	}
-	return drgn_error_create_nodup(code, message);
+	return drgn_error_create_nodup(code, sb->str);
 }
 
 LIBDRGN_PUBLIC struct drgn_error *drgn_error_copy(struct drgn_error *src)
@@ -225,6 +224,16 @@ bool string_builder_append_error(struct string_builder *sb,
 #undef emit_error_format
 }
 
+LIBDRGN_PUBLIC char *drgn_error_string(struct drgn_error *err)
+{
+	char *tmp;
+#define emit_error_format(...) (asprintf(&tmp, __VA_ARGS__) < 0 ? NULL : tmp)
+#define emit_error_string(s) strdup(s)
+	return emit_error(err);
+#undef emit_error_string
+#undef emit_error_format
+}
+
 LIBDRGN_PUBLIC int drgn_error_fwrite(FILE *file, struct drgn_error *err)
 {
 #define emit_error_format(format, ...) fprintf(file, format "\n", __VA_ARGS__)
