@@ -536,14 +536,13 @@ static struct drgn_error *
 c_format_type_name(struct drgn_qualified_type qualified_type, char **ret)
 {
 	struct drgn_error *err;
-	struct string_builder sb = STRING_BUILDER_INIT;
+	STRING_BUILDER(sb);
 	err = c_format_type_name_impl(qualified_type, &sb);
-	if (err) {
-		free(sb.str);
+	if (err)
 		return err;
-	}
-	if (!(*ret = string_builder_null_terminate(&sb)))
+	if (!string_builder_null_terminate(&sb))
 		return &drgn_enomem;
+	*ret = string_builder_steal(&sb);
 	return NULL;
 }
 
@@ -551,17 +550,16 @@ static struct drgn_error *
 c_format_type(struct drgn_qualified_type qualified_type, char **ret)
 {
 	struct drgn_error *err;
-	struct string_builder sb = STRING_BUILDER_INIT;
+	STRING_BUILDER(sb);
 	if (drgn_type_is_complete(qualified_type.type))
 		err = c_define_type(qualified_type, 0, &sb);
 	else
 		err = c_format_type_name_impl(qualified_type, &sb);
-	if (err) {
-		free(sb.str);
+	if (err)
 		return err;
-	}
-	if (!(*ret = string_builder_null_terminate(&sb)))
+	if (!string_builder_null_terminate(&sb))
 		return &drgn_enomem;
+	*ret = string_builder_steal(&sb);
 	return NULL;
 }
 
@@ -690,30 +688,70 @@ c_format_int_object(const struct drgn_object *obj,
 		return NULL;
 	}
 
+	union drgn_value value_mem;
+	const union drgn_value *value;
+	err = drgn_object_read_value(obj, &value_mem, &value);
+	if (err)
+		return err;
 	switch (obj->encoding) {
-	case DRGN_OBJECT_ENCODING_SIGNED: {
-		int64_t svalue;
-
-		err = drgn_object_read_signed(obj, &svalue);
-		if (err)
-			return err;
-		if (!string_builder_appendf(sb, "%" PRId64, svalue))
-			return &drgn_enomem;
-		return NULL;
-	}
-	case DRGN_OBJECT_ENCODING_UNSIGNED: {
-		uint64_t uvalue;
-
-		err = drgn_object_read_unsigned(obj, &uvalue);
-		if (err)
-			return err;
-		if (!string_builder_appendf(sb, "%" PRIu64, uvalue))
-			return &drgn_enomem;
-		return NULL;
+	case DRGN_OBJECT_ENCODING_SIGNED:
+		if (!string_builder_appendf(sb, "%" PRId64, value->svalue)) {
+			err = &drgn_enomem;
+			goto out;
+		}
+		break;
+	case DRGN_OBJECT_ENCODING_UNSIGNED:
+		if (!string_builder_appendf(sb, "%" PRIu64, value->uvalue)) {
+			err = &drgn_enomem;
+			goto out;
+		}
+		break;
+	case DRGN_OBJECT_ENCODING_SIGNED_BIG:
+	case DRGN_OBJECT_ENCODING_UNSIGNED_BIG: {
+		if (!string_builder_append(sb, "0x")) {
+			err = &drgn_enomem;
+			goto out;
+		}
+		const uint8_t *buf = (uint8_t *)value->bufp;
+		size_t bytes = drgn_object_size(obj);
+		if (obj->little_endian) {
+			size_t i = bytes - 1;
+			while (i > 0 && buf[i] == 0)
+				i--;
+			if (!string_builder_appendf(sb, "%" PRIx8, buf[i])) {
+				err = &drgn_enomem;
+				goto out;
+			}
+			while (i-- > 0) {
+				if (!string_builder_appendf(sb, "%02" PRIx8, buf[i])) {
+					err = &drgn_enomem;
+					goto out;
+				}
+			}
+		} else {
+			size_t i = 0;
+			while (i < bytes - 1 && buf[i] == 0)
+				i++;
+			if (!string_builder_appendf(sb, "%" PRIx8, buf[i])) {
+				err = &drgn_enomem;
+				goto out;
+			}
+			while (++i < bytes) {
+				if (!string_builder_appendf(sb, "%02" PRIx8, buf[i])) {
+					err = &drgn_enomem;
+					goto out;
+				}
+			}
+		}
+		break;
 	}
 	default:
 		UNREACHABLE();
 	}
+	err = NULL;
+out:
+	drgn_object_deinit_value(obj, value);
+	return err;
 }
 
 static struct drgn_error *
@@ -759,18 +797,15 @@ static struct drgn_error *c_format_initializer(struct drgn_program *prog,
 					       struct string_builder *sb)
 {
 	struct drgn_error *err;
-	struct drgn_object obj;
 	enum drgn_format_object_flags initializer_flags;
 	size_t brace, remaining_columns, start_columns;
 
-	drgn_object_init(&obj, prog);
+	DRGN_OBJECT(obj, prog);
 
 	/* First, try to fit everything on one line. */
 	brace = sb->len;
-	if (!string_builder_appendc(sb, '{')) {
-		err = &drgn_enomem;
-		goto out;
-	}
+	if (!string_builder_appendc(sb, '{'))
+		return &drgn_enomem;
 	if (__builtin_sub_overflow(one_line_columns, 1, &remaining_columns))
 		remaining_columns = 0;
 	for (;;) {
@@ -780,7 +815,7 @@ static struct drgn_error *c_format_initializer(struct drgn_program *prog,
 		if (err == &drgn_stop)
 			break;
 		else if (err)
-			goto out;
+			return err;
 
 		if (!same_line) {
 			err = &drgn_line_wrap;
@@ -794,10 +829,8 @@ static struct drgn_error *c_format_initializer(struct drgn_program *prog,
 				err = &drgn_line_wrap;
 				break;
 			}
-			if (!string_builder_appendc(sb, ' ')) {
-				err = &drgn_enomem;
-				goto out;
-			}
+			if (!string_builder_appendc(sb, ' '))
+				return &drgn_enomem;
 			remaining_columns--;
 		} else {
 			if (remaining_columns < 4) {
@@ -808,10 +841,8 @@ static struct drgn_error *c_format_initializer(struct drgn_program *prog,
 				err = &drgn_line_wrap;
 				break;
 			}
-			if (!string_builder_append(sb, ", ")) {
-				err = &drgn_enomem;
-				goto out;
-			}
+			if (!string_builder_append(sb, ", "))
+				return &drgn_enomem;
 			remaining_columns -= 2;
 		}
 
@@ -820,7 +851,7 @@ static struct drgn_error *c_format_initializer(struct drgn_program *prog,
 
 			err = iter->append_designation(iter, sb);
 			if (err)
-				goto out;
+				return err;
 			if (__builtin_sub_overflow(remaining_columns,
 						   sb->len - designation_start,
 						   &remaining_columns)) {
@@ -836,7 +867,7 @@ static struct drgn_error *c_format_initializer(struct drgn_program *prog,
 		if (err == &drgn_line_wrap)
 			break;
 		else if (err)
-			goto out;
+			return err;
 
 		if (__builtin_sub_overflow(remaining_columns,
 					   sb->len - initializer_start,
@@ -850,16 +881,14 @@ static struct drgn_error *c_format_initializer(struct drgn_program *prog,
 		if (sb->len == brace + 1) {
 			/* There were no initializers. */
 			if (string_builder_appendc(sb, '}'))
-				err = NULL;
+				return NULL;
 			else
-				err = &drgn_enomem;
-			goto out;
+				return &drgn_enomem;
 		} else if (remaining_columns >= 2) {
 			if (string_builder_append(sb, " }"))
-				err = NULL;
+				return NULL;
 			else
-				err = &drgn_enomem;
-			goto out;
+				return &drgn_enomem;
 		}
 		/* The final space and closing brace didn't fit. */
 	}
@@ -868,8 +897,7 @@ static struct drgn_error *c_format_initializer(struct drgn_program *prog,
 
 	if (multi_line_columns == 0) {
 		/* We were asked to stay on one line. */
-		err = &drgn_line_wrap;
-		goto out;
+		return &drgn_line_wrap;
 	}
 
 	sb->len = brace + 1;
@@ -885,21 +913,19 @@ static struct drgn_error *c_format_initializer(struct drgn_program *prog,
 		if (err == &drgn_stop)
 			break;
 		else if (err)
-			goto out;
+			return err;
 
 		newline = sb->len;
 		if (!string_builder_appendc(sb, '\n') ||
-		    !append_tabs(indent + 1, sb)) {
-			err = &drgn_enomem;
-			goto out;
-		}
+		    !append_tabs(indent + 1, sb))
+			return &drgn_enomem;
 
 		designation_start = sb->len;
 		line_columns = start_columns;
 		if (iter->append_designation) {
 			err = iter->append_designation(iter, sb);
 			if (err)
-				goto out;
+				return err;
 			if (__builtin_sub_overflow(line_columns,
 						   sb->len - designation_start,
 						   &line_columns))
@@ -924,25 +950,21 @@ static struct drgn_error *c_format_initializer(struct drgn_program *prog,
 						&sb->str[designation_start],
 						len);
 					sb->len = newline + 1 + len;
-					if (!string_builder_appendc(sb, ',')) {
-						err = &drgn_enomem;
-						goto out;
-					}
+					if (!string_builder_appendc(sb, ','))
+						return &drgn_enomem;
 					remaining_columns -= len + 2;
 					continue;
 				}
 				if (len < start_columns) {
 					/* It fit on the new line. */
-					if (!string_builder_appendc(sb, ',')) {
-						err = &drgn_enomem;
-						goto out;
-					}
+					if (!string_builder_appendc(sb, ','))
+						return &drgn_enomem;
 					remaining_columns =
 						start_columns - len - 1;
 					continue;
 				}
 			} else if (err != &drgn_line_wrap) {
-				goto out;
+				return err;
 			}
 			/* It didn't fit. */
 			sb->len = initializer_start;
@@ -952,23 +974,16 @@ static struct drgn_error *c_format_initializer(struct drgn_program *prog,
 					   multi_line_columns,
 					   initializer_flags, sb);
 		if (err)
-			goto out;
-		if (!string_builder_appendc(sb, ',')) {
-			err = &drgn_enomem;
-			goto out;
-		}
+			return err;
+		if (!string_builder_appendc(sb, ','))
+			return &drgn_enomem;
 		remaining_columns = 0;
 	}
 
 	if (!string_builder_appendc(sb, '\n') || !append_tabs(indent, sb) ||
-	    !string_builder_appendc(sb, '}')) {
-		err = &drgn_enomem;
-		goto out;
-	}
-	err = NULL;
-out:
-	drgn_object_deinit(&obj);
-	return err;
+	    !string_builder_appendc(sb, '}'))
+		return &drgn_enomem;
+	return NULL;
 }
 
 struct compound_initializer_state {
@@ -976,7 +991,7 @@ struct compound_initializer_state {
 	uint64_t bit_offset;
 };
 
-DEFINE_VECTOR(compound_initializer_stack, struct compound_initializer_state)
+DEFINE_VECTOR(compound_initializer_stack, struct compound_initializer_state);
 
 struct compound_initializer_iter {
 	struct initializer_iter iter;
@@ -995,13 +1010,13 @@ compound_initializer_iter_next(struct initializer_iter *iter_,
 		container_of(iter_, struct compound_initializer_iter, iter);
 
 	for (;;) {
-		if (!iter->stack.size)
+		if (compound_initializer_stack_empty(&iter->stack))
 			return &drgn_stop;
 
 		struct compound_initializer_state *top =
-			&iter->stack.data[iter->stack.size - 1];
+			compound_initializer_stack_last(&iter->stack);
 		if (top->member == top->end) {
-			iter->stack.size--;
+			compound_initializer_stack_pop(&iter->stack);
 			continue;
 		}
 
@@ -1061,11 +1076,9 @@ static void compound_initializer_iter_reset(struct initializer_iter *iter_)
 {
 	struct compound_initializer_iter *iter =
 		container_of(iter_, struct compound_initializer_iter, iter);
-	struct drgn_type *underlying_type =
-		drgn_underlying_type(iter->obj->type);
-
-	iter->stack.size = 1;
-	iter->stack.data[0].member = drgn_type_members(underlying_type);
+	compound_initializer_stack_resize(&iter->stack, 1);
+	compound_initializer_stack_first(&iter->stack)->member =
+		drgn_type_members(drgn_underlying_type(iter->obj->type));
 }
 
 static struct drgn_error *
@@ -1075,7 +1088,7 @@ compound_initializer_append_designation(struct initializer_iter *iter_,
 	struct compound_initializer_iter *iter =
 		container_of(iter_, struct compound_initializer_iter, iter);
 	struct compound_initializer_state *top =
-		&iter->stack.data[iter->stack.size - 1];
+		compound_initializer_stack_last(&iter->stack);
 	const char *name = top->member[-1].name;
 
 	if (name && !string_builder_appendf(sb, ".%s = ", name))
@@ -1145,34 +1158,30 @@ c_format_compound_object(const struct drgn_object *obj,
 	if (!(flags & (DRGN_FORMAT_OBJECT_MEMBER_NAMES |
 		       DRGN_FORMAT_OBJECT_IMPLICIT_MEMBERS)) &&
 	    new->member < new->end) {
-		struct drgn_object member;
-		drgn_object_init(&member, drgn_object_program(obj));
+		DRGN_OBJECT(member, drgn_object_program(obj));
 		do {
 			struct drgn_qualified_type member_type;
 			uint64_t member_bit_field_size;
 			err = drgn_member_type(&new->end[-1], &member_type,
 					       &member_bit_field_size);
 			if (err)
-				break;
+				goto out;
 
 			err = drgn_object_slice(&member, obj, member_type,
 						new->end[-1].bit_offset,
 						member_bit_field_size);
 			if (err)
-				break;
+				goto out;
 
 			bool zero;
 			err = drgn_object_is_zero(&member, &zero);
 			if (err)
-				break;
+				goto out;
 			if (zero)
 				new->end--;
 			else
 				break;
 		} while (new->member < new->end);
-		drgn_object_deinit(&member);
-		if (err)
-			goto out;
 	}
 
 	err = c_format_initializer(drgn_object_program(obj), &iter.iter, indent,
@@ -1307,12 +1316,9 @@ c_format_pointer_object(const struct drgn_object *obj,
 		err = c_format_string(drgn_object_program(obj), uvalue,
 				      UINT64_MAX, sb);
 	} else {
-		struct drgn_object dereferenced;
-
-		drgn_object_init(&dereferenced, drgn_object_program(obj));
+		DRGN_OBJECT(dereferenced, drgn_object_program(obj));
 		err = drgn_object_dereference(&dereferenced, obj);
 		if (err) {
-			drgn_object_deinit(&dereferenced);
 			if (err->code == DRGN_ERROR_TYPE)
 				goto no_dereference;
 			return err;
@@ -1323,7 +1329,6 @@ c_format_pointer_object(const struct drgn_object *obj,
 		err = c_format_object_impl(&dereferenced, indent,
 					   one_line_columns, multi_line_columns,
 					   passthrough_flags, sb);
-		drgn_object_deinit(&dereferenced);
 	}
 	if (!err || (err->code != DRGN_ERROR_FAULT && err->code != DRGN_ERROR_OUT_OF_BOUNDS)) {
 		/* We either succeeded or hit a fatal error. */
@@ -1479,9 +1484,7 @@ c_format_array_object(const struct drgn_object *obj,
 	if (!(flags & (DRGN_FORMAT_OBJECT_ELEMENT_INDICES |
 		       DRGN_FORMAT_OBJECT_IMPLICIT_ELEMENTS)) &&
 	    iter.length) {
-		struct drgn_object element;
-
-		drgn_object_init(&element, drgn_object_program(obj));
+		DRGN_OBJECT(element, drgn_object_program(obj));
 		do {
 			bool zero;
 
@@ -1491,19 +1494,16 @@ c_format_array_object(const struct drgn_object *obj,
 						iter.element_bit_size,
 						0);
 			if (err)
-				break;
+				return err;
 
 			err = drgn_object_is_zero(&element, &zero);
 			if (err)
-				break;
+				return err;
 			if (zero)
 				iter.length--;
 			else
 				break;
 		} while (iter.length);
-		drgn_object_deinit(&element);
-		if (err)
-			return err;
 	}
 	return c_format_initializer(drgn_object_program(obj), &iter.iter,
 				    indent, one_line_columns,
@@ -1602,15 +1602,14 @@ static struct drgn_error *c_format_object(const struct drgn_object *obj,
 					  char **ret)
 {
 	struct drgn_error *err;
-	struct string_builder sb = STRING_BUILDER_INIT;
+	STRING_BUILDER(sb);
 	err = c_format_object_impl(obj, 0, columns, max(columns, (size_t)1),
 				   flags, &sb);
-	if (err) {
-		free(sb.str);
+	if (err)
 		return err;
-	}
-	if (!(*ret = string_builder_null_terminate(&sb)))
+	if (!string_builder_null_terminate(&sb))
 		return &drgn_enomem;
+	*ret = string_builder_steal(&sb);
 	return NULL;
 }
 
@@ -2235,16 +2234,15 @@ c_parse_specifier_qualifier_list(struct drgn_program *prog,
 	}
 
 	if (specifier == SPECIFIER_NONE) {
-		enum drgn_type_kind kind;
-
+		uint64_t kinds;
 		if (tag_token == C_TOKEN_STRUCT) {
-			kind = DRGN_TYPE_STRUCT;
+			kinds = 1 << DRGN_TYPE_STRUCT;
 		} else if (tag_token == C_TOKEN_UNION) {
-			kind = DRGN_TYPE_UNION;
+			kinds = 1 << DRGN_TYPE_UNION;
 		} else if (tag_token == C_TOKEN_CLASS) {
-			kind = DRGN_TYPE_CLASS;
+			kinds = 1 << DRGN_TYPE_CLASS;
 		} else if (tag_token == C_TOKEN_ENUM) {
-			kind = DRGN_TYPE_ENUM;
+			kinds = 1 << DRGN_TYPE_ENUM;
 		} else if (identifier) {
 			if (identifier_len == sizeof("size_t") - 1 &&
 			    memcmp(identifier, "size_t",
@@ -2266,15 +2264,21 @@ c_parse_specifier_qualifier_list(struct drgn_program *prog,
 					return err;
 				ret->qualifiers = 0;
 				goto out;
+			} else if (((struct drgn_c_family_lexer *)lexer)->cpp) {
+				kinds = ((1 << DRGN_TYPE_STRUCT)
+					 | (1 << DRGN_TYPE_UNION)
+					 | (1 << DRGN_TYPE_CLASS)
+					 | (1 << DRGN_TYPE_ENUM)
+					 | (1 << DRGN_TYPE_TYPEDEF));
 			} else {
-				kind = DRGN_TYPE_TYPEDEF;
+				kinds = 1 << DRGN_TYPE_TYPEDEF;
 			}
 		} else {
 			return drgn_error_create(DRGN_ERROR_SYNTAX,
 						 "expected type specifier");
 		}
 
-		err = drgn_program_find_type_impl(prog, kind, identifier,
+		err = drgn_program_find_type_impl(prog, kinds, identifier,
 						  identifier_len, filename,
 						  ret);
 		if (err)
