@@ -18,6 +18,7 @@
 #include <unistd.h>
 
 #include "binary_buffer.h"
+#include "cleanup.h"
 #include "debug_info.h"
 #include "drgn.h"
 #include "error.h"
@@ -107,7 +108,6 @@ struct drgn_error *read_vmcoreinfo_fallback(struct drgn_program *prog)
 	FILE *file;
 	uint64_t address;
 	size_t size;
-	char *buf;
 	Elf64_Nhdr *nhdr;
 
 	file = fopen("/sys/kernel/vmcoreinfo", "r");
@@ -122,13 +122,13 @@ struct drgn_error *read_vmcoreinfo_fallback(struct drgn_program *prog)
 	}
 	fclose(file);
 
-	buf = malloc(size);
+	_cleanup_free_ char *buf = malloc(size);
 	if (!buf)
 		return &drgn_enomem;
 
 	err = drgn_program_read_memory(prog, buf, address, size, true);
 	if (err)
-		goto out;
+		return err;
 
 	/*
 	 * The first 12 bytes are the Elf{32,64}_Nhdr (it's the same in both
@@ -141,13 +141,10 @@ struct drgn_error *read_vmcoreinfo_fallback(struct drgn_program *prog)
 	    nhdr->n_descsz > size - 24) {
 		err = drgn_error_create(DRGN_ERROR_OTHER,
 					"VMCOREINFO is invalid");
-		goto out;
+		return err;
 	}
 
-	err = drgn_program_parse_vmcoreinfo(prog, buf + 24, nhdr->n_descsz);
-out:
-	free(buf);
-	return err;
+	return drgn_program_parse_vmcoreinfo(prog, buf + 24, nhdr->n_descsz);
 }
 
 static struct drgn_error *linux_kernel_get_page_shift(struct drgn_program *prog,
@@ -220,8 +217,7 @@ static struct drgn_error *linux_kernel_get_jiffies(struct drgn_program *prog,
 						   struct drgn_object *ret)
 {
 	struct drgn_error *err;
-	struct drgn_object jiffies_64;
-	drgn_object_init(&jiffies_64, prog);
+	DRGN_OBJECT(jiffies_64, prog);
 	err = drgn_program_find_object(prog, "jiffies_64", NULL,
 				       DRGN_FIND_OBJECT_VARIABLE, &jiffies_64);
 	if (err) {
@@ -229,12 +225,10 @@ static struct drgn_error *linux_kernel_get_jiffies(struct drgn_program *prog,
 			drgn_error_destroy(err);
 			err = &drgn_not_found;
 		}
-		goto out;
-	}
-	if (jiffies_64.kind != DRGN_OBJECT_REFERENCE) {
-		err = &drgn_not_found;
-		goto out;
+		return err;
 	}
+	if (jiffies_64.kind != DRGN_OBJECT_REFERENCE)
+		return &drgn_not_found;
 	uint64_t address = jiffies_64.address;
 	struct drgn_qualified_type qualified_type;
 	err = drgn_program_find_primitive_type(prog, DRGN_C_TYPE_UNSIGNED_LONG,
@@ -245,10 +239,27 @@ static struct drgn_error *linux_kernel_get_jiffies(struct drgn_program *prog,
 	if (drgn_type_size(qualified_type.type) == 4 &&
 	    !drgn_type_little_endian(qualified_type.type))
 		address += 4;
-	err = drgn_object_set_reference(ret, qualified_type, address, 0, 0);
-out:
-	drgn_object_deinit(&jiffies_64);
-	return err;
+	return drgn_object_set_reference(ret, qualified_type, address, 0, 0);
+}
+
+static struct drgn_error *
+linux_kernel_get_vmcoreinfo(struct drgn_program *prog, struct drgn_object *ret)
+{
+	struct drgn_error *err;
+	struct drgn_qualified_type qualified_type;
+	err = drgn_program_find_primitive_type(prog,
+					       DRGN_C_TYPE_CHAR,
+					       &qualified_type.type);
+	if (err)
+		return err;
+	qualified_type.qualifiers = DRGN_QUALIFIER_CONST;
+	err = drgn_array_type_create(prog, qualified_type, prog->vmcoreinfo.raw_size,
+				     &drgn_language_c, &qualified_type.type);
+	if (err)
+		return err;
+	qualified_type.qualifiers = 0;
+	return drgn_object_set_from_buffer(ret, qualified_type, prog->vmcoreinfo.raw,
+					   prog->vmcoreinfo.raw_size, 0, 0);
 }
 
 // The vmemmap address can vary depending on architecture, kernel version,
@@ -261,10 +272,9 @@ linux_kernel_get_vmemmap_address(struct drgn_program *prog, uint64_t *ret)
 	static const uint64_t SECTION_MAP_MASK = ~((UINT64_C(1) << 6) - 1);
 	struct drgn_error *err;
 
-	struct drgn_object mem_section, root, section;
-	drgn_object_init(&mem_section, prog);
-	drgn_object_init(&root, prog);
-	drgn_object_init(&section, prog);
+	DRGN_OBJECT(mem_section, prog);
+	DRGN_OBJECT(root, prog);
+	DRGN_OBJECT(section, prog);
 
 	err = drgn_program_find_object(prog, "vmemmap_populate", NULL,
 				       DRGN_FIND_OBJECT_FUNCTION, &mem_section);
@@ -274,13 +284,13 @@ linux_kernel_get_vmemmap_address(struct drgn_program *prog, uint64_t *ret)
 			drgn_error_destroy(err);
 			err = &drgn_not_found;
 		}
-		goto out;
+		return err;
 	}
 
 	err = drgn_program_find_object(prog, "mem_section", NULL,
 				       DRGN_FIND_OBJECT_VARIABLE, &mem_section);
 	if (err)
-		goto out;
+		return err;
 
 	const uint64_t nr_section_roots = prog->vmcoreinfo.mem_section_length;
 	uint64_t sections_per_root;
@@ -297,9 +307,8 @@ linux_kernel_get_vmemmap_address(struct drgn_program *prog, uint64_t *ret)
 		for (int i = 0; i < 2; i++) {
 			if (drgn_type_kind(mem_section_type) != DRGN_TYPE_POINTER) {
 unrecognized_mem_section_type:
-				err = drgn_type_error("mem_section has unrecognized type '%s'",
-						      mem_section.type);
-				goto out;
+				return drgn_type_error("mem_section has unrecognized type '%s'",
+						       mem_section.type);
 			}
 			mem_section_type = drgn_type_type(mem_section_type).type;
 		}
@@ -316,41 +325,34 @@ unrecognized_mem_section_type:
 	for (uint64_t i = 0; i < nr_section_roots; i++) {
 		err = drgn_object_subscript(&root, &mem_section, i);
 		if (err)
-			goto out;
+			return err;
 		bool truthy;
 		err = drgn_object_bool(&root, &truthy);
 		if (err)
-			goto out;
+			return err;
 		if (!truthy)
 			continue;
 
 		for (uint64_t j = 0; j < sections_per_root; j++) {
 			err = drgn_object_subscript(&section, &root, j);
 			if (err)
-				goto out;
+				return err;
 			err = drgn_object_member(&section, &section,
 						 "section_mem_map");
 			if (err)
-				goto out;
+				return err;
 			uint64_t section_mem_map;
 			err = drgn_object_read_unsigned(&section,
 							&section_mem_map);
 			if (err)
-				goto out;
+				return err;
 			if (section_mem_map & SECTION_HAS_MEM_MAP) {
 				*ret = section_mem_map & SECTION_MAP_MASK;
-				err = NULL;
-				goto out;
+				return NULL;
 			}
 		}
 	}
-	err = &drgn_not_found;
-
-out:
-	drgn_object_deinit(&section);
-	drgn_object_deinit(&root);
-	drgn_object_deinit(&mem_section);
-	return err;
+	return &drgn_not_found;
 }
 
 static struct drgn_error *linux_kernel_get_vmemmap(struct drgn_program *prog,
@@ -395,6 +397,7 @@ struct kernel_module_iterator {
 	/* Address of `struct list_head modules`. */
 	uint64_t head;
 	bool use_sys_module;
+	bool use_sys_module_sections;
 };
 
 static void kernel_module_iterator_deinit(struct kernel_module_iterator *it)
@@ -418,6 +421,7 @@ kernel_module_iterator_init(struct kernel_module_iterator *it,
 	it->build_id_buf = NULL;
 	it->build_id_buf_capacity = 0;
 	it->use_sys_module = use_sys_module;
+	it->use_sys_module_sections = use_sys_module;
 	err = drgn_program_find_type(prog, "struct module", NULL,
 				     &it->module_type);
 	if (err)
@@ -719,81 +723,70 @@ kernel_module_iterator_gnu_build_id(struct kernel_module_iterator *it,
 	struct drgn_program *prog = drgn_object_program(&it->mod);
 	const bool bswap = drgn_platform_bswap(&prog->platform);
 
-	struct drgn_object attrs, attr, tmp;
-	drgn_object_init(&attrs, prog);
-	drgn_object_init(&attr, prog);
-	drgn_object_init(&tmp, prog);
+	DRGN_OBJECT(attrs, prog);
+	DRGN_OBJECT(attr, prog);
+	DRGN_OBJECT(tmp, prog);
 
 	// n = mod->notes_attrs->notes
 	uint64_t n;
 	err = drgn_object_member(&attrs, &it->mod, "notes_attrs");
 	if (err)
-		goto out;
+		return err;
 	err = drgn_object_member_dereference(&tmp, &attrs, "notes");
 	if (err)
-		goto out;
+		return err;
 	err = drgn_object_read_unsigned(&tmp, &n);
 	if (err)
-		goto out;
+		return err;
 
 	// attrs = mod->notes_attrs->attrs
 	err = drgn_object_member_dereference(&attrs, &attrs, "attrs");
 	if (err)
-		goto out;
+		return err;
 
 	for (uint64_t i = 0; i < n; i++) {
 		// attr = attrs[i]
 		err = drgn_object_subscript(&attr, &attrs, i);
 		if (err)
-			goto out;
+			return err;
 
 		// address = attr.private
 		err = drgn_object_member(&tmp, &attr, "private");
 		if (err)
-			goto out;
+			return err;
 		uint64_t address;
 		err = drgn_object_read_unsigned(&tmp, &address);
 		if (err)
-			goto out;
+			return err;
 
 		// size = attr.size
 		err = drgn_object_member(&tmp, &attr, "size");
 		if (err)
-			goto out;
+			return err;
 		uint64_t size;
 		err = drgn_object_read_unsigned(&tmp, &size);
 		if (err)
-			goto out;
+			return err;
 
 		if (size > SIZE_MAX ||
 		    !alloc_or_reuse(&it->build_id_buf,
-				    &it->build_id_buf_capacity, size)) {
-			err = &drgn_enomem;
-			goto out;
-		}
+				    &it->build_id_buf_capacity, size))
+			return &drgn_enomem;
 
 		err = drgn_program_read_memory(prog, it->build_id_buf, address,
 					       size, false);
 		if (err)
-			goto out;
+			return err;
 
 		*build_id_len_ret =
 			parse_gnu_build_id_from_note(it->build_id_buf, size,
 						     bswap, build_id_ret);
-		if (*build_id_len_ret) {
-			err = NULL;
-			goto out;
-		}
+		if (*build_id_len_ret)
+			return NULL;
 	}
 	*build_id_ret = NULL;
 	*build_id_len_ret = 0;
-	err = NULL;
-
-out:
-	drgn_object_deinit(&tmp);
-	drgn_object_deinit(&attr);
-	drgn_object_deinit(&attrs);
-	return err;
+	return NULL;
 }
 
 struct kernel_module_section_iterator {
@@ -808,14 +801,37 @@ struct kernel_module_section_iterator {
 };
 
 static struct drgn_error *
-kernel_module_section_iterator_init(struct kernel_module_section_iterator *it,
-				    struct kernel_module_iterator *kmod_it)
+kernel_module_section_iterator_init_no_sys_module(struct kernel_module_section_iterator *it,
+						  struct kernel_module_iterator *kmod_it)
 {
 	struct drgn_error *err;
 
+	it->sections_dir = NULL;
+	it->i = 0;
+	it->name = NULL;
+	/* it->nsections = mod->sect_attrs->nsections */
+	err = drgn_object_member(&kmod_it->tmp1, &kmod_it->mod, "sect_attrs");
+	if (err)
+		return err;
+	err = drgn_object_member_dereference(&kmod_it->tmp2, &kmod_it->tmp1,
+					     "nsections");
+	if (err)
+		return err;
+	err = drgn_object_read_unsigned(&kmod_it->tmp2, &it->nsections);
+	if (err)
+		return err;
+	/* kmod_it->tmp1 = mod->sect_attrs->attrs */
+	return drgn_object_member_dereference(&kmod_it->tmp1, &kmod_it->tmp1,
+					      "attrs");
+}
+
+static struct drgn_error *
+kernel_module_section_iterator_init(struct kernel_module_section_iterator *it,
+				    struct kernel_module_iterator *kmod_it)
+{
 	it->kmod_it = kmod_it;
 	it->yielded_percpu = false;
-	if (kmod_it->use_sys_module) {
+	if (kmod_it->use_sys_module_sections) {
 		char *path;
 		if (asprintf(&path, "/sys/module/%s/sections",
 			     kmod_it->name) == -1)
@@ -829,26 +845,7 @@ kernel_module_section_iterator_init(struct kernel_module_section_iterator *it,
 		}
 		return NULL;
 	} else {
-		it->sections_dir = NULL;
-		it->i = 0;
-		it->name = NULL;
-		/* it->nsections = mod->sect_attrs->nsections */
-		err = drgn_object_member(&kmod_it->tmp1, &kmod_it->mod,
-					 "sect_attrs");
-		if (err)
-			return err;
-		err = drgn_object_member_dereference(&kmod_it->tmp2,
-						     &kmod_it->tmp1,
-						     "nsections");
-		if (err)
-			return err;
-		err = drgn_object_read_unsigned(&kmod_it->tmp2,
-						&it->nsections);
-		if (err)
-			return err;
-		/* kmod_it->tmp1 = mod->sect_attrs->attrs */
-		return drgn_object_member_dereference(&kmod_it->tmp1,
-						      &kmod_it->tmp1, "attrs");
+		return kernel_module_section_iterator_init_no_sys_module(it, kmod_it);
 	}
 }
 
@@ -954,8 +951,18 @@ kernel_module_section_iterator_next(struct kernel_module_section_iterator *it,
 	}
 
 	if (it->sections_dir) {
-		return kernel_module_section_iterator_next_live(it, name_ret,
-								address_ret);
+		err = kernel_module_section_iterator_next_live(it, name_ret,
+							       address_ret);
+		if (err && err->code == DRGN_ERROR_OS && err->errnum == EACCES) {
+			closedir(it->sections_dir);
+			drgn_error_destroy(err);
+			it->kmod_it->use_sys_module_sections = false;
+			err = kernel_module_section_iterator_init_no_sys_module(it, it->kmod_it);
+			if (err)
+				return err;
+		} else {
+			return err;
+		}
 	}
 
 	if (it->i >= it->nsections)
@@ -1252,7 +1259,7 @@ static struct drgn_error *identify_kernel_elf(Elf *elf,
 }
 
 DEFINE_HASH_MAP(elf_scn_name_map, const char *, Elf_Scn *,
-		c_string_key_hash_pair, c_string_key_eq)
+		c_string_key_hash_pair, c_string_key_eq);
 
 static struct drgn_error *
 cache_kernel_module_sections(struct kernel_module_iterator *kmod_it, Elf *elf)
@@ -1351,7 +1358,7 @@ kernel_module_table_key(struct kernel_module_file * const *entry)
 }
 
 DEFINE_HASH_TABLE(kernel_module_table, struct kernel_module_file *,
-		  kernel_module_table_key, nstring_hash_pair, nstring_eq)
+		  kernel_module_table_key, nstring_hash_pair, nstring_eq);
 
 static struct drgn_error *
 report_loaded_kernel_module(struct drgn_debug_info_load_state *load,
@@ -1554,10 +1561,12 @@ report_kernel_modules(struct drgn_debug_info_load_state *load,
 	 * If we're debugging the running kernel, we can use
 	 * /sys/module/$module/notes and /sys/module/$module/sections instead of
 	 * getting the equivalent information from the core dump. This fast path
-	 * can be disabled via an environment variable for testing.
+	 * can be disabled via an environment variable for testing. It may also
+	 * be disabled if we encounter permission issues using
+	 * /sys/module/$module/sections.
 	 */
 	bool use_sys_module = false;
-	if (prog->flags & DRGN_PROGRAM_IS_LIVE) {
+	if (prog->flags & DRGN_PROGRAM_IS_LOCAL) {
 		char *env = getenv("DRGN_USE_SYS_MODULE");
 		use_sys_module = !env || atoi(env);
 	}
