@@ -5,6 +5,7 @@
 #include <stdio.h>
 
 #include "drgn.h"
+#include "error.h"
 #include "helpers.h"
 #include "minmax.h"
 #include "platform.h"
@@ -226,66 +227,73 @@ struct drgn_error *linux_helper_per_cpu_ptr(struct drgn_object *res,
 	struct drgn_error *err;
 	struct drgn_program *prog = drgn_object_program(ptr);
 
-	struct drgn_object tmp;
-	drgn_object_init(&tmp, prog);
+	DRGN_OBJECT(tmp, prog);
 	err = drgn_program_find_object(prog, "__per_cpu_offset", NULL,
 				       DRGN_FIND_OBJECT_ANY, &tmp);
 	if (!err) {
 		err = drgn_object_subscript(&tmp, &tmp, cpu);
 		if (err)
-			goto out;
+			return err;
 		union drgn_value per_cpu_offset;
 		err = drgn_object_read_integer(&tmp, &per_cpu_offset);
 		if (err)
-			goto out;
+			return err;
 
 		uint64_t ptr_value;
 		err = drgn_object_read_unsigned(ptr, &ptr_value);
 		if (err)
-			goto out;
+			return err;
 
-		err = drgn_object_set_unsigned(res,
-					       drgn_object_qualified_type(ptr),
-					       ptr_value + per_cpu_offset.uvalue,
-					       0);
+		return drgn_object_set_unsigned(res,
+						drgn_object_qualified_type(ptr),
+						ptr_value + per_cpu_offset.uvalue,
+						0);
 	} else if (err->code == DRGN_ERROR_LOOKUP) {
 		drgn_error_destroy(err);
-		err = drgn_object_copy(res, ptr);
+		return drgn_object_copy(res, ptr);
+	} else {
+		return err;
 	}
-out:
-	drgn_object_deinit(&tmp);
-	return err;
 }
 
-struct drgn_error *linux_helper_idle_task(struct drgn_object *res, uint64_t cpu)
+static struct drgn_error *cpu_rq_member(struct drgn_object *res, uint64_t cpu,
+					const char *member_name)
 {
 	struct drgn_error *err;
 	struct drgn_program *prog = drgn_object_program(res);
 
-	struct drgn_object tmp;
-	drgn_object_init(&tmp, prog);
+	DRGN_OBJECT(tmp, prog);
 	err = drgn_program_find_object(prog, "runqueues", NULL,
 				       DRGN_FIND_OBJECT_ANY, &tmp);
 	if (err)
-		goto out;
+		return err;
 	err = drgn_object_address_of(&tmp, &tmp);
 	if (err)
-		goto out;
+		return err;
 	err = linux_helper_per_cpu_ptr(&tmp, &tmp, cpu);
 	if (err)
-		goto out;
-	err = drgn_object_member_dereference(res, &tmp, "idle");
-out:
-	drgn_object_deinit(&tmp);
-	return err;
+		return err;
+	err = drgn_object_member_dereference(&tmp, &tmp, member_name);
+	if (err)
+		return err;
+	return drgn_object_read(res, &tmp);
+}
+
+struct drgn_error *linux_helper_cpu_curr(struct drgn_object *res, uint64_t cpu)
+{
+	return cpu_rq_member(res, cpu, "curr");
+}
+
+struct drgn_error *linux_helper_idle_task(struct drgn_object *res, uint64_t cpu)
+{
+	return cpu_rq_member(res, cpu, "idle");
 }
 
 struct drgn_error *linux_helper_task_cpu(const struct drgn_object *task,
 					 uint64_t *ret)
 {
 	struct drgn_error *err;
-	struct drgn_object tmp;
-	drgn_object_init(&tmp, drgn_object_program(task));
+	DRGN_OBJECT(tmp, drgn_object_program(task));
 
 	// If CONFIG_THREAD_INFO_IN_TASK=y and since Linux kernel commit
 	// bcf9033e5449 ("sched: move CPU field back into thread_info if
@@ -312,16 +320,16 @@ struct drgn_error *linux_helper_task_cpu(const struct drgn_object *task,
 		drgn_error_destroy(err);
 		err = drgn_object_member_dereference(&tmp, task, "stack");
 		if (err)
-			goto out;
+			return err;
 		struct drgn_qualified_type thread_info_type;
 		err = drgn_program_find_type(drgn_object_program(task),
 					     "struct thread_info *", NULL,
 					     &thread_info_type);
 		if (err)
-			goto out;
+			return err;
 		err = drgn_object_cast(&tmp, thread_info_type, &tmp);
 		if (err)
-			goto out;
+			return err;
 		err = drgn_object_member_dereference(&tmp, &tmp, "cpu");
 	}
 	if (!err) {
@@ -335,8 +343,6 @@ struct drgn_error *linux_helper_task_cpu(const struct drgn_object *task,
 		*ret = 0;
 		err = NULL;
 	}
-out:
-	drgn_object_deinit(&tmp);
 	return err;
 }
 
@@ -346,13 +352,12 @@ linux_helper_xa_load(struct drgn_object *res,
 {
 	struct drgn_error *err;
 
-	struct drgn_object entry, node, tmp;
 	struct drgn_qualified_type node_type;
 	uint64_t internal_flag, node_min;
 
-	drgn_object_init(&entry, drgn_object_program(res));
-	drgn_object_init(&node, drgn_object_program(res));
-	drgn_object_init(&tmp, drgn_object_program(res));
+	DRGN_OBJECT(entry, drgn_object_program(res));
+	DRGN_OBJECT(node, drgn_object_program(res));
+	DRGN_OBJECT(tmp, drgn_object_program(res));
 
 	// See xa_for_each() in drgn/helpers/linux/xarray.py for a description
 	// of the cases we have to handle.
@@ -361,13 +366,13 @@ linux_helper_xa_load(struct drgn_object *res,
 	if (!err) {
 		err = drgn_object_read(&entry, &entry);
 		if (err)
-			goto out;
+			return err;
 		// node_type = struct xa_node *
 		err = drgn_program_find_type(drgn_object_program(res),
 					     "struct xa_node *", NULL,
 					     &node_type);
 		if (err)
-			goto out;
+			return err;
 		internal_flag = 2;
 		node_min = 4097;
 	} else if (err->code == DRGN_ERROR_LOOKUP) {
@@ -375,21 +380,21 @@ linux_helper_xa_load(struct drgn_object *res,
 		// entry = (void *)xa->rnode
 		err = drgn_object_member_dereference(&entry, xa, "rnode");
 		if (err)
-			goto out;
+			return err;
 		// node_type = typeof(xa->rnode)
 		node_type = drgn_object_qualified_type(&entry);
 		struct drgn_qualified_type voidp_type;
 		err = drgn_program_find_type(drgn_object_program(res), "void *",
 					     NULL, &voidp_type);
 		if (err)
-			goto out;
+			return err;
 		err = drgn_object_cast(&entry, voidp_type, &entry);
 		if (err)
-			goto out;
+			return err;
 		internal_flag = 1;
 		node_min = 0;
 	} else {
-		goto out;
+		return err;
 	}
 
 	// xa_is_node() or radix_tree_is_internal_node()
@@ -401,28 +406,27 @@ linux_helper_xa_load(struct drgn_object *res,
 	err = drgn_type_find_member(drgn_type_type(node_type.type).type,
 				    "slots", &member, &member_bit_offset);
 	if (err)
-		goto out;
+		return err;
 	struct drgn_qualified_type member_type;
 	err = drgn_member_type(member, &member_type, NULL);
 	if (err)
-		goto out;
+		return err;
 	if (drgn_type_kind(member_type.type) != DRGN_TYPE_ARRAY) {
-		err = drgn_error_create(DRGN_ERROR_TYPE,
-					"struct xa_node slots member is not an array");
-		goto out;
+		return drgn_error_create(DRGN_ERROR_TYPE,
+					 "struct xa_node slots member is not an array");
 	}
 	uint64_t XA_CHUNK_MASK = drgn_type_length(member_type.type) - 1;
 	uint64_t sizeof_slots;
 	if (node_min == 0) { // !xarray
 		err = drgn_type_sizeof(member_type.type, &sizeof_slots);
 		if (err)
-			goto out;
+			return err;
 	}
 
 	uint64_t entry_value;
 	err = drgn_object_read_unsigned(&entry, &entry_value);
 	if (err)
-		goto out;
+		return err;
 	if (is_node(entry_value)) {
 		// node = xa_to_node(entry)
 		// or
@@ -430,15 +434,15 @@ linux_helper_xa_load(struct drgn_object *res,
 		err = drgn_object_set_unsigned(&node, node_type,
 					       entry_value - internal_flag, 0);
 		if (err)
-			goto out;
+			return err;
 		// node_shift = node->shift
 		err = drgn_object_member_dereference(&tmp, &node, "shift");
 		if (err)
-			goto out;
+			return err;
 		union drgn_value node_shift;
 		err = drgn_object_read_integer(&tmp, &node_shift);
 		if (err)
-			goto out;
+			return err;
 
 		uint64_t offset;
 		if (node_shift.uvalue >= 64) // Avoid undefined behavior.
@@ -453,16 +457,16 @@ linux_helper_xa_load(struct drgn_object *res,
 			err = drgn_object_member_dereference(&tmp, &node,
 							     "slots");
 			if (err)
-				goto out;
+				return err;
 			err = drgn_object_subscript(&entry, &tmp, offset);
 			if (err)
-				goto out;
+				return err;
 			err = drgn_object_read(&entry, &entry);
 			if (err)
-				goto out;
+				return err;
 			err = drgn_object_read_unsigned(&entry, &entry_value);
 			if (err)
-				goto out;
+				return err;
 
 			if ((entry_value & 3) == internal_flag) {
 				if (node_min != 0 && // xarray
@@ -472,14 +476,14 @@ linux_helper_xa_load(struct drgn_object *res,
 								    &tmp,
 								    entry_value >> 2);
 					if (err)
-						goto out;
+						return err;
 					err = drgn_object_read(&entry, &entry);
 					if (err)
-						goto out;
+						return err;
 					err = drgn_object_read_unsigned(&entry,
 									&entry_value);
 					if (err)
-						goto out;
+						return err;
 				} else if (node_min == 0 && // !xarray
 					   tmp.address <= entry_value &&
 					   entry_value < tmp.address + sizeof_slots) { // is_sibling_entry()
@@ -490,24 +494,24 @@ linux_helper_xa_load(struct drgn_object *res,
 								     NULL,
 								     &voidpp_type);
 					if (err)
-						goto out;
+						return err;
 					err = drgn_object_set_unsigned(&entry,
 								       voidpp_type,
 								       entry_value - 1,
 								       0);
 					if (err)
-						goto out;
+						return err;
 					err = drgn_object_dereference(&entry,
 								      &entry);
 					if (err)
-						goto out;
+						return err;
 					err = drgn_object_read(&entry, &entry);
 					if (err)
-						goto out;
+						return err;
 					err = drgn_object_read_unsigned(&entry,
 									&entry_value);
 					if (err)
-						goto out;
+						return err;
 				}
 			}
 
@@ -521,15 +525,15 @@ linux_helper_xa_load(struct drgn_object *res,
 						       entry_value - internal_flag,
 						       0);
 			if (err)
-				goto out;
+				return err;
 			// node_shift = node->shift
 			err = drgn_object_member_dereference(&tmp, &node,
 							     "shift");
 			if (err)
-				goto out;
+				return err;
 			err = drgn_object_read_integer(&tmp, &node_shift);
 			if (err)
-				goto out;
+				return err;
 
 			if (node_shift.uvalue >= 64) // Avoid undefined behavior.
 				offset = 0;
@@ -540,17 +544,11 @@ linux_helper_xa_load(struct drgn_object *res,
 		goto null;
 	}
 
-	err = drgn_object_copy(res, &entry);
-out:
-	drgn_object_deinit(&tmp);
-	drgn_object_deinit(&node);
-	drgn_object_deinit(&entry);
-	return err;
+	return drgn_object_copy(res, &entry);
 
 null:
-	err = drgn_object_set_unsigned(res, drgn_object_qualified_type(&entry),
-				       0, 0);
-	goto out;
+	return drgn_object_set_unsigned(res, drgn_object_qualified_type(&entry),
+					0, 0);
 
 #undef is_node
 }
@@ -565,9 +563,8 @@ struct drgn_error *linux_helper_idr_find(struct drgn_object *res,
 					 uint64_t id)
 {
 	struct drgn_error *err;
-	struct drgn_object tmp;
 
-	drgn_object_init(&tmp, drgn_object_program(res));
+	DRGN_OBJECT(tmp, drgn_object_program(res));
 
 	/* id -= idr->idr_base */
 	err = drgn_object_member_dereference(&tmp, idr, "idr_base");
@@ -576,26 +573,23 @@ struct drgn_error *linux_helper_idr_find(struct drgn_object *res,
 
 		err = drgn_object_read_integer(&tmp, &idr_base);
 		if (err)
-			goto out;
+			return err;
 		id -= idr_base.uvalue;
 	} else if (err->code == DRGN_ERROR_LOOKUP) {
 		/* idr_base was added in v4.16. */
 		drgn_error_destroy(err);
 	} else {
-		goto out;
+		return err;
 	}
 
 	/* radix_tree_lookup(&idr->idr_rt, id) */
 	err = drgn_object_member_dereference(&tmp, idr, "idr_rt");
 	if (err)
-		goto out;
+		return err;
 	err = drgn_object_address_of(&tmp, &tmp);
 	if (err)
-		goto out;
-	err = linux_helper_xa_load(res, &tmp, id);
-out:
-	drgn_object_deinit(&tmp);
-	return err;
+		return err;
+	return linux_helper_xa_load(res, &tmp, id);
 }
 
 /*
@@ -653,35 +647,34 @@ find_pid_in_pid_hash(struct drgn_object *res, const struct drgn_object *ns,
 	if (err)
 		return err;
 
-	struct drgn_object node, tmp;
-	drgn_object_init(&node, drgn_object_program(res));
-	drgn_object_init(&tmp, drgn_object_program(res));
+	DRGN_OBJECT(node, drgn_object_program(res));
+	DRGN_OBJECT(tmp, drgn_object_program(res));
 
 	err = drgn_object_read(&tmp, ns);
 	if (err)
-		goto out;
+		return err;
 	uint64_t ns_addr;
 	err = drgn_object_read_unsigned(&tmp, &ns_addr);
 	if (err)
-		goto out;
+		return err;
 	union drgn_value ns_level;
 	err = drgn_object_member_dereference(&tmp, &tmp, "level");
 	if (err)
-		goto out;
+		return err;
 	err = drgn_object_read_integer(&tmp, &ns_level);
 	if (err)
-		goto out;
+		return err;
 
 	/* i = 1 << pidhash_shift */
 	err = drgn_program_find_object(drgn_object_program(res),
 				       "pidhash_shift", NULL,
 				       DRGN_FIND_OBJECT_ANY, &tmp);
 	if (err)
-		goto out;
+		return err;
 	union drgn_value pidhash_shift;
 	err = drgn_object_read_integer(&tmp, &pidhash_shift);
 	if (err)
-		goto out;
+		return err;
 	uint64_t i;
 	if (pidhash_shift.uvalue >= 64)
 		i = 0;
@@ -691,10 +684,10 @@ find_pid_in_pid_hash(struct drgn_object *res, const struct drgn_object *ns,
 		/* for (node = pid_hash[i].first; node; node = node->next) */
 		err = drgn_object_subscript(&node, pid_hash, i);
 		if (err)
-			goto out;
+			return err;
 		err = drgn_object_member(&node, &node, "first");
 		if (err)
-			goto out;
+			return err;
 		for (;;) {
 			uint64_t addr, tmp_addr;
 			union drgn_value node_nr;
@@ -702,10 +695,10 @@ find_pid_in_pid_hash(struct drgn_object *res, const struct drgn_object *ns,
 
 			err = drgn_object_read(&node, &node);
 			if (err)
-				goto out;
+				return err;
 			err = drgn_object_read_unsigned(&node, &addr);
 			if (err)
-				goto out;
+				return err;
 			if (!addr)
 				break;
 			addr -= pid_chain_bit_offset / 8;
@@ -715,10 +708,10 @@ find_pid_in_pid_hash(struct drgn_object *res, const struct drgn_object *ns,
 			err = drgn_object_set_reference(&tmp, nr_type, tmp_addr,
 							0, 0);
 			if (err)
-				goto out;
+				return err;
 			err = drgn_object_read_integer(&tmp, &node_nr);
 			if (err)
-				goto out;
+				return err;
 			if (node_nr.uvalue != pid)
 				goto next;
 
@@ -727,11 +720,11 @@ find_pid_in_pid_hash(struct drgn_object *res, const struct drgn_object *ns,
 			err = drgn_object_set_reference(&tmp, ns_type, tmp_addr,
 							0, 0);
 			if (err)
-				goto out;
+				return err;
 
 			err = drgn_object_read_unsigned(&tmp, &node_ns);
 			if (err)
-				goto out;
+				return err;
 			if (node_ns != ns_addr)
 				goto next;
 
@@ -743,23 +736,18 @@ find_pid_in_pid_hash(struct drgn_object *res, const struct drgn_object *ns,
 			snprintf(member, sizeof(member), FORMAT,
 				 ns_level.uvalue);
 #undef FORMAT
-			err = drgn_object_container_of(res, &node,
-						       drgn_type_type(pidp_type.type),
-						       member);
-			goto out;
+			return drgn_object_container_of(res, &node,
+							drgn_type_type(pidp_type.type),
+							member);
 
 next:
 			err = drgn_object_member_dereference(&node, &node, "next");
 			if (err)
-				goto out;
+				return err;
 		}
 	}
 
-	err = drgn_object_set_unsigned(res, pidp_type, 0, 0);
-out:
-	drgn_object_deinit(&tmp);
-	drgn_object_deinit(&node);
-	return err;
+	return drgn_object_set_unsigned(res, pidp_type, 0, 0);
 }
 
 struct drgn_error *linux_helper_find_pid(struct drgn_object *res,
@@ -767,9 +755,8 @@ struct drgn_error *linux_helper_find_pid(struct drgn_object *res,
 					 uint64_t pid)
 {
 	struct drgn_error *err;
-	struct drgn_object tmp;
 
-	drgn_object_init(&tmp, drgn_object_program(res));
+	DRGN_OBJECT(tmp, drgn_object_program(res));
 
 	/* (struct pid *)idr_find(&ns->idr, pid) */
 	err = drgn_object_member_dereference(&tmp, ns, "idr");
@@ -778,28 +765,27 @@ struct drgn_error *linux_helper_find_pid(struct drgn_object *res,
 
 		err = drgn_object_address_of(&tmp, &tmp);
 		if (err)
-			goto out;
+			return err;
 		err = linux_helper_idr_find(&tmp, &tmp, pid);
 		if (err)
-			goto out;
+			return err;
 		err = drgn_program_find_type(drgn_object_program(res),
 					     "struct pid *", NULL,
 					     &qualified_type);
 		if (err)
-			goto out;
-		err = drgn_object_cast(res, qualified_type, &tmp);
+			return err;
+		return drgn_object_cast(res, qualified_type, &tmp);
 	} else if (err->code == DRGN_ERROR_LOOKUP) {
 		drgn_error_destroy(err);
 		err = drgn_program_find_object(drgn_object_program(res),
 					       "pid_hash", NULL,
 					       DRGN_FIND_OBJECT_ANY, &tmp);
 		if (err)
-			goto out;
-		err = find_pid_in_pid_hash(res, ns, &tmp, pid);
+			return err;
+		return find_pid_in_pid_hash(res, ns, &tmp, pid);
+	} else {
+		return err;
 	}
-out:
-	drgn_object_deinit(&tmp);
-	return err;
 }
 
 struct drgn_error *linux_helper_pid_task(struct drgn_object *res,
@@ -810,37 +796,36 @@ struct drgn_error *linux_helper_pid_task(struct drgn_object *res,
 	struct drgn_qualified_type task_structp_type;
 	struct drgn_qualified_type task_struct_type;
 	bool truthy;
-	struct drgn_object first;
 
-	drgn_object_init(&first, drgn_object_program(res));
+	DRGN_OBJECT(first, drgn_object_program(res));
 
 	err = drgn_program_find_type(drgn_object_program(res),
 				     "struct task_struct *", NULL,
 				     &task_structp_type);
 	if (err)
-		goto out;
+		return err;
 	task_struct_type = drgn_type_type(task_structp_type.type);
 
 	err = drgn_object_bool(pid, &truthy);
 	if (err)
-		goto out;
+		return err;
 	if (!truthy)
 		goto null;
 
 	/* first = &pid->tasks[pid_type].first */
 	err = drgn_object_member_dereference(&first, pid, "tasks");
 	if (err)
-		goto out;
+		return err;
 	err = drgn_object_subscript(&first, &first, pid_type);
 	if (err)
-		goto out;
+		return err;
 	err = drgn_object_member(&first, &first, "first");
 	if (err)
-		goto out;
+		return err;
 
 	err = drgn_object_bool(&first, &truthy);
 	if (err)
-		goto out;
+		return err;
 	if (!truthy)
 		goto null;
 
@@ -863,13 +848,10 @@ struct drgn_error *linux_helper_pid_task(struct drgn_object *res,
 		err = drgn_object_container_of(res, &first, task_struct_type,
 					       member);
 	}
-out:
-	drgn_object_deinit(&first);
 	return err;
 
 null:
-	err = drgn_object_set_unsigned(res, task_structp_type, 0, 0);
-	goto out;
+	return drgn_object_set_unsigned(res, task_structp_type, 0, 0);
 }
 
 struct drgn_error *linux_helper_find_task(struct drgn_object *res,
@@ -877,29 +859,45 @@ struct drgn_error *linux_helper_find_task(struct drgn_object *res,
 					  uint64_t pid)
 {
 	struct drgn_error *err;
-	struct drgn_object pid_obj;
-	struct drgn_object pid_type_obj;
 	union drgn_value pid_type;
 
-	drgn_object_init(&pid_obj, drgn_object_program(res));
-	drgn_object_init(&pid_type_obj, drgn_object_program(res));
+	DRGN_OBJECT(pid_obj, drgn_object_program(res));
+	DRGN_OBJECT(pid_type_obj, drgn_object_program(res));
 
 	err = linux_helper_find_pid(&pid_obj, ns, pid);
 	if (err)
-		goto out;
+		return err;
 	err = drgn_program_find_object(drgn_object_program(res), "PIDTYPE_PID",
 				       NULL, DRGN_FIND_OBJECT_CONSTANT,
 				       &pid_type_obj);
 	if (err)
-		goto out;
+		return err;
 	err = drgn_object_read_integer(&pid_type_obj, &pid_type);
 	if (err)
-		goto out;
-	err = linux_helper_pid_task(res, &pid_obj, pid_type.uvalue);
-out:
-	drgn_object_deinit(&pid_type_obj);
-	drgn_object_deinit(&pid_obj);
-	return err;
+		return err;
+	return linux_helper_pid_task(res, &pid_obj, pid_type.uvalue);
+}
+
+static inline struct drgn_error *
+linux_helper_task_iterator_set_thread_node(struct linux_helper_task_iterator *it)
+{
+	struct drgn_error *err;
+	err = drgn_object_container_of(&it->thread_node, &it->tasks_node,
+				       it->task_struct_type, "tasks");
+	if (err)
+		return err;
+	err = drgn_object_member_dereference(&it->thread_node, &it->thread_node,
+					     "signal");
+	if (err)
+		return err;
+	err = drgn_object_member_dereference(&it->thread_node, &it->thread_node,
+					     "thread_head");
+	if (err)
+		return err;
+	err = drgn_object_address_of(&it->thread_node, &it->thread_node);
+	if (err)
+		return err;
+	return drgn_object_read_unsigned(&it->thread_node, &it->thread_head);
 }
 
 struct drgn_error *
@@ -907,74 +905,99 @@ linux_helper_task_iterator_init(struct linux_helper_task_iterator *it,
 				struct drgn_program *prog)
 {
 	struct drgn_error *err;
-	it->done = false;
-	drgn_object_init(&it->task, prog);
+	drgn_object_init(&it->tasks_node, prog);
+	drgn_object_init(&it->thread_node, prog);
+
 	err = drgn_program_find_object(prog, "init_task", NULL,
-				       DRGN_FIND_OBJECT_VARIABLE, &it->task);
+				       DRGN_FIND_OBJECT_VARIABLE,
+				       &it->tasks_node);
 	if (err)
 		goto err;
-	it->task_struct_type = drgn_object_qualified_type(&it->task);
-	err = drgn_object_address_of(&it->task, &it->task);
+	it->task_struct_type = drgn_object_qualified_type(&it->tasks_node);
+	err = drgn_object_member(&it->tasks_node, &it->tasks_node, "tasks");
 	if (err)
 		goto err;
-	err = drgn_object_read_unsigned(&it->task, &it->init_task_address);
+	if (it->tasks_node.kind != DRGN_OBJECT_REFERENCE) {
+		err = drgn_error_create(DRGN_ERROR_OTHER,
+					"can't get address of tasks list");
+		goto err;
+	}
+	it->tasks_head = it->tasks_node.address;
+	err = drgn_object_member(&it->tasks_node, &it->tasks_node, "next");
 	if (err)
 		goto err;
-	it->thread_group_address = it->init_task_address;
+	err = drgn_object_read(&it->tasks_node, &it->tasks_node);
+	if (err)
+		goto err;
+	uint64_t tasks_node_value;
+	err = drgn_object_read_unsigned(&it->tasks_node, &tasks_node_value);
+	if (err)
+		goto err;
+	if (tasks_node_value == it->tasks_head) {
+		it->done = true;
+	} else {
+		it->done = false;
+		err = linux_helper_task_iterator_set_thread_node(it);
+		if (err)
+			goto err;
+	}
 	return NULL;
 
 err:
-	drgn_object_deinit(&it->task);
+	linux_helper_task_iterator_deinit(it);
 	return err;
 }
 
 void linux_helper_task_iterator_deinit(struct linux_helper_task_iterator *it)
 {
-	drgn_object_deinit(&it->task);
+	drgn_object_deinit(&it->thread_node);
+	drgn_object_deinit(&it->tasks_node);
 }
 
 struct drgn_error *
 linux_helper_task_iterator_next(struct linux_helper_task_iterator *it,
-				const struct drgn_object **ret)
+				struct drgn_object *ret)
 {
-	if (it->done) {
-		*ret = NULL;
-		return NULL;
-	}
 	struct drgn_error *err;
-	struct drgn_object *task = &it->task;
-	err = drgn_object_member_dereference(task, task, "thread_group");
-	if (err)
-		return err;
-	err = drgn_object_member(task, task, "next");
-	if (err)
-		return err;
-	err = drgn_object_container_of(task, task, it->task_struct_type,
-				       "thread_group");
-	if (err)
-		return err;
-	uint64_t task_address;
-	err = drgn_object_read_unsigned(task, &task_address);
-	if (err)
-		return err;
-	if (task_address == it->thread_group_address) {
-		err = drgn_object_member_dereference(task, task, "tasks");
+
+	if (it->done)
+		return &drgn_stop;
+
+	for (;;) {
+		err = drgn_object_member_dereference(&it->thread_node,
+						     &it->thread_node, "next");
 		if (err)
 			return err;
-		err = drgn_object_member(task, task, "next");
+		err = drgn_object_read(&it->thread_node, &it->thread_node);
 		if (err)
 			return err;
-		err = drgn_object_container_of(task, task,
-					       it->task_struct_type, "tasks");
+		uint64_t thread_node_value;
+		err = drgn_object_read_unsigned(&it->thread_node, &thread_node_value);
 		if (err)
 			return err;
-		err = drgn_object_read_unsigned(task,
-						&it->thread_group_address);
+		if (thread_node_value != it->thread_head)
+			break;
+
+		err = drgn_object_member_dereference(&it->tasks_node,
+						     &it->tasks_node, "next");
 		if (err)
 			return err;
-		if (it->thread_group_address == it->init_task_address)
+		err = drgn_object_read(&it->tasks_node, &it->tasks_node);
+		if (err)
+			return err;
+		uint64_t tasks_node_value;
+		err = drgn_object_read_unsigned(&it->tasks_node,
+						&tasks_node_value);
+		if (err)
+			return err;
+		if (tasks_node_value == it->tasks_head) {
 			it->done = true;
+			return &drgn_stop;
+		}
+		err = linux_helper_task_iterator_set_thread_node(it);
+		if (err)
+			return err;
 	}
-	*ret = task;
-	return NULL;
+	return drgn_object_container_of(ret, &it->thread_node,
+					it->task_struct_type, "thread_node");
 }
