@@ -18,11 +18,13 @@
 #include <sys/types.h>
 #include <unistd.h>
 
+#include "cleanup.h"
 #include "debug_info.h"
 #include "error.h"
 #include "helpers.h"
 #include "io.h"
 #include "language.h"
+#include "log.h"
 #include "linux_kernel.h"
 #include "memory_reader.h"
 #include "minmax.h"
@@ -37,9 +39,9 @@ static inline uint32_t drgn_thread_to_key(const struct drgn_thread *entry)
 	return entry->tid;
 }
 
-DEFINE_VECTOR_FUNCTIONS(drgn_prstatus_vector)
+DEFINE_VECTOR_FUNCTIONS(drgn_prstatus_vector);
 DEFINE_HASH_TABLE_FUNCTIONS(drgn_thread_set, drgn_thread_to_key,
-			    int_key_hash_pair, scalar_key_eq)
+			    int_key_hash_pair, scalar_key_eq);
 
 struct drgn_thread_iterator {
 	struct drgn_program *prog;
@@ -99,11 +101,14 @@ void drgn_program_init(struct drgn_program *prog,
 	drgn_memory_reader_init(&prog->reader);
 	drgn_program_init_types(prog);
 	drgn_object_index_init(&prog->oindex);
+	drgn_debug_info_init(&prog->dbinfo, prog);
 	prog->core_fd = -1;
 	if (platform)
 		drgn_program_set_platform(prog, platform);
 	char *env = getenv("DRGN_PREFER_ORC_UNWINDER");
 	prog->prefer_orc_unwinder = env && atoi(env);
+	drgn_program_set_log_level(prog, DRGN_LOG_NONE);
+	drgn_program_set_log_file(prog, stderr);
 	drgn_object_init(&prog->vmemmap, prog);
 }
 
@@ -134,6 +139,7 @@ void drgn_program_deinit(struct drgn_program *prog)
 	drgn_memory_reader_deinit(&prog->reader);
 
 	free(prog->file_segments);
+	free(prog->vmcoreinfo.raw);
 
 #ifdef WITH_LIBKDUMPFILE
 	if (prog->kdump_ctx)
@@ -143,7 +149,7 @@ void drgn_program_deinit(struct drgn_program *prog)
 	if (prog->core_fd != -1)
 		close(prog->core_fd);
 
-	drgn_debug_info_destroy(prog->dbinfo);
+	drgn_debug_info_deinit(&prog->dbinfo);
 }
 
 LIBDRGN_PUBLIC struct drgn_error *
@@ -185,11 +191,19 @@ drgn_program_add_memory_segment(struct drgn_program *prog, uint64_t address,
 					      physical);
 }
 
+struct drgn_error *
+drgn_program_add_object_finder_impl(struct drgn_program *prog,
+				    struct drgn_object_finder *finder,
+				    drgn_object_find_fn fn, void *arg)
+{
+	return drgn_object_index_add_finder(&prog->oindex, finder, fn, arg);
+}
+
 LIBDRGN_PUBLIC struct drgn_error *
 drgn_program_add_object_finder(struct drgn_program *prog,
 			       drgn_object_find_fn fn, void *arg)
 {
-	return drgn_object_index_add_finder(&prog->oindex, fn, arg);
+	return drgn_program_add_object_finder_impl(prog, NULL, fn, arg);
 }
 
 static struct drgn_error *
@@ -202,20 +216,30 @@ drgn_program_check_initialized(struct drgn_program *prog)
 	return NULL;
 }
 
-static struct drgn_error *has_kdump_signature(const char *path, int fd,
-					      bool *ret)
+static struct drgn_error *
+has_kdump_signature(struct drgn_program *prog, const char *path, bool *ret)
 {
-	char signature[KDUMP_SIG_LEN];
-	ssize_t r = pread_all(fd, signature, sizeof(signature), 0);
+	char signature[max_iconst(KDUMP_SIG_LEN, FLATTENED_SIG_LEN)];
+	ssize_t r = pread_all(prog->core_fd, signature, sizeof(signature), 0);
 	if (r < 0)
 		return drgn_error_create_os("pread", errno, path);
-	*ret = (r == sizeof(signature)
-		&& memcmp(signature, KDUMP_SIGNATURE, sizeof(signature)) == 0);
+	*ret = false;
+	if (r >= FLATTENED_SIG_LEN
+	    && memcmp(signature, FLATTENED_SIGNATURE, FLATTENED_SIG_LEN) == 0) {
+		drgn_log_warning(prog,
+				 "the given file is in the makedumpfile flattened "
+				 "format; if open fails or is too slow, reassemble "
+				 "it with 'makedumpfile -R newfile <oldfile'");
+		*ret = true;
+	} else if (r >= KDUMP_SIG_LEN
+		   && memcmp(signature, KDUMP_SIGNATURE, KDUMP_SIG_LEN) == 0)
+		*ret = true;
 	return NULL;
 }
 
-LIBDRGN_PUBLIC struct drgn_error *
-drgn_program_set_core_dump(struct drgn_program *prog, const char *path)
+static struct drgn_error *
+drgn_program_set_core_dump_fd_internal(struct drgn_program *prog, int fd,
+				       const char *path)
 {
 	struct drgn_error *err;
 	GElf_Ehdr ehdr_mem, *ehdr;
@@ -229,15 +253,8 @@ drgn_program_set_core_dump(struct drgn_program *prog, const char *path)
 	size_t vmcoreinfo_size = 0;
 	bool have_nt_taskstruct = false, is_proc_kcore;
 
-	err = drgn_program_check_initialized(prog);
-	if (err)
-		return err;
-
-	prog->core_fd = open(path, O_RDONLY);
-	if (prog->core_fd == -1)
-		return drgn_error_create_os("open", errno, path);
-
-	err = has_kdump_signature(path, prog->core_fd, &is_kdump);
+	prog->core_fd = fd;
+	err = has_kdump_signature(prog, path, &is_kdump);
 	if (err)
 		goto out_fd;
 	if (is_kdump) {
@@ -535,7 +552,8 @@ drgn_program_set_core_dump(struct drgn_program *prog, const char *path)
 				goto out_segments;
 		}
 		prog->flags |= (DRGN_PROGRAM_IS_LINUX_KERNEL |
-				DRGN_PROGRAM_IS_LIVE);
+				DRGN_PROGRAM_IS_LIVE |
+		                DRGN_PROGRAM_IS_LOCAL);
 		elf_end(prog->core);
 		prog->core = NULL;
 	} else if (vmcoreinfo_note) {
@@ -570,6 +588,7 @@ out_segments:
 out_notes:
 	// Reset anything we parsed from ELF notes.
 	prog->aarch64_insn_pac_mask = 0;
+	free(prog->vmcoreinfo.raw);
 	memset(&prog->vmcoreinfo, 0, sizeof(prog->vmcoreinfo));
 out_platform:
 	prog->has_platform = had_platform;
@@ -582,6 +601,39 @@ out_fd:
 	return err;
 }
 
+LIBDRGN_PUBLIC struct drgn_error *
+drgn_program_set_core_dump_fd(struct drgn_program *prog, int fd)
+{
+	struct drgn_error *err;
+
+	err = drgn_program_check_initialized(prog);
+	if (err)
+		return err;
+
+	#define FORMAT "/proc/self/fd/%d"
+	char path[sizeof(FORMAT) - sizeof("%d") + max_decimal_length(int) + 1];
+	snprintf(path, sizeof(path), FORMAT, fd);
+	#undef FORMAT
+
+	return drgn_program_set_core_dump_fd_internal(prog, fd, path);
+}
+
+LIBDRGN_PUBLIC struct drgn_error *
+drgn_program_set_core_dump(struct drgn_program *prog, const char *path)
+{
+	struct drgn_error *err;
+
+	err = drgn_program_check_initialized(prog);
+	if (err)
+		return err;
+
+	int fd = open(path, O_RDONLY);
+	if (fd == -1)
+		return drgn_error_create_os("open", errno, path);
+
+	return drgn_program_set_core_dump_fd_internal(prog, fd, path);
+}
+
 LIBDRGN_PUBLIC struct drgn_error *
 drgn_program_set_kernel(struct drgn_program *prog)
 {
@@ -625,7 +677,7 @@ drgn_program_set_pid(struct drgn_program *prog, pid_t pid)
 		goto out_segments;
 
 	prog->pid = pid;
-	prog->flags |= DRGN_PROGRAM_IS_LIVE;
+	prog->flags |= DRGN_PROGRAM_IS_LIVE | DRGN_PROGRAM_IS_LOCAL;
 	return NULL;
 
 out_segments:
@@ -648,9 +700,11 @@ static void drgn_program_set_language_from_main(struct drgn_program *prog)
 	if (prog->flags & DRGN_PROGRAM_IS_LINUX_KERNEL)
 		return;
 	const struct drgn_language *lang;
-	err = drgn_debug_info_main_language(prog->dbinfo, &lang);
-	if (err)
+	err = drgn_debug_info_main_language(&prog->dbinfo, &lang);
+	if (err) {
 		drgn_error_destroy(err);
+		return;
+	}
 	if (lang)
 		prog->lang = lang;
 }
@@ -684,35 +738,13 @@ drgn_program_load_debug_info(struct drgn_program *prog, const char **paths,
 	if (!n && !load_default && !load_main)
 		return NULL;
 
-	struct drgn_debug_info *dbinfo = prog->dbinfo;
-	if (!dbinfo) {
-		err = drgn_debug_info_create(prog, &dbinfo);
-		if (err)
-			return err;
-		err = drgn_program_add_object_finder(prog,
-						     drgn_debug_info_find_object,
-						     dbinfo);
-		if (err) {
-			drgn_debug_info_destroy(dbinfo);
-			return err;
-		}
-		err = drgn_program_add_type_finder(prog,
-						   drgn_debug_info_find_type,
-						   dbinfo);
-		if (err) {
-			drgn_object_index_remove_finder(&prog->oindex);
-			drgn_debug_info_destroy(dbinfo);
-			return err;
-		}
-		prog->dbinfo = dbinfo;
-	}
-
-	err = drgn_debug_info_load(dbinfo, paths, n, load_default, load_main);
+	drgn_blocking_guard(prog);
+	err = drgn_debug_info_load(&prog->dbinfo, paths, n, load_default, load_main);
 	if ((!err || err->code == DRGN_ERROR_MISSING_DEBUG_INFO)) {
 		if (!prog->lang)
 			drgn_program_set_language_from_main(prog);
 		if (!prog->has_platform) {
-			dwfl_getdwarf(dbinfo->dwfl,
+			dwfl_getdwarf(prog->dbinfo.dwfl,
 				      drgn_set_platform_from_dwarf, prog, 0);
 		}
 	}
@@ -1060,25 +1092,20 @@ drgn_thread_iterator_next_linux_kernel(struct drgn_thread_iterator *it,
 				       struct drgn_thread **ret)
 {
 	struct drgn_error *err;
-	const struct drgn_object *task;
-	err = linux_helper_task_iterator_next(&it->task_iter, &task);
-	if (err)
-		return err;
-	if (!task) {
+	err = linux_helper_task_iterator_next(&it->task_iter,
+					      &it->entry.object);
+	if (err == &drgn_stop) {
 		*ret = NULL;
 		return NULL;
-	}
-	it->entry.prog = drgn_object_program(task);
-	err = drgn_object_copy(&it->entry.object, task);
-	if (err)
+	} else if (err) {
 		return err;
-	struct drgn_object tid;
+	}
+	it->entry.prog = drgn_object_program(&it->entry.object);
 	union drgn_value tid_value;
-	drgn_object_init(&tid, drgn_object_program(task));
-	err = drgn_object_member_dereference(&tid, task, "pid");
+	DRGN_OBJECT(tid, drgn_object_program(&it->entry.object));
+	err = drgn_object_member_dereference(&tid, &it->entry.object, "pid");
 	if (!err)
 		err = drgn_object_read_integer(&tid, &tid_value);
-	drgn_object_deinit(&tid);
 	if (err)
 		return err;
 	it->entry.tid = tid_value.uvalue;
@@ -1237,8 +1264,7 @@ static struct drgn_error *
 drgn_program_kernel_get_crashed_cpu(struct drgn_program *prog, uint64_t *ret)
 {
 	struct drgn_error *err;
-	struct drgn_object cpu;
-	drgn_object_init(&cpu, prog);
+	DRGN_OBJECT(cpu, prog);
 	union drgn_value cpu_value;
 
 	// Since Linux kernel commit 1717f2096b54 ("panic, x86: Fix re-entrance
@@ -1249,7 +1275,7 @@ drgn_program_kernel_get_crashed_cpu(struct drgn_program *prog, uint64_t *ret)
 	if (!err) {
 		err = drgn_object_member(&cpu, &cpu, "counter");
 		if (err)
-			goto out;
+			return err;
 		err = drgn_object_read_integer(&cpu, &cpu_value);
 		if (!err)
 			*ret = cpu_value.uvalue;
@@ -1264,7 +1290,7 @@ drgn_program_kernel_get_crashed_cpu(struct drgn_program *prog, uint64_t *ret)
 		if (!err) {
 			err = drgn_object_read_integer(&cpu, &cpu_value);
 			if (err)
-				goto out;
+				return err;
 			// Since Linux kernel commit 5bc329503e81 ("x86/mce:
 			// Handle broadcasted MCE gracefully with kexec") (in
 			// v4.12), crashing_cpu is defined in !SMP kernels, but
@@ -1282,9 +1308,43 @@ drgn_program_kernel_get_crashed_cpu(struct drgn_program *prog, uint64_t *ret)
 			*ret = 0;
 		}
 	}
+	return err;
+}
+
+static struct drgn_error *
+drgn_program_find_thread_kernel_cpu_curr(struct drgn_program *prog,
+					 uint64_t cpu,
+					 struct drgn_thread **ret)
+{
+	struct drgn_error *err;
+	struct drgn_thread *thread = malloc(sizeof(*thread));
+	if (!thread)
+		return &drgn_enomem;
+	thread->prog = prog;
+
+	DRGN_OBJECT(tmp, prog);
+	drgn_object_init(&thread->object, prog);
+
+	err = linux_helper_cpu_curr(&thread->object, cpu);
+	if (err)
+		goto out;
+
+	err = drgn_object_member_dereference(&tmp, &thread->object, "pid");
+	if (err)
+		goto out;
+	union drgn_value tid;
+	err = drgn_object_read_integer(&tmp, &tid);
+	if (err)
+		goto out;
+	thread->tid = tid.uvalue;
+
+	*ret = thread;
 
 out:
-	drgn_object_deinit(&cpu);
+	if (err) {
+		drgn_object_deinit(&thread->object);
+		free(thread);
+	}
 	return err;
 }
 
@@ -1304,37 +1364,17 @@ drgn_program_kernel_core_dump_cache_crashed_thread(struct drgn_program *prog)
 	if (err)
 		return err;
 
-	if (crashed_cpu >= prog->prstatus_vector.size)
+	if (crashed_cpu >= drgn_prstatus_vector_size(&prog->prstatus_vector))
 		return NULL;
-	struct nstring *prstatus = &prog->prstatus_vector.data[crashed_cpu];
-	uint32_t crashed_thread_tid;
-	err = get_prstatus_pid(prog, prstatus->str, prstatus->len,
-			       &crashed_thread_tid);
-	if (err)
-		return err;
 
-	if (crashed_thread_tid == 0) {
-		prog->crashed_thread = malloc(sizeof(*prog->crashed_thread));
-		if (!prog->crashed_thread)
-			return &drgn_enomem;
-		prog->crashed_thread->prog = prog;
-		prog->crashed_thread->tid = crashed_thread_tid;
-		drgn_object_init(&prog->crashed_thread->object, prog);
-		err = linux_helper_idle_task(&prog->crashed_thread->object,
-					     crashed_cpu);
-		if (err) {
-			drgn_object_deinit(&prog->crashed_thread->object);
-			free(prog->crashed_thread);
-		}
-	} else {
-		err = drgn_program_find_thread(prog, crashed_thread_tid,
-					       &prog->crashed_thread);
-	}
+	err = drgn_program_find_thread_kernel_cpu_curr(prog, crashed_cpu,
+						       &prog->crashed_thread);
 	if (err) {
 		prog->crashed_thread = NULL;
 		return err;
 	}
-	prog->crashed_thread->prstatus = *prstatus;
+	prog->crashed_thread->prstatus =
+		*drgn_prstatus_vector_at(&prog->prstatus_vector, crashed_cpu);
 	return NULL;
 }
 
@@ -1407,22 +1447,20 @@ drgn_thread_object(struct drgn_thread *thread, const struct drgn_object **ret)
 
 struct drgn_error *drgn_program_find_prstatus_by_cpu(struct drgn_program *prog,
 						     uint32_t cpu,
-						     struct nstring *ret,
-						     uint32_t *tid_ret)
+						     struct nstring *ret)
 {
 	assert(prog->flags & DRGN_PROGRAM_IS_LINUX_KERNEL);
 	struct drgn_error *err = drgn_program_cache_core_dump_notes(prog);
 	if (err)
 		return err;
 
-	if (cpu < prog->prstatus_vector.size) {
-		*ret = prog->prstatus_vector.data[cpu];
-		return get_prstatus_pid(prog, ret->str, ret->len, tid_ret);
+	if (cpu < drgn_prstatus_vector_size(&prog->prstatus_vector)) {
+		*ret = *drgn_prstatus_vector_at(&prog->prstatus_vector, cpu);
 	} else {
 		ret->str = NULL;
 		ret->len = 0;
-		return NULL;
 	}
+	return NULL;
 }
 
 struct drgn_error *drgn_program_find_prstatus_by_tid(struct drgn_program *prog,
@@ -1462,6 +1500,21 @@ struct drgn_error *drgn_program_init_core_dump(struct drgn_program *prog,
 	return err;
 }
 
+struct drgn_error *drgn_program_init_core_dump_fd(struct drgn_program *prog, int fd)
+{
+	struct drgn_error *err;
+
+	err = drgn_program_set_core_dump_fd(prog, fd);
+	if (err)
+		return err;
+	err = drgn_program_load_debug_info(prog, NULL, 0, true, true);
+	if (err && err->code == DRGN_ERROR_MISSING_DEBUG_INFO) {
+		drgn_error_destroy(err);
+		err = NULL;
+	}
+	return err;
+}
+
 struct drgn_error *drgn_program_init_kernel(struct drgn_program *prog)
 {
 	struct drgn_error *err;
@@ -1514,6 +1567,28 @@ drgn_program_from_core_dump(const char *path, struct drgn_program **ret)
 	return NULL;
 }
 
+LIBDRGN_PUBLIC struct drgn_error *
+drgn_program_from_core_dump_fd(int fd, struct drgn_program **ret)
+{
+	struct drgn_error *err;
+	struct drgn_program *prog;
+
+	prog = malloc(sizeof(*prog));
+	if (!prog)
+		return &drgn_enomem;
+
+	drgn_program_init(prog, NULL);
+	err = drgn_program_init_core_dump_fd(prog, fd);
+	if (err) {
+		drgn_program_deinit(prog);
+		free(prog);
+		return err;
+	}
+
+	*ret = prog;
+	return NULL;
+}
+
 LIBDRGN_PUBLIC struct drgn_error *
 drgn_program_from_kernel(struct drgn_program **ret)
 {
@@ -1581,7 +1656,7 @@ drgn_program_read_memory(struct drgn_program *prog, void *buf, uint64_t address,
 	return NULL;
 }
 
-DEFINE_VECTOR(char_vector, char)
+DEFINE_VECTOR(char_vector, char);
 
 LIBDRGN_PUBLIC struct drgn_error *
 drgn_program_read_c_string(struct drgn_program *prog, uint64_t address,
@@ -1591,21 +1666,17 @@ drgn_program_read_c_string(struct drgn_program *prog, uint64_t address,
 	struct drgn_error *err = drgn_program_address_mask(prog, &address_mask);
 	if (err)
 		return err;
-	struct char_vector str = VECTOR_INIT;
+	_cleanup_(char_vector_deinit) struct char_vector str = VECTOR_INIT;
 	for (;;) {
 		address &= address_mask;
 		char *c = char_vector_append_entry(&str);
-		if (!c) {
-			char_vector_deinit(&str);
+		if (!c)
 			return &drgn_enomem;
-		}
-		if (str.size <= max_size) {
+		if (char_vector_size(&str) <= max_size) {
 			err = drgn_memory_reader_read(&prog->reader, c, address,
 						      1, physical);
-			if (err) {
-				char_vector_deinit(&str);
+			if (err)
 				return err;
-			}
 			if (!*c)
 				break;
 		} else {
@@ -1615,7 +1686,7 @@ drgn_program_read_c_string(struct drgn_program *prog, uint64_t address,
 		address++;
 	}
 	char_vector_shrink_to_fit(&str);
-	*ret = str.data;
+	char_vector_steal(&str, ret, NULL);
 	return NULL;
 }
 
@@ -1705,13 +1776,9 @@ bool drgn_program_find_symbol_by_address_internal(struct drgn_program *prog,
 						  struct drgn_symbol *ret)
 {
 	if (!module) {
-		if (prog->dbinfo) {
-			module = dwfl_addrmodule(prog->dbinfo->dwfl, address);
-			if (!module)
-				return false;
-		} else {
+		module = dwfl_addrmodule(prog->dbinfo.dwfl, address);
+		if (!module)
 			return false;
-		}
 	}
 
 	GElf_Off offset;
@@ -1749,7 +1816,7 @@ drgn_program_find_symbol_by_address(struct drgn_program *prog, uint64_t address,
 	return NULL;
 }
 
-DEFINE_VECTOR(symbolp_vector, struct drgn_symbol *)
+DEFINE_VECTOR(symbolp_vector, struct drgn_symbol *);
 
 enum {
 	SYMBOLS_SEARCH_NAME = (1 << 0),
@@ -1815,11 +1882,6 @@ symbols_search(struct drgn_program *prog, struct symbols_search_arg *arg,
 {
 	struct drgn_error *err;
 
-	if (!prog->dbinfo) {
-		return drgn_error_create(DRGN_ERROR_MISSING_DEBUG_INFO,
-					 "could not find matching symbols");
-	}
-
 	symbolp_vector_init(&arg->results);
 
 	/*
@@ -1828,24 +1890,23 @@ symbols_search(struct drgn_program *prog, struct symbols_search_arg *arg,
 	 */
 	err = NULL;
 	if (arg->flags & SYMBOLS_SEARCH_ADDRESS) {
-		Dwfl_Module *module = dwfl_addrmodule(prog->dbinfo->dwfl,
+		Dwfl_Module *module = dwfl_addrmodule(prog->dbinfo.dwfl,
 						      arg->address);
 		if (module && symbols_search_cb(module, NULL, NULL, 0, arg))
 			err = &drgn_enomem;
 	} else {
-		if (dwfl_getmodules(prog->dbinfo->dwfl, symbols_search_cb, arg,
+		if (dwfl_getmodules(prog->dbinfo.dwfl, symbols_search_cb, arg,
 				    0))
 			err = &drgn_enomem;
 	}
 
 	if (err) {
-		for (size_t i = 0; i < arg->results.size; i++)
-			drgn_symbol_destroy(arg->results.data[i]);
+		vector_for_each(symbolp_vector, symbolp, &arg->results)
+			drgn_symbol_destroy(*symbolp);
 		symbolp_vector_deinit(&arg->results);
 	} else {
 		symbolp_vector_shrink_to_fit(&arg->results);
-		*count_ret = arg->results.size;
-		*syms_ret = arg->results.data;
+		symbolp_vector_steal(&arg->results, syms_ret, count_ret);
 	}
 	return err;
 }
@@ -1937,17 +1998,14 @@ drgn_program_find_symbol_by_name(struct drgn_program *prog,
 	struct find_symbol_by_name_arg arg = {
 		.name = name,
 	};
-	if (prog->dbinfo) {
-		dwfl_getmodules(prog->dbinfo->dwfl, find_symbol_by_name_cb,
-				&arg, 0);
-		if (arg.found) {
-			struct drgn_symbol *sym = malloc(sizeof(*sym));
-			if (!sym)
-				return &drgn_enomem;
-			drgn_symbol_from_elf(name, arg.addr, &arg.sym, sym);
-			*ret = sym;
-			return NULL;
-		}
+	dwfl_getmodules(prog->dbinfo.dwfl, find_symbol_by_name_cb, &arg, 0);
+	if (arg.found) {
+		struct drgn_symbol *sym = malloc(sizeof(*sym));
+		if (!sym)
+			return &drgn_enomem;
+		drgn_symbol_from_elf(name, arg.addr, &arg.sym, sym);
+		*ret = sym;
+		return NULL;
 	}
 	return drgn_error_format(DRGN_ERROR_LOOKUP,
 				 "could not find symbol with name '%s'%s", name,
@@ -1971,3 +2029,38 @@ drgn_program_element_info(struct drgn_program *prog, struct drgn_type *type,
 	ret->qualified_type = drgn_type_type(underlying_type);
 	return drgn_type_bit_size(ret->qualified_type.type, &ret->bit_size);
 }
+
+LIBDRGN_PUBLIC void
+drgn_program_set_blocking_callback(struct drgn_program *prog,
+				   drgn_program_begin_blocking_fn *begin_callback,
+				   drgn_program_end_blocking_fn *end_callback,
+				   void *callback_arg)
+{
+	prog->begin_blocking_fn = begin_callback;
+	prog->end_blocking_fn = end_callback;
+	prog->blocking_arg = callback_arg;
+}
+
+LIBDRGN_PUBLIC void
+drgn_program_get_blocking_callback(struct drgn_program *prog,
+				   drgn_program_begin_blocking_fn **begin_callback_ret,
+				   drgn_program_end_blocking_fn **end_callback_ret,
+				   void **callback_arg_ret)
+{
+	*begin_callback_ret = prog->begin_blocking_fn;
+	*end_callback_ret = prog->end_blocking_fn;
+	*callback_arg_ret = prog->blocking_arg;
+}
+
+void *drgn_program_begin_blocking(struct drgn_program *prog)
+{
+	if (!prog->begin_blocking_fn)
+		return NULL;
+	return prog->begin_blocking_fn(prog, prog->blocking_arg);
+}
+
+void drgn_program_end_blocking(struct drgn_program *prog, void *state)
+{
+	if (prog->end_blocking_fn)
+		prog->end_blocking_fn(prog, prog->blocking_arg, state);
+}
