@@ -19,12 +19,14 @@
 #include <libkdumpfile/kdumpfile.h>
 #endif
 
+#include "debug_info.h"
 #include "drgn.h"
 #include "hash_table.h"
 #include "language.h"
 #include "memory_reader.h"
 #include "object_index.h"
 #include "platform.h"
+#include "pp.h"
 #include "type.h"
 #include "vector.h"
 
@@ -49,9 +51,9 @@ struct drgn_thread {
 	struct drgn_object object;
 };
 
-DEFINE_VECTOR_TYPE(drgn_typep_vector, struct drgn_type *)
-DEFINE_VECTOR_TYPE(drgn_prstatus_vector, struct nstring)
-DEFINE_HASH_TABLE_TYPE(drgn_thread_set, struct drgn_thread)
+DEFINE_VECTOR_TYPE(drgn_typep_vector, struct drgn_type *);
+DEFINE_VECTOR_TYPE(drgn_prstatus_vector, struct nstring);
+DEFINE_HASH_TABLE_TYPE(drgn_thread_set, struct drgn_thread);
 
 struct drgn_program {
 	/** @privatesection */
@@ -106,7 +108,7 @@ struct drgn_program {
 	 * Debugging information.
 	 */
 	struct drgn_object_index oindex;
-	struct drgn_debug_info *dbinfo;
+	struct drgn_debug_info dbinfo;
 
 	/*
 	 * Program information.
@@ -167,6 +169,10 @@ struct drgn_program {
 		bool pgtable_l5_enabled;
 		/** PAGE_SHIFT of the kernel (derived from PAGE_SIZE). */
 		int page_shift;
+
+		/** The original vmcoreinfo data, to expose as an object */
+		char *raw;
+		size_t raw_size;
 	} vmcoreinfo;
 	/*
 	 * Difference between a virtual address in the direct mapping and the
@@ -184,6 +190,20 @@ struct drgn_program {
 	bool in_address_translation;
 	/* Whether @ref drgn_program::direct_mapping_offset has been cached. */
 	bool direct_mapping_offset_cached;
+
+	/*
+	 * Logging.
+	 */
+	drgn_log_fn *log_fn;
+	void *log_arg;
+	enum drgn_log_level log_level;
+
+	/*
+	 * Blocking callbacks.
+	 */
+	drgn_program_begin_blocking_fn *begin_blocking_fn;
+	drgn_program_end_blocking_fn *end_blocking_fn;
+	void *blocking_arg;
 };
 
 /** Initialize a @ref drgn_program. */
@@ -207,6 +227,13 @@ void drgn_program_set_platform(struct drgn_program *prog,
 struct drgn_error *drgn_program_init_core_dump(struct drgn_program *prog,
 					       const char *path);
 
+/**
+ * Implement @ref drgn_program_from_core_dump_fd() on an initialized @ref
+ * drgn_program.
+ */
+struct drgn_error *drgn_program_init_core_dump_fd(struct drgn_program *prog,
+						  int fd);
+
 /**
  * Implement @ref drgn_program_from_kernel() on an initialized @ref
  * drgn_program.
@@ -218,6 +245,11 @@ struct drgn_error *drgn_program_init_kernel(struct drgn_program *prog);
  */
 struct drgn_error *drgn_program_init_pid(struct drgn_program *prog, pid_t pid);
 
+struct drgn_error *
+drgn_program_add_object_finder_impl(struct drgn_program *prog,
+				    struct drgn_object_finder *finder,
+				    drgn_object_find_fn fn, void *arg);
+
 static inline struct drgn_error *
 drgn_program_is_little_endian(struct drgn_program *prog, bool *ret)
 {
@@ -289,12 +321,10 @@ void drgn_thread_deinit(struct drgn_thread *thread);
  *
  * @param[out] ret Returned note data. If not found, <tt>ret->str</tt> is set to
  * @c NULL and <tt>ret->len</tt> is set to zero.
- * @param[out] tid_ret Returned thread ID of note.
  */
 struct drgn_error *drgn_program_find_prstatus_by_cpu(struct drgn_program *prog,
 						     uint32_t cpu,
-						     struct nstring *ret,
-						     uint32_t *tid_ret);
+						     struct nstring *ret);
 
 /**
  * Find the @c NT_PRSTATUS note for the given thread ID.
@@ -334,6 +364,50 @@ bool drgn_program_find_symbol_by_address_internal(struct drgn_program *prog,
 						  Dwfl_Module *module,
 						  struct drgn_symbol *ret);
 
+/**
+ * Call before a blocking (I/O or long-running) operation.
+ *
+ * Must be paired with @ref drgn_program_end_blocking().
+ *
+ * @return Opaque pointer to pass to @ref drgn_program_end_blocking().
+ */
+void *drgn_program_begin_blocking(struct drgn_program *prog);
+
+/**
+ * Call after a blocking (I/O or long-running) operation.
+ *
+ * @param[in] state Return value of @ref drgn_program_begin_blocking().
+ */
+void drgn_program_end_blocking(struct drgn_program *prog, void *state);
+
+struct drgn_blocking_guard_struct {
+	struct drgn_program *prog;
+	void *state;
+};
+
+static inline struct drgn_blocking_guard_struct
+drgn_blocking_guard_init(struct drgn_program *prog)
+{
+	return (struct drgn_blocking_guard_struct){
+		prog, drgn_program_begin_blocking(prog),
+	};
+}
+
+static inline void
+drgn_blocking_guard_cleanup(struct drgn_blocking_guard_struct *guard)
+{
+	drgn_program_end_blocking(guard->prog, guard->state);
+}
+
+/**
+ * Scope guard that wraps @ref drgn_program_begin_blocking() and @ref
+ * drgn_program_end_blocking().
+ */
+#define drgn_blocking_guard(prog)						\
+	struct drgn_blocking_guard_struct PP_UNIQUE(guard)			\
+	__attribute__((__cleanup__(drgn_blocking_guard_cleanup), __unused__)) =	\
+	drgn_blocking_guard_init(prog)
+
 /**
  * @}
  * @}
