@@ -11,10 +11,13 @@
 #include "structmember.h"
 
 #include "docstrings.h"
+#include "../cleanup.h"
 #include "../drgn.h"
 // IWYU pragma: end_exports
 
 #include "../hash_table.h"
+#include "../hash_table.h"
+#include "../pp.h"
 #include "../program.h"
 
 /* These were added in Python 3.7. */
@@ -73,6 +76,23 @@
 		Py_RETURN_FALSE;	\
 } while (0)
 
+static inline void pydecrefp(void *p)
+{
+	Py_XDECREF(*(PyObject **)p);
+}
+
+/** Scope guard that wraps PyGILState_Ensure() and PyGILState_Release(). */
+#define PyGILState_guard()						\
+	__attribute__((__cleanup__(PyGILState_Releasep), __unused__))	\
+	PyGILState_STATE PP_UNIQUE(gstate) = PyGILState_Ensure()
+static inline void PyGILState_Releasep(PyGILState_STATE *gstatep)
+{
+	PyGILState_Release(*gstatep);
+}
+
+/** Call @c Py_XDECREF() when the variable goes out of scope. */
+#define _cleanup_pydecref_ _cleanup_(pydecrefp)
+
 typedef struct {
 	PyObject_HEAD
 	struct drgn_object obj;
@@ -111,7 +131,7 @@ typedef struct {
 	struct drgn_platform *platform;
 } Platform;
 
-DEFINE_HASH_SET_TYPE(pyobjectp_set, PyObject *)
+DEFINE_HASH_SET_TYPE(pyobjectp_set, PyObject *);
 
 typedef struct {
 	PyObject_HEAD
@@ -224,6 +244,7 @@ extern PyObject *ObjectAbsentError;
 extern PyObject *OutOfBoundsError;
 
 int add_module_constants(PyObject *m);
+int init_logging(void);
 
 bool set_drgn_in_python(void);
 void clear_drgn_in_python(void);
@@ -309,7 +330,9 @@ struct index_arg {
 int index_converter(PyObject *o, void *p);
 
 struct path_arg {
+	bool allow_fd;
 	bool allow_none;
+	int fd;
 	char *path;
 	Py_ssize_t length;
 	PyObject *object;
@@ -333,23 +356,19 @@ PyObject *drgnpy_linux_helper_follow_phys(PyObject *self, PyObject *args,
 					  PyObject *kwds);
 DrgnObject *drgnpy_linux_helper_per_cpu_ptr(PyObject *self, PyObject *args,
 					    PyObject *kwds);
-DrgnObject *drgnpy_linux_helper_idle_task(PyObject *self, PyObject *args,
-					  PyObject *kwds);
+DrgnObject *drgnpy_linux_helper_cpu_curr(PyObject *self, PyObject *args);
+DrgnObject *drgnpy_linux_helper_idle_task(PyObject *self, PyObject *args);
 PyObject *drgnpy_linux_helper_task_cpu(PyObject *self, PyObject *args,
 				       PyObject *kwds);
 DrgnObject *drgnpy_linux_helper_xa_load(PyObject *self, PyObject *args,
 					PyObject *kwds);
 DrgnObject *drgnpy_linux_helper_idr_find(PyObject *self, PyObject *args,
 					 PyObject *kwds);
-DrgnObject *drgnpy_linux_helper_find_pid(PyObject *self, PyObject *args,
-					 PyObject *kwds);
+DrgnObject *drgnpy_linux_helper_find_pid(PyObject *self, PyObject *args);
 DrgnObject *drgnpy_linux_helper_pid_task(PyObject *self, PyObject *args,
 					 PyObject *kwds);
-DrgnObject *drgnpy_linux_helper_find_task(PyObject *self, PyObject *args,
-					  PyObject *kwds);
-PyObject *drgnpy_linux_helper_kaslr_offset(PyObject *self, PyObject *args,
-					   PyObject *kwds);
-PyObject *drgnpy_linux_helper_pgtable_l5_enabled(PyObject *self, PyObject *args,
-						 PyObject *kwds);
+DrgnObject *drgnpy_linux_helper_find_task(PyObject *self, PyObject *args);
+PyObject *drgnpy_linux_helper_kaslr_offset(PyObject *self, PyObject *arg);
+PyObject *drgnpy_linux_helper_pgtable_l5_enabled(PyObject *self, PyObject *arg);
 
 #endif /* DRGNPY_H */
