@@ -28,8 +28,6 @@ PyObject *drgnpy_linux_helper_read_vm(PyObject *self, PyObject *args,
 	struct index_arg pgtable = {};
 	struct index_arg address = {};
 	Py_ssize_t size;
-	PyObject *buf;
-
 	if (!PyArg_ParseTupleAndKeywords(args, kwds, "O!O&O&n:read_vm",
 					 keywords, &Program_type, &prog,
 					 index_converter, &pgtable,
@@ -40,16 +38,14 @@ PyObject *drgnpy_linux_helper_read_vm(PyObject *self, PyObject *args,
 		PyErr_SetString(PyExc_ValueError, "negative size");
 		return NULL;
 	}
-	buf = PyBytes_FromStringAndSize(NULL, size);
+	_cleanup_pydecref_ PyObject *buf = PyBytes_FromStringAndSize(NULL, size);
 	if (!buf)
 		return NULL;
 	err = linux_helper_read_vm(&prog->prog, pgtable.uvalue, address.uvalue,
 				   PyBytes_AS_STRING(buf), size);
-	if (err) {
-		Py_DECREF(buf);
+	if (err)
 		return set_drgn_error(err);
-	}
-	return buf;
+	return_ptr(buf);
 }
 
 PyObject *drgnpy_linux_helper_follow_phys(PyObject *self, PyObject *args,
@@ -81,45 +77,54 @@ DrgnObject *drgnpy_linux_helper_per_cpu_ptr(PyObject *self, PyObject *args,
 	struct drgn_error *err;
 	DrgnObject *ptr;
 	struct index_arg cpu = {};
-
 	if (!PyArg_ParseTupleAndKeywords(args, kwds, "O!O&:per_cpu_ptr",
 					 keywords, &DrgnObject_type, &ptr,
 					 index_converter, &cpu))
 		return NULL;
 
-	DrgnObject *res = DrgnObject_alloc(DrgnObject_prog(ptr));
+	_cleanup_pydecref_ DrgnObject *res = DrgnObject_alloc(DrgnObject_prog(ptr));
 	if (!res)
 		return NULL;
 	err = linux_helper_per_cpu_ptr(&res->obj, &ptr->obj, cpu.uvalue);
-	if (err) {
-		Py_DECREF(res);
+	if (err)
 		return set_drgn_error(err);
-	}
-	return res;
+	return_ptr(res);
 }
 
-DrgnObject *drgnpy_linux_helper_idle_task(PyObject *self, PyObject *args,
-					  PyObject *kwds)
+DrgnObject *drgnpy_linux_helper_cpu_curr(PyObject *self, PyObject *args)
 {
-	static char *keywords[] = {"prog", "cpu", NULL};
 	struct drgn_error *err;
 	Program *prog;
 	struct index_arg cpu = {};
-
-	if (!PyArg_ParseTupleAndKeywords(args, kwds, "O!O&:idle_task", keywords,
-					 &Program_type, &prog, index_converter,
-					 &cpu))
+	if (!PyArg_ParseTuple(args, "O!O&:cpu_curr", &Program_type, &prog,
+			      index_converter, &cpu))
 		return NULL;
 
-	DrgnObject *res = DrgnObject_alloc(prog);
+	_cleanup_pydecref_ DrgnObject *res = DrgnObject_alloc(prog);
+	if (!res)
+		return NULL;
+	err = linux_helper_cpu_curr(&res->obj, cpu.uvalue);
+	if (err)
+		return set_drgn_error(err);
+	return_ptr(res);
+}
+
+DrgnObject *drgnpy_linux_helper_idle_task(PyObject *self, PyObject *args)
+{
+	struct drgn_error *err;
+	Program *prog;
+	struct index_arg cpu = {};
+	if (!PyArg_ParseTuple(args, "O!O&:idle_task", &Program_type, &prog,
+			      index_converter, &cpu))
+		return NULL;
+
+	_cleanup_pydecref_ DrgnObject *res = DrgnObject_alloc(prog);
 	if (!res)
 		return NULL;
 	err = linux_helper_idle_task(&res->obj, cpu.uvalue);
-	if (err) {
-		Py_DECREF(res);
+	if (err)
 		return set_drgn_error(err);
-	}
-	return res;
+	return_ptr(res);
 }
 
 PyObject *drgnpy_linux_helper_task_cpu(PyObject *self, PyObject *args,
@@ -151,15 +156,13 @@ DrgnObject *drgnpy_linux_helper_xa_load(PyObject *self, PyObject *args,
 					 &index))
 		return NULL;
 
-	DrgnObject *res = DrgnObject_alloc(DrgnObject_prog(xa));
+	_cleanup_pydecref_ DrgnObject *res = DrgnObject_alloc(DrgnObject_prog(xa));
 	if (!res)
 		return NULL;
 	err = linux_helper_xa_load(&res->obj, &xa->obj, index.uvalue);
-	if (err) {
-		Py_DECREF(res);
+	if (err)
 		return set_drgn_error(err);
-	}
-	return res;
+	return_ptr(res);
 }
 
 DrgnObject *drgnpy_linux_helper_idr_find(PyObject *self, PyObject *args,
@@ -169,99 +172,38 @@ DrgnObject *drgnpy_linux_helper_idr_find(PyObject *self, PyObject *args,
 	struct drgn_error *err;
 	DrgnObject *idr;
 	struct index_arg id = {};
-	DrgnObject *res;
-
 	if (!PyArg_ParseTupleAndKeywords(args, kwds, "O!O&:idr_find", keywords,
 					 &DrgnObject_type, &idr,
 					 index_converter, &id))
 		return NULL;
 
-	res = DrgnObject_alloc(DrgnObject_prog(idr));
+	_cleanup_pydecref_ DrgnObject *res =
+		DrgnObject_alloc(DrgnObject_prog(idr));
 	if (!res)
 		return NULL;
 	err = linux_helper_idr_find(&res->obj, &idr->obj, id.uvalue);
-	if (err) {
-		Py_DECREF(res);
+	if (err)
 		return set_drgn_error(err);
-	}
-	return res;
+	return_ptr(res);
 }
 
-struct prog_or_ns_arg {
-	Program *prog;
-	struct drgn_object *ns;
-	struct drgn_object tmp;
-};
-
-static void prog_or_ns_cleanup(struct prog_or_ns_arg *arg)
+DrgnObject *drgnpy_linux_helper_find_pid(PyObject *self, PyObject *args)
 {
-	if (arg->ns == &arg->tmp)
-		drgn_object_deinit(arg->ns);
-}
-
-static int prog_or_pid_ns_converter(PyObject *o, void *p)
-{
-	struct prog_or_ns_arg *arg = p;
-
-	if (!o) {
-		prog_or_ns_cleanup(arg);
-		return 1;
-	}
-
-	if (PyObject_TypeCheck(o, &Program_type)) {
-		struct drgn_error *err;
-
-		arg->prog = (Program *)o;
-		arg->ns = &arg->tmp;
-		drgn_object_init(arg->ns, &arg->prog->prog);
-		err = drgn_program_find_object(&arg->prog->prog, "init_pid_ns",
-					       NULL, DRGN_FIND_OBJECT_ANY,
-					       arg->ns);
-		if (!err)
-			err = drgn_object_address_of(arg->ns, arg->ns);
-		if (err) {
-			drgn_object_deinit(arg->ns);
-			set_drgn_error(err);
-			return 0;
-		}
-	} else if (PyObject_TypeCheck(o, &DrgnObject_type)) {
-		arg->prog = DrgnObject_prog((DrgnObject *)o);
-		arg->ns = &((DrgnObject *)o)->obj;
-	} else {
-		PyErr_Format(PyExc_TypeError,
-			     "expected Program or Object, not %s",
-			     Py_TYPE(o)->tp_name);
-		return 0;
-	}
-	return Py_CLEANUP_SUPPORTED;
-}
-
-DrgnObject *drgnpy_linux_helper_find_pid(PyObject *self, PyObject *args,
-					  PyObject *kwds)
-{
-	static char *keywords[] = {"prog_or_ns", "pid", NULL};
 	struct drgn_error *err;
-	struct prog_or_ns_arg prog_or_ns;
+	DrgnObject *ns;
 	struct index_arg pid = {};
-	DrgnObject *res;
-
-	if (!PyArg_ParseTupleAndKeywords(args, kwds, "O&O&:find_pid", keywords,
-					 &prog_or_pid_ns_converter, &prog_or_ns,
-					 index_converter, &pid))
+	if (!PyArg_ParseTuple(args, "O!O&:find_pid", &DrgnObject_type, &ns,
+			      index_converter, &pid))
 		return NULL;
 
-	res = DrgnObject_alloc(prog_or_ns.prog);
+	_cleanup_pydecref_ DrgnObject *res =
+		DrgnObject_alloc(DrgnObject_prog(ns));
 	if (!res)
-		goto out;
-	err = linux_helper_find_pid(&res->obj, prog_or_ns.ns, pid.uvalue);
-	if (err) {
-		Py_DECREF(res);
-		set_drgn_error(err);
-		res = NULL;
-	}
-out:
-	prog_or_ns_cleanup(&prog_or_ns);
-	return res;
+		return NULL;
+	err = linux_helper_find_pid(&res->obj, &ns->obj, pid.uvalue);
+	if (err)
+		return set_drgn_error(err);
+	return_ptr(res);
 }
 
 DrgnObject *drgnpy_linux_helper_pid_task(PyObject *self, PyObject *args,
@@ -271,77 +213,60 @@ DrgnObject *drgnpy_linux_helper_pid_task(PyObject *self, PyObject *args,
 	struct drgn_error *err;
 	DrgnObject *pid;
 	struct index_arg pid_type = {};
-	DrgnObject *res;
-
 	if (!PyArg_ParseTupleAndKeywords(args, kwds, "O!O&:pid_task", keywords,
 					 &DrgnObject_type, &pid,
 					 index_converter, &pid_type))
 		return NULL;
 
-	res = DrgnObject_alloc(DrgnObject_prog(pid));
+	_cleanup_pydecref_ DrgnObject *res = DrgnObject_alloc(DrgnObject_prog(pid));
 	if (!res)
 		return NULL;
 	err = linux_helper_pid_task(&res->obj, &pid->obj, pid_type.uvalue);
-	if (err) {
-		Py_DECREF(res);
+	if (err)
 		return set_drgn_error(err);
-	}
-	return res;
+	return_ptr(res);
 }
 
-DrgnObject *drgnpy_linux_helper_find_task(PyObject *self, PyObject *args,
-					  PyObject *kwds)
+DrgnObject *drgnpy_linux_helper_find_task(PyObject *self, PyObject *args)
 {
-	static char *keywords[] = {"ns", "pid", NULL};
 	struct drgn_error *err;
-	struct prog_or_ns_arg prog_or_ns;
+	DrgnObject *ns;
 	struct index_arg pid = {};
-	DrgnObject *res;
-
-	if (!PyArg_ParseTupleAndKeywords(args, kwds, "O&O&:find_task", keywords,
-					 &prog_or_pid_ns_converter, &prog_or_ns,
-					 index_converter, &pid))
+	if (!PyArg_ParseTuple(args, "O!O&:find_task", &DrgnObject_type, &ns,
+			      index_converter, &pid))
 		return NULL;
 
-	res = DrgnObject_alloc(prog_or_ns.prog);
+	_cleanup_pydecref_ DrgnObject *res =
+		DrgnObject_alloc(DrgnObject_prog(ns));
 	if (!res)
-		goto out;
-	err = linux_helper_find_task(&res->obj, prog_or_ns.ns, pid.uvalue);
-	if (err) {
-		Py_DECREF(res);
-		set_drgn_error(err);
-		res = NULL;
-	}
-out:
-	prog_or_ns_cleanup(&prog_or_ns);
-	return res;
+		return NULL;
+	err = linux_helper_find_task(&res->obj, &ns->obj, pid.uvalue);
+	if (err)
+		return set_drgn_error(err);
+	return_ptr(res);
 }
 
-PyObject *drgnpy_linux_helper_kaslr_offset(PyObject *self, PyObject *args,
-					   PyObject *kwds)
+PyObject *drgnpy_linux_helper_kaslr_offset(PyObject *self, PyObject *arg)
 
 {
-	static char *keywords[] = {"prog", NULL};
-	Program *prog;
-	if (!PyArg_ParseTupleAndKeywords(args, kwds, "O!:kaslr_offset",
-					 keywords, &Program_type, &prog))
-		return NULL;
-
+	if (!PyObject_TypeCheck(arg, &Program_type)) {
+		return PyErr_Format(PyExc_TypeError, "expected Program, not %s",
+				    Py_TYPE(arg)->tp_name);
+	}
+	Program *prog = (Program *)arg;
 	if (!(prog->prog.flags & DRGN_PROGRAM_IS_LINUX_KERNEL))
 		return PyErr_Format(PyExc_ValueError, "not Linux kernel");
 	return PyLong_FromUint64(prog->prog.vmcoreinfo.kaslr_offset);
 }
 
-PyObject *drgnpy_linux_helper_pgtable_l5_enabled(PyObject *self, PyObject *args,
-						 PyObject *kwds)
+PyObject *drgnpy_linux_helper_pgtable_l5_enabled(PyObject *self, PyObject *arg)
 
 {
-	static char *keywords[] = {"prog", NULL};
-	Program *prog;
-	if (!PyArg_ParseTupleAndKeywords(args, kwds, "O!:pgtable_l5_enabled",
-					 keywords, &Program_type, &prog))
-		return NULL;
-
+	if (!PyObject_TypeCheck(arg, &Program_type)) {
+		return PyErr_Format(PyExc_TypeError, "expected Program, not %s",
+				    Py_TYPE(arg)->tp_name);
+	}
+	Program *prog = (Program *)arg;
 	if (!(prog->prog.flags & DRGN_PROGRAM_IS_LINUX_KERNEL))
 		return PyErr_Format(PyExc_ValueError, "not Linux kernel");
 	Py_RETURN_BOOL(prog->prog.vmcoreinfo.pgtable_l5_enabled);
