@@ -27,9 +27,37 @@ static int add_type(PyObject *module, PyTypeObject *type)
 }
 
 PyObject *MissingDebugInfoError;
+static PyObject *NoDefaultProgramError;
 PyObject *ObjectAbsentError;
 PyObject *OutOfBoundsError;
 
+static _Thread_local PyObject *default_prog;
+
+static PyObject *get_default_prog(PyObject *self, PyObject *_)
+{
+	if (!default_prog) {
+		PyErr_SetString(NoDefaultProgramError, "no default program");
+		return NULL;
+	}
+	Py_INCREF(default_prog);
+	return default_prog;
+}
+
+static PyObject *set_default_prog(PyObject *self, PyObject *arg)
+{
+	if (arg == Py_None) {
+		Py_CLEAR(default_prog);
+	} else if (PyObject_TypeCheck(arg, &Program_type)) {
+		Py_INCREF(arg);
+		Py_XSETREF(default_prog, arg);
+	} else {
+		PyErr_SetString(PyExc_TypeError,
+				"prog must be Program or None");
+		return NULL;
+	}
+	Py_RETURN_NONE;
+}
+
 static PyObject *filename_matches(PyObject *self, PyObject *args,
 				  PyObject *kwds)
 {
@@ -102,6 +130,10 @@ static PyObject *offsetof_(PyObject *self, PyObject *args, PyObject *kwds)
 }
 
 static PyMethodDef drgn_methods[] = {
+	{"get_default_prog", get_default_prog, METH_NOARGS,
+	 drgn_get_default_prog_DOC},
+	{"set_default_prog", set_default_prog, METH_O,
+	 drgn_set_default_prog_DOC},
 	{"filename_matches", (PyCFunction)filename_matches,
 	 METH_VARARGS | METH_KEYWORDS, drgn_filename_matches_DOC},
 	{"NULL", (PyCFunction)DrgnObject_NULL, METH_VARARGS | METH_KEYWORDS,
@@ -131,8 +163,10 @@ static PyMethodDef drgn_methods[] = {
 	{"_linux_helper_per_cpu_ptr",
 	 (PyCFunction)drgnpy_linux_helper_per_cpu_ptr,
 	 METH_VARARGS | METH_KEYWORDS},
+	{"_linux_helper_cpu_curr", (PyCFunction)drgnpy_linux_helper_cpu_curr,
+	 METH_VARARGS},
 	{"_linux_helper_idle_task", (PyCFunction)drgnpy_linux_helper_idle_task,
-	 METH_VARARGS | METH_KEYWORDS},
+	 METH_VARARGS},
 	{"_linux_helper_task_cpu", (PyCFunction)drgnpy_linux_helper_task_cpu,
 	 METH_VARARGS | METH_KEYWORDS},
 	{"_linux_helper_xa_load",
@@ -141,17 +175,15 @@ static PyMethodDef drgn_methods[] = {
 	{"_linux_helper_idr_find", (PyCFunction)drgnpy_linux_helper_idr_find,
 	 METH_VARARGS | METH_KEYWORDS},
 	{"_linux_helper_find_pid", (PyCFunction)drgnpy_linux_helper_find_pid,
-	 METH_VARARGS | METH_KEYWORDS},
+	 METH_VARARGS},
 	{"_linux_helper_pid_task", (PyCFunction)drgnpy_linux_helper_pid_task,
 	 METH_VARARGS | METH_KEYWORDS},
 	{"_linux_helper_find_task", (PyCFunction)drgnpy_linux_helper_find_task,
-	 METH_VARARGS | METH_KEYWORDS},
-	{"_linux_helper_kaslr_offset",
-	 (PyCFunction)drgnpy_linux_helper_kaslr_offset,
-	 METH_VARARGS | METH_KEYWORDS},
+	 METH_VARARGS},
+	{"_linux_helper_kaslr_offset", drgnpy_linux_helper_kaslr_offset,
+	 METH_O},
 	{"_linux_helper_pgtable_l5_enabled",
-	 (PyCFunction)drgnpy_linux_helper_pgtable_l5_enabled,
-	 METH_VARARGS | METH_KEYWORDS},
+	 drgnpy_linux_helper_pgtable_l5_enabled, METH_O},
 	{},
 };
 
@@ -163,74 +195,75 @@ static struct PyModuleDef drgnmodule = {
 	drgn_methods,
 };
 
-/*
- * These are for type checking and aren't strictly required at runtime, but
- * adding them anyways results in better pydoc output and saves us from fiddling
- * with typing.TYPE_CHECKING/forward references.
- */
+// These are for type checking and aren't strictly required at runtime, but
+// adding them anyways results in better pydoc output and saves us from fiddling
+// with typing.TYPE_CHECKING/forward references.
 static int add_type_aliases(PyObject *m)
 {
-	/*
-	 * This should be a subclass of typing.Protocol, but that is only
-	 * available since Python 3.8.
-	 */
+	_cleanup_pydecref_ PyObject *os_module = PyImport_ImportModule("os");
+	if (!os_module)
+		return -1;
+	_cleanup_pydecref_ PyObject *os_PathLike =
+		PyObject_GetAttrString(os_module, "PathLike");
+	if (!os_PathLike)
+		return -1;
+
+	_cleanup_pydecref_ PyObject *typing_module =
+		PyImport_ImportModule("typing");
+	if (!typing_module)
+		return -1;
+	_cleanup_pydecref_ PyObject *typing_Union =
+		PyObject_GetAttrString(typing_module, "Union");
+	if (!typing_Union)
+		return -1;
+
+	// This should be a subclass of typing.Protocol, but that is only
+	// available since Python 3.8.
 	PyObject *IntegerLike = PyType_FromSpec(&(PyType_Spec){
 		.name = "_drgn.IntegerLike",
 		.flags = Py_TPFLAGS_DEFAULT,
 		.slots = (PyType_Slot []){{0, NULL}},
 	});
-       if (!IntegerLike)
-	       return -1;
-       if (PyModule_AddObject(m, "IntegerLike", IntegerLike) == -1) {
-	       Py_DECREF(IntegerLike);
-	       return -1;
-       }
+	if (!IntegerLike)
+		return -1;
+	if (PyModule_AddObject(m, "IntegerLike", IntegerLike) == -1) {
+		Py_DECREF(IntegerLike);
+		return -1;
+	}
 
-       PyObject *os_module = PyImport_ImportModule("os");
-       if (!os_module)
-	       return -1;
-       PyObject *os_PathLike = PyObject_GetAttrString(os_module, "PathLike");
-       Py_DECREF(os_module);
-       if (!os_PathLike)
-	       return -1;
-       PyObject *item = Py_BuildValue("OOO", &PyUnicode_Type, &PyBytes_Type,
-				      os_PathLike);
-       Py_DECREF(os_PathLike);
-       if (!item)
-	       return -1;
+	_cleanup_pydecref_ PyObject *item =
+		Py_BuildValue("OOO", &PyUnicode_Type, &PyBytes_Type,
+			      os_PathLike);
+	if (!item)
+		return -1;
+	PyObject *Path = PyObject_GetItem(typing_Union, item);
+	if (!Path)
+		return -1;
+	if (PyModule_AddObject(m, "Path", Path) == -1) {
+		Py_DECREF(Path);
+		return -1;
+	}
 
-       PyObject *typing_module = PyImport_ImportModule("typing");
-       if (!typing_module) {
-	       Py_DECREF(item);
-	       return -1;
-       }
-       PyObject *typing_Union = PyObject_GetAttrString(typing_module, "Union");
-       Py_DECREF(typing_module);
-       if (!typing_Union) {
-	       Py_DECREF(item);
-	       return -1;
-       }
-
-       PyObject *Path = PyObject_GetItem(typing_Union, item);
-       Py_DECREF(typing_Union);
-       Py_DECREF(item);
-       if (!Path)
-	       return -1;
-       if (PyModule_AddObject(m, "Path", Path) == -1) {
-	       Py_DECREF(Path);
-	       return -1;
-       }
-       return 0;
+	return 0;
 }
 
+PyMODINIT_FUNC PyInit__drgn(void); // Silence -Wmissing-prototypes.
 DRGNPY_PUBLIC PyMODINIT_FUNC PyInit__drgn(void)
 {
 	PyObject *m = PyModule_Create(&drgnmodule);
 	if (!m)
 		return NULL;
 
+	#define add_new_exception(m, name) ({					\
+		name = PyErr_NewExceptionWithDoc("_drgn." #name,		\
+						 drgn_##name##_DOC, NULL,	\
+						 NULL);				\
+		if (name && PyModule_AddObject(m, #name, name))			\
+			Py_CLEAR(name);						\
+		!name;								\
+	})
+
 	if (add_module_constants(m) ||
-	    add_type_aliases(m) ||
 	    add_type(m, &Language_type) || add_languages() ||
 	    add_type(m, &DrgnObject_type) ||
 	    PyType_Ready(&ObjectIterator_type) ||
@@ -246,46 +279,19 @@ DRGNPY_PUBLIC PyMODINIT_FUNC PyInit__drgn(void)
 	    add_type(m, &TypeEnumerator_type) ||
 	    add_type(m, &TypeMember_type) ||
 	    add_type(m, &TypeParameter_type) ||
-	    add_type(m, &TypeTemplateParameter_type))
+	    add_type(m, &TypeTemplateParameter_type) ||
+	    add_new_exception(m, MissingDebugInfoError) ||
+	    add_new_exception(m, NoDefaultProgramError) ||
+	    add_new_exception(m, ObjectAbsentError) ||
+	    add_new_exception(m, OutOfBoundsError) ||
+	    add_type_aliases(m) ||
+	    init_logging())
 		goto err;
 
 	FaultError_type.tp_base = (PyTypeObject *)PyExc_Exception;
 	if (add_type(m, &FaultError_type))
 		goto err;
 
-	MissingDebugInfoError =
-		PyErr_NewExceptionWithDoc("_drgn.MissingDebugInfoError",
-					  drgn_MissingDebugInfoError_DOC, NULL,
-					  NULL);
-	if (!MissingDebugInfoError)
-		goto err;
-	if (PyModule_AddObject(m, "MissingDebugInfoError",
-			       MissingDebugInfoError)) {
-		Py_DECREF(MissingDebugInfoError);
-		goto err;
-	}
-
-	ObjectAbsentError =
-		PyErr_NewExceptionWithDoc("_drgn.ObjectAbsentError",
-					  drgn_ObjectAbsentError_DOC,
-					  NULL, NULL);
-	if (!ObjectAbsentError)
-		goto err;
-	if (PyModule_AddObject(m, "ObjectAbsentError", ObjectAbsentError)) {
-		Py_DECREF(ObjectAbsentError);
-		goto err;
-	}
-
-	OutOfBoundsError = PyErr_NewExceptionWithDoc("_drgn.OutOfBoundsError",
-						     drgn_OutOfBoundsError_DOC,
-						     NULL, NULL);
-	if (!OutOfBoundsError)
-		goto err;
-	if (PyModule_AddObject(m, "OutOfBoundsError", OutOfBoundsError)) {
-		Py_DECREF(OutOfBoundsError);
-		goto err;
-	}
-
 	PyObject *host_platform_obj = Platform_wrap(&drgn_host_platform);
 	if (!host_platform_obj)
 		goto err;
