@@ -9,6 +9,7 @@
 #include <string.h>
 
 #include "cfi.h"
+#include "cleanup.h"
 #include "debug_info.h"
 #include "drgn.h"
 #include "dwarf_constants.h"
@@ -108,17 +109,17 @@ drgn_stack_trace_num_frames(struct drgn_stack_trace *trace)
 LIBDRGN_PUBLIC struct drgn_error *
 drgn_format_stack_trace(struct drgn_stack_trace *trace, char **ret)
 {
-	struct string_builder str = STRING_BUILDER_INIT;
+	STRING_BUILDER(str);
 	for (size_t frame = 0; frame < trace->num_frames; frame++) {
 		if (!string_builder_appendf(&str, "#%-2zu ", frame))
-			goto enomem;
+			return &drgn_enomem;
 
 		struct drgn_register_state *regs = trace->frames[frame].regs;
 		struct optional_uint64 pc;
 		const char *name = drgn_stack_frame_name(trace, frame);
 		if (name) {
 			if (!string_builder_append(&str, name))
-				goto enomem;
+				return &drgn_enomem;
 		} else if ((pc = drgn_register_state_get_pc(regs)).has_value) {
 			Dwfl_Module *dwfl_module =
 				regs->module ? regs->module->dwfl_module : NULL;
@@ -133,15 +134,15 @@ drgn_format_stack_trace(struct drgn_stack_trace *trace, char **ret)
 							    sym.name,
 							    pc.value - sym.address,
 							    sym.size))
-					goto enomem;
+					return &drgn_enomem;
 			} else {
 				if (!string_builder_appendf(&str, "0x%" PRIx64,
 							    pc.value))
-					goto enomem;
+					return &drgn_enomem;
 			}
 		} else {
 			if (!string_builder_append(&str, "???"))
-				goto enomem;
+				return &drgn_enomem;
 		}
 
 		int line, column;
@@ -150,38 +151,35 @@ drgn_format_stack_trace(struct drgn_stack_trace *trace, char **ret)
 		if (filename && column) {
 			if (!string_builder_appendf(&str, " (%s:%d:%d)",
 						    filename, line, column))
-				goto enomem;
+				return &drgn_enomem;
 		} else if (filename) {
 			if (!string_builder_appendf(&str, " (%s:%d)", filename,
 						    line))
-				goto enomem;
+				return &drgn_enomem;
 		}
 
 		if (frame != trace->num_frames - 1 &&
 		    !string_builder_appendc(&str, '\n'))
-			goto enomem;
+			return &drgn_enomem;
 	}
-	if (!(*ret = string_builder_null_terminate(&str)))
-		goto enomem;
+	if (!string_builder_null_terminate(&str))
+		return &drgn_enomem;
+	*ret = string_builder_steal(&str);
 	return NULL;
-
-enomem:
-	free(str.str);
-	return &drgn_enomem;
 }
 
 LIBDRGN_PUBLIC struct drgn_error *
 drgn_format_stack_frame(struct drgn_stack_trace *trace, size_t frame, char **ret)
 {
-	struct string_builder str = STRING_BUILDER_INIT;
+	STRING_BUILDER(str);
 	struct drgn_register_state *regs = trace->frames[frame].regs;
 	if (!string_builder_appendf(&str, "#%zu at ", frame))
-		goto enomem;
+		return &drgn_enomem;
 
 	struct optional_uint64 pc = drgn_register_state_get_pc(regs);
 	if (pc.has_value) {
 		if (!string_builder_appendf(&str, "%#" PRIx64, pc.value))
-			goto enomem;
+			return &drgn_enomem;
 
 		Dwfl_Module *dwfl_module =
 			regs->module ? regs->module->dwfl_module : NULL;
@@ -194,15 +192,15 @@ drgn_format_stack_frame(struct drgn_stack_trace *trace, size_t frame, char **ret
 		    !string_builder_appendf(&str, " (%s+0x%" PRIx64 "/0x%" PRIx64 ")",
 					    sym.name, pc.value - sym.address,
 					    sym.size))
-				goto enomem;
+			return &drgn_enomem;
 	} else {
 		if (!string_builder_append(&str, "???"))
-			goto enomem;
+			return &drgn_enomem;
 	}
 
 	const char *name = drgn_stack_frame_name(trace, frame);
 	if (name && !string_builder_appendf(&str, " in %s", name))
-		goto enomem;
+		return &drgn_enomem;
 
 	int line, column;
 	const char *filename = drgn_stack_frame_source(trace, frame, &line,
@@ -210,23 +208,20 @@ drgn_format_stack_frame(struct drgn_stack_trace *trace, size_t frame, char **ret
 	if (filename && column) {
 		if (!string_builder_appendf(&str, " at %s:%d:%d", filename,
 					    line, column))
-			goto enomem;
+			return &drgn_enomem;
 	} else if (filename) {
 		if (!string_builder_appendf(&str, " at %s:%d", filename, line))
-			goto enomem;
+			return &drgn_enomem;
 	}
 
 	if (drgn_stack_frame_is_inline(trace, frame) &&
 	    !string_builder_append(&str, " (inlined)"))
-		goto enomem;
+		return &drgn_enomem;
 
-	if (!(*ret = string_builder_null_terminate(&str)))
-		goto enomem;
+	if (!string_builder_null_terminate(&str))
+		return &drgn_enomem;
+	*ret = string_builder_steal(&str);
 	return NULL;
-
-enomem:
-	free(str.str);
-	return &drgn_enomem;
 }
 
 LIBDRGN_PUBLIC const char *drgn_stack_frame_name(struct drgn_stack_trace *trace,
@@ -377,15 +372,13 @@ drgn_stack_frame_symbol(struct drgn_stack_trace *trace, size_t frame,
 		regs->module ? regs->module->dwfl_module : NULL;
 	if (!dwfl_module)
 		return drgn_error_symbol_not_found(pc.value);
-	struct drgn_symbol *sym = malloc(sizeof(*sym));
+	_cleanup_free_ struct drgn_symbol *sym = malloc(sizeof(*sym));
 	if (!sym)
 		return &drgn_enomem;
 	if (!drgn_program_find_symbol_by_address_internal(trace->prog, pc.value,
-							  dwfl_module, sym)) {
-		free(sym);
+							  dwfl_module, sym))
 		return drgn_error_symbol_not_found(pc.value);
-	}
-	*ret = sym;
+	*ret = no_cleanup_ptr(sym);
 	return NULL;
 }
 
@@ -483,14 +476,20 @@ not_found:;
 		}
 	}
 
+	const struct drgn_register_state *regs = frame->regs;
+	struct drgn_elf_file *file =
+		drgn_module_find_dwarf_file(regs->module,
+					    dwarf_cu_getdwarf(die.cu));
+	if (!file) {
+		return drgn_error_create(DRGN_ERROR_OTHER,
+					 "couldn't find file containing DIE");
+	}
 	// It doesn't make sense to use the registers if the file has a
 	// different platform than the program.
-	const struct drgn_register_state *regs = frame->regs;
-	struct drgn_elf_file *file = regs->module->debug_file;
 	if (!drgn_platforms_equal(&file->platform, &trace->prog->platform))
 		regs = NULL;
 	Dwarf_Die function_die = frame->scopes[frame->function_scope];
-	return drgn_object_from_dwarf(trace->prog->dbinfo, file, &die,
+	return drgn_object_from_dwarf(&trace->prog->dbinfo, file, &die,
 				      dwarf_tag(&die) == DW_TAG_enumerator ?
 				      &type_die : NULL,
 				      &function_die, regs, ret);
@@ -560,172 +559,156 @@ type_error:
 				 ", struct task_struct *" : "");
 }
 
+static struct drgn_error *
+drgn_get_initial_registers_from_prstatus(struct drgn_program *prog,
+					 const struct nstring *prstatus,
+					 struct drgn_register_state **ret)
+{
+	if (!prog->platform.arch->prstatus_get_initial_registers) {
+		return drgn_error_format(DRGN_ERROR_NOT_IMPLEMENTED,
+					 "core dump stack unwinding is not supported for %s architecture",
+					 prog->platform.arch->name);
+	}
+	return prog->platform.arch->prstatus_get_initial_registers(prog,
+								   prstatus->str,
+								   prstatus->len,
+								   ret);
+}
+
 static struct drgn_error *
 drgn_get_initial_registers(struct drgn_program *prog, uint32_t tid,
 			   const struct drgn_object *thread_obj,
-			   struct nstring *prstatus,
 			   struct drgn_register_state **ret)
 {
 	struct drgn_error *err;
-	struct drgn_object obj;
-	struct drgn_object tmp;
-	struct nstring _prstatus;
 
-	drgn_object_init(&obj, prog);
-	drgn_object_init(&tmp, prog);
+	DRGN_OBJECT(obj, prog);
+	DRGN_OBJECT(tmp, prog);
 
 	/* First, try pt_regs. */
 	if (thread_obj) {
 		bool is_pt_regs;
 		err = drgn_get_stack_trace_obj(&obj, thread_obj, &is_pt_regs);
 		if (err)
-			goto out;
+			return err;
 
 		if (is_pt_regs) {
 			assert(obj.encoding == DRGN_OBJECT_ENCODING_BUFFER);
 			assert(obj.kind == DRGN_OBJECT_VALUE);
 			if (!prog->platform.arch->pt_regs_get_initial_registers) {
-				err = drgn_error_format(DRGN_ERROR_INVALID_ARGUMENT,
-							"pt_regs stack unwinding is not supported for %s architecture",
-							prog->platform.arch->name);
-				goto out;
+				return drgn_error_format(DRGN_ERROR_NOT_IMPLEMENTED,
+							 "pt_regs stack unwinding is not supported for %s architecture",
+							 prog->platform.arch->name);
 			}
-			err = prog->platform.arch->pt_regs_get_initial_registers(&obj,
-										 ret);
-			goto out;
+			return prog->platform.arch->pt_regs_get_initial_registers(&obj,
+										  ret);
 		}
 	} else if (prog->flags & DRGN_PROGRAM_IS_LINUX_KERNEL) {
 		err = drgn_program_find_object(prog, "init_pid_ns", NULL,
 					       DRGN_FIND_OBJECT_ANY, &tmp);
 		if (err)
-			goto out;
+			return err;
 		err = drgn_object_address_of(&tmp, &tmp);
 		if (err)
-			goto out;
+			return err;
 		err = linux_helper_find_task(&obj, &tmp, tid);
 		if (err)
-			goto out;
+			return err;
 		bool found;
 		err = drgn_object_bool(&obj, &found);
 		if (err)
-			goto out;
+			return err;
 		if (!found) {
-			err = drgn_error_create(DRGN_ERROR_LOOKUP, "task not found");
-			goto out;
+			return drgn_error_create(DRGN_ERROR_LOOKUP,
+						 "task not found");
 		}
 	}
 
 	if (prog->flags & DRGN_PROGRAM_IS_LINUX_KERNEL) {
-		if (prog->flags & DRGN_PROGRAM_IS_LIVE) {
-			err = drgn_object_member_dereference(&tmp, &obj, "on_cpu");
-			if (!err) {
-				bool on_cpu;
-				err = drgn_object_bool(&tmp, &on_cpu);
-				if (err)
-					goto out;
-				if (on_cpu) {
-					err = drgn_error_create(DRGN_ERROR_INVALID_ARGUMENT,
-								"cannot unwind stack of running task");
-					goto out;
-				}
-			} else if (err->code == DRGN_ERROR_LOOKUP) {
-				/*
-				 * The running kernel is !SMP. Assume that the
-				 * task isn't running (which can only be wrong
-				 * for this thread itself).
-				 */
-				drgn_error_destroy(err);
-			} else {
-				goto out;
-			}
-		} else if (prstatus) {
-			goto prstatus;
+		bool on_cpu;
+		err = drgn_object_member_dereference(&tmp, &obj, "on_cpu");
+		if (!err) {
+			err = drgn_object_bool(&tmp, &on_cpu);
+			if (err)
+				return err;
+		} else if (err->code == DRGN_ERROR_LOOKUP) {
+			// The kernel must be !SMP. We have to check cpu_curr(0)
+			// instead.
+			drgn_error_destroy(err);
+			err = linux_helper_cpu_curr(&tmp, 0);
+			if (err)
+				return err;
+			int cmp;
+			err = drgn_object_cmp(&tmp, &obj, &cmp);
+			if (err)
+				return err;
+			on_cpu = cmp == 0;
 		} else {
-			prstatus = &_prstatus;
-			/*
-			 * For kernel core dumps, we look up the PRSTATUS note
-			 * by CPU rather than by PID. This is because there is
-			 * an idle task with PID 0 for each CPU, so we must find
-			 * the idle task by CPU. Rather than making PID 0 a
-			 * special case, we handle all tasks this way.
-			 */
+			return err;
+		}
+		if (on_cpu) {
+			if (prog->flags & DRGN_PROGRAM_IS_LIVE) {
+				return drgn_error_create(DRGN_ERROR_INVALID_ARGUMENT,
+							 "cannot unwind stack of running task");
+			}
+			// The PID field in PRSTATUS is unreliable for the
+			// kernel for multiple reasons:
+			//
+			// 1. Each CPU has its own idle task, all with PID 0.
+			// 2. Kdump on s390x populates the PID field with the
+			//    CPU number + 1.
+			// 3. QEMU's dump-guest-memory command does the same.
+			//
+			// So, for the kernel, we index the PRSTATUS notes by
+			// CPU number instead.
+			//
+			// Note that all of this is inherently racy: cpu_curr()
+			// [1], current [2], registers [3], and the stack
+			// pointer [4] are all updated at different times. This
+			// might result in confusing stack traces during a
+			// context switch, but there's not much we can do about
+			// that.
+			//
+			// 1: https://git.kernel.org/pub/scm/linux/kernel/git/torvalds/linux.git/tree/kernel/sched/core.c?h=v6.6#n6672
+			// 2: https://git.kernel.org/pub/scm/linux/kernel/git/torvalds/linux.git/tree/arch/x86/kernel/process_64.c?h=v6.6#n623
+			// 3: https://git.kernel.org/pub/scm/linux/kernel/git/torvalds/linux.git/tree/arch/x86/entry/entry_64.S?h=v6.6#n267
+			// 4: https://git.kernel.org/pub/scm/linux/kernel/git/torvalds/linux.git/tree/arch/x86/entry/entry_64.S?h=v6.6#n249
 			uint64_t cpu;
 			err = linux_helper_task_cpu(&obj, &cpu);
 			if (err)
-				goto out;
-			uint32_t prstatus_tid;
+				return err;
+			struct nstring prstatus;
 			err = drgn_program_find_prstatus_by_cpu(prog, cpu,
-								prstatus,
-								&prstatus_tid);
+								&prstatus);
 			if (err)
-				goto out;
-			if (prstatus->str) {
-				/*
-				 * The PRSTATUS note is for the CPU that the
-				 * task is assigned to, but it is not
-				 * necessarily for this task. Only use it if the
-				 * PID matches.
-				 *
-				 * Note that this isn't perfect: the PID is
-				 * populated by the kernel from "current" (the
-				 * current task) via a non-maskable interrupt
-				 * (NMI). During a context switch, the stack
-				 * pointer and current are not updated
-				 * atomically, so if the NMI arrives in the
-				 * middle of a context switch, the stack pointer
-				 * may not actually be that of current.
-				 * Therefore, the stack pointer in PRSTATUS may
-				 * not actually be for the PID in PRSTATUS.
-				 * Unfortunately, we can't easily fix this.
-				 */
-				err = drgn_object_member_dereference(&tmp, &obj, "pid");
-				if (err)
-					goto out;
-				union drgn_value value;
-				err = drgn_object_read_integer(&tmp, &value);
-				if (err)
-					goto out;
-				if (prstatus_tid == value.uvalue)
-					goto prstatus;
+				return err;
+			if (!prstatus.str) {
+				return drgn_error_format(DRGN_ERROR_LOOKUP,
+							 "CPU status not found; task_struct or core dump may be corrupted");
 			}
+			return drgn_get_initial_registers_from_prstatus(prog,
+									&prstatus,
+									ret);
 		}
 		if (!prog->platform.arch->linux_kernel_get_initial_registers) {
-			err = drgn_error_format(DRGN_ERROR_INVALID_ARGUMENT,
-						"Linux kernel stack unwinding is not supported for %s architecture",
-						prog->platform.arch->name);
-			goto out;
+			return drgn_error_format(DRGN_ERROR_NOT_IMPLEMENTED,
+						 "Linux kernel stack unwinding is not supported for %s architecture",
+						 prog->platform.arch->name);
 		}
-		err = prog->platform.arch->linux_kernel_get_initial_registers(&obj,
-									      ret);
+		return prog->platform.arch->linux_kernel_get_initial_registers(&obj,
+									       ret);
 	} else {
-		if (!prstatus) {
-			prstatus = &_prstatus;
-			err = drgn_program_find_prstatus_by_tid(prog, tid, prstatus);
-			if (err)
-				goto out;
-			if (!prstatus->str) {
-				err = drgn_error_create(DRGN_ERROR_LOOKUP,
-							"thread not found");
-				goto out;
-			}
+		struct nstring prstatus;
+		err = drgn_program_find_prstatus_by_tid(prog, tid, &prstatus);
+		if (err)
+			return err;
+		if (!prstatus.str) {
+			return drgn_error_create(DRGN_ERROR_LOOKUP,
+						 "thread not found");
 		}
-prstatus:
-		if (!prog->platform.arch->prstatus_get_initial_registers) {
-			err = drgn_error_format(DRGN_ERROR_INVALID_ARGUMENT,
-						"core dump stack unwinding is not supported for %s architecture",
-						prog->platform.arch->name);
-			goto out;
-		}
-		err = prog->platform.arch->prstatus_get_initial_registers(prog,
-									  prstatus->str,
-									  prstatus->len,
-									  ret);
+		return drgn_get_initial_registers_from_prstatus(prog, &prstatus,
+								ret);
 	}
-
-out:
-	drgn_object_deinit(&tmp);
-	drgn_object_deinit(&obj);
-	return err;
 }
 
 static void drgn_add_to_register(void *dst, size_t dst_size, const void *src,
@@ -940,7 +923,6 @@ drgn_unwind_one_register(struct drgn_program *prog, struct drgn_elf_file *file,
 			     little_endian);
 		if (rule->kind == DRGN_CFI_RULE_AT_REGISTER_PLUS_OFFSET)
 			address += rule->offset;
-		address &= drgn_platform_address_mask(&prog->platform);
 		err = drgn_program_read_memory(prog, buf, address, size, false);
 		if (!err && rule->kind == DRGN_CFI_RULE_AT_REGISTER_ADD_OFFSET) {
 			drgn_add_to_register(buf, size, buf, size, rule->offset,
@@ -1075,7 +1057,7 @@ drgn_unwind_with_cfi(struct drgn_program *prog, struct drgn_cfi_row **row,
 static struct drgn_error *drgn_get_stack_trace(struct drgn_program *prog,
 					       uint32_t tid,
 					       const struct drgn_object *obj,
-					       struct nstring *prstatus,
+					       const struct nstring *prstatus,
 					       struct drgn_stack_trace **ret)
 {
 	struct drgn_error *err;
@@ -1086,7 +1068,7 @@ static struct drgn_error *drgn_get_stack_trace(struct drgn_program *prog,
 	}
 	if ((prog->flags & (DRGN_PROGRAM_IS_LINUX_KERNEL |
 			    DRGN_PROGRAM_IS_LIVE)) == DRGN_PROGRAM_IS_LIVE) {
-		return drgn_error_create(DRGN_ERROR_INVALID_ARGUMENT,
+		return drgn_error_create(DRGN_ERROR_NOT_IMPLEMENTED,
 					 "stack unwinding is not yet supported for live processes");
 	}
 
@@ -1102,7 +1084,12 @@ static struct drgn_error *drgn_get_stack_trace(struct drgn_program *prog,
 	struct drgn_cfi_row *row = drgn_empty_cfi_row;
 
 	struct drgn_register_state *regs;
-	err = drgn_get_initial_registers(prog, tid, obj, prstatus, &regs);
+	if (prstatus) {
+		err = drgn_get_initial_registers_from_prstatus(prog, prstatus,
+							       &regs);
+	} else {
+		err = drgn_get_initial_registers(prog, tid, obj, &regs);
+	}
 	if (err)
 		goto out;
 
@@ -1168,7 +1155,7 @@ drgn_thread_stack_trace(struct drgn_thread *thread,
 			struct drgn_stack_trace **ret)
 {
 	if (thread->prog->flags & DRGN_PROGRAM_IS_LINUX_KERNEL) {
-		struct nstring *prstatus =
+		const struct nstring *prstatus =
 		        thread->prstatus.str ? &thread->prstatus : NULL;
 		return drgn_get_stack_trace(thread->prog, thread->tid,
 					    &thread->object, prstatus, ret);
