@@ -41,8 +41,7 @@ struct string_builder {
 	/**
 	 * Current string buffer.
 	 *
-	 * This may be reallocated when appending. It must be freed with @c
-	 * free() when it will no longer be used.
+	 * This may be reallocated when appending.
 	 */
 	char *str;
 	/** Length of @c str. */
@@ -54,15 +53,44 @@ struct string_builder {
 /** String builder initializer. */
 #define STRING_BUILDER_INIT { 0 }
 
+/** Free memory allocated by a @ref string_builder. */
+static inline void string_builder_deinit(struct string_builder *sb)
+{
+	free(sb->str);
+}
+
 /**
- * Null-terminate and return a string from a @ref string_builder.
+ * Define and initialize a @ref string_builder named @p sb that is automatically
+ * deinitialized when it goes out of scope.
+ */
+#define STRING_BUILDER(sb)					\
+	__attribute__((__cleanup__(string_builder_deinit)))	\
+	struct string_builder sb = STRING_BUILDER_INIT
+
+/**
+ * Steal the string buffer from a @ref string_builder.
+ *
+ * The string builder can no longer be used except to be passed to @ref
+ * string_builder_deinit(), which will do nothing.
+ *
+ * @return String buffer. This must be freed with @c free().
+ */
+static inline char *string_builder_steal(struct string_builder *sb)
+{
+	char *str = sb->str;
+	sb->str = NULL;
+	return str;
+}
+
+/**
+ * Null-terminate a @ref string_builder.
  *
  * This appends a null character without incrementing @ref string_builder::len.
  *
- * @return @ref string_builder::str on success, @c NULL on error (if we couldn't
- * allocate memory).
+ * @return @c true on success, @c false on error (if we couldn't allocate
+ * memory).
  */
-char *string_builder_null_terminate(struct string_builder *sb);
+bool string_builder_null_terminate(struct string_builder *sb);
 
 /**
  * Resize the buffer of a @ref string_builder to a given capacity.
