@@ -1,8 +1,10 @@
 # Copyright (c) Meta Platforms, Inc. and affiliates.
 # SPDX-License-Identifier: LGPL-2.1-or-later
 
+import contextlib
 import functools
-from typing import Any, NamedTuple, Optional
+import os
+from typing import Any, Mapping, NamedTuple, Optional
 import unittest
 from unittest.mock import Mock
 
@@ -119,17 +121,42 @@ def assertReprPrettyEqualsStr(obj):
     pretty_printer_mock.text.assert_called_with("...")
 
 
+_IDENTICAL_EQ_TYPES = (
+    type(None),
+    Language,
+    Program,
+    TypeEnumerator,
+    TypeKind,
+    bool,
+    float,
+    int,
+    str,
+)
+_IDENTICAL_MAPPING_TYPES = (dict,)
+_IDENTICAL_SEQUENCE_TYPES = (list, tuple)
+_IDENTICAL_SUPPORTED_TYPES = (
+    _IDENTICAL_EQ_TYPES
+    + _IDENTICAL_MAPPING_TYPES
+    + _IDENTICAL_SEQUENCE_TYPES
+    + (Object, Type, TypeMember, TypeParameter, TypeTemplateParameter)
+)
+
+
 def identical(a, b):
     """
     Return whether two objects are "identical".
 
-    drgn.Object, drgn.Type, drgn.TypeMember, or drgn.TypeParameter objects are
-    identical iff they have they have the same type and identical attributes.
-    Note that for drgn.Object, this is different from the objects comparing
-    equal: their type, address, value, etc. must be identical.
+    drgn.Object, drgn.Type, drgn.TypeMember, drgn.TypeParameter, and
+    drgn.TypeTemplateParameter objects are identical iff they have they have
+    the same type and identical attributes. Note that for drgn.Object, this is
+    different from the objects comparing equal: their type, address, value,
+    etc. must be identical.
 
-    Two sequences are identical iff they have the same type, length, and all of
-    their items are identical.
+    Two mappings or sequences are identical iff they have the same type,
+    length, and all of their items are identical.
+
+    Types in _IDENTICAL_EQ_TYPES are identical iff they have the same type and
+    they compare equal.
     """
     compared_types = set()
 
@@ -139,13 +166,28 @@ def identical(a, b):
                 return False
         return True
 
+    def _identical_mapping(a, b):
+        return len(a) == len(b) and all(
+            _identical(key_a, key_b) and _identical(value_a, value_b)
+            for (key_a, value_a), (key_b, value_b) in zip(a.items(), b.items())
+        )
+
     def _identical_sequence(a, b):
         return len(a) == len(b) and all(
             _identical(elem_a, elem_b) for elem_a, elem_b in zip(a, b)
         )
 
     def _identical(a, b):
-        if isinstance(a, Object) and isinstance(b, Object):
+        if (
+            type(a) not in _IDENTICAL_SUPPORTED_TYPES
+            or type(b) not in _IDENTICAL_SUPPORTED_TYPES
+        ):
+            raise NotImplementedError(f"can't compare {type(a)} to {type(b)}")
+        if type(a) != type(b):  # noqa: E721
+            return False
+
+        t = type(a)
+        if t == Object:
             if not _identical_attrs(
                 a,
                 b,
@@ -173,7 +215,7 @@ def identical(a, b):
                 return exc_a or _identical(value_a, value_b)
             else:
                 return True
-        elif isinstance(a, Type) and isinstance(b, Type):
+        elif t == Type:
             if a.qualifiers != b.qualifiers:
                 return False
             if a._ptr == b._ptr:
@@ -211,27 +253,27 @@ def identical(a, b):
                     if hasattr(a, name) or hasattr(b, name)
                 ],
             )
-        elif isinstance(a, TypeMember) and isinstance(b, TypeMember):
+        elif t == TypeMember:
             return _identical_attrs(a, b, ("object", "name", "bit_offset"))
-        elif isinstance(a, TypeParameter) and isinstance(b, TypeParameter):
+        elif t == TypeParameter:
             return _identical_attrs(a, b, ("default_argument", "name"))
-        elif isinstance(a, TypeTemplateParameter) and isinstance(
-            b, TypeTemplateParameter
-        ):
+        elif t == TypeTemplateParameter:
             return _identical_attrs(a, b, ("argument", "name", "is_default"))
-        elif (isinstance(a, tuple) and isinstance(b, tuple)) or (
-            isinstance(a, list) and isinstance(b, list)
-        ):
+        elif t in _IDENTICAL_EQ_TYPES:
+            return a == b
+        elif t in _IDENTICAL_MAPPING_TYPES:
+            return _identical_mapping(a, b)
+        elif t in _IDENTICAL_SEQUENCE_TYPES:
             return _identical_sequence(a, b)
         else:
-            return a == b
+            assert False
 
     return _identical(a, b)
 
 
-# Wrapper class that defines == using identical(). This lets us use unittest's
-# nice formatting of assert{,Not}Equal() failures.
-class _AssertIdenticalWrapper:
+# Wrapper class that defines == using identical(). This is useful for
+# unittest.mock.Mock.assert_called_with().
+class IdenticalMatcher:
     def __init__(self, obj):
         self._obj = obj
 
@@ -242,21 +284,17 @@ class _AssertIdenticalWrapper:
         return repr(self._obj)
 
     def __eq__(self, other):
-        if not isinstance(other, _AssertIdenticalWrapper):
-            return NotImplemented
-        return identical(self._obj, other._obj)
+        if isinstance(other, IdenticalMatcher):
+            other = other._obj
+        return identical(self._obj, other)
 
 
 class TestCase(unittest.TestCase):
     def assertIdentical(self, a, b, msg=None):
-        return self.assertEqual(
-            _AssertIdenticalWrapper(a), _AssertIdenticalWrapper(b), msg
-        )
+        return self.assertEqual(IdenticalMatcher(a), IdenticalMatcher(b), msg)
 
     def assertNotIdentical(self, a, b, msg=None):
-        return self.assertNotEqual(
-            _AssertIdenticalWrapper(a), _AssertIdenticalWrapper(b), msg
-        )
+        return self.assertNotEqual(IdenticalMatcher(a), IdenticalMatcher(b), msg)
 
     def bool(self, value):
         return Object(self.prog, "_Bool", value=value)
@@ -332,3 +370,24 @@ class MockProgramTestCase(TestCase):
             self.prog.add_memory_segment(
                 phys_addr, len(buf), functools.partial(mock_memory_read, buf), True
             )
+
+
+@contextlib.contextmanager
+def modifyenv(vars: Mapping[str, Optional[str]]):
+    to_restore = []
+    for key, value in vars.items():
+        old_value = os.environ.get(key)
+        if value != old_value:
+            if value is None:
+                del os.environ[key]
+            else:
+                os.environ[key] = value
+            to_restore.append((key, old_value))
+    try:
+        yield
+    finally:
+        for key, old_value in to_restore:
+            if old_value is None:
+                del os.environ[key]
+            else:
+                os.environ[key] = old_value
