@@ -91,7 +91,14 @@ skip_unless_have_test_kmod = unittest.skipUnless(
     "DRGN_TEST_KMOD" in os.environ, "test requires drgn_test Linux kernel module"
 )
 
-HAVE_FULL_MM_SUPPORT = NORMALIZED_MACHINE_NAME in ("aarch64", "s390x", "x86_64")
+# Please keep this in sync with docs/support_matrix.rst and the module
+# docstring in drgn/helpers/linux/mm.py.
+HAVE_FULL_MM_SUPPORT = NORMALIZED_MACHINE_NAME in (
+    "aarch64",
+    "ppc64",
+    "s390x",
+    "x86_64",
+)
 
 skip_unless_have_full_mm_support = unittest.skipUnless(
     HAVE_FULL_MM_SUPPORT,
@@ -294,6 +301,49 @@ _mlock.restype = ctypes.c_int
 _mlock.argtypes = [ctypes.c_void_p, ctypes.c_size_t]
 
 
+_MAPS_RE = re.compile(
+    rb"(?P<start>[0-9a-f]+)-(?P<end>[0-9a-f]+) (?P<flags>\S+) (?P<offset>[0-9a-f]+) (?P<dev_major>[0-9a-f]+):(?P<dev_minor>[0-9a-f]+) (?P<ino>[0-9]+)\s*(?P<path>.*)"
+)
+
+
+class VmMap(NamedTuple):
+    start: int
+    end: int
+    read: bool
+    write: bool
+    execute: bool
+    shared: bool
+    offset: int
+    dev: int
+    ino: int
+    path: str
+
+    def is_gate(self) -> bool:
+        # Arm has a gate VMA named "vectors", and x86 has one named "vsyscall".
+        return self.path in ("[vectors]", "[vsyscall]")
+
+
+def iter_maps(pid="self"):
+    with open(f"/proc/{pid}/maps", "rb") as f:
+        for line in f:
+            match = _MAPS_RE.match(line)
+            flags = match["flags"]
+            yield VmMap(
+                start=int(match["start"], 16),
+                end=int(match["end"], 16),
+                read=b"r" in flags,
+                write=b"w" in flags,
+                execute=b"x" in flags,
+                shared=b"s" in flags,
+                offset=int(match["offset"], 16),
+                dev=os.makedev(
+                    int(match["dev_major"], 16), int(match["dev_minor"], 16)
+                ),
+                ino=int(match["ino"]),
+                path=os.fsdecode(match["path"]),
+            )
+
+
 def mlock(addr, len):
     _check_ctypes_syscall(_mlock(addr, len))
 
@@ -310,19 +360,3 @@ def create_socket(*args, **kwds):
             raise unittest.SkipTest("kernel does not support TCP")
         else:
             raise
-
-
-@contextlib.contextmanager
-def setenv(key, value):
-    old_value = os.environ.get(key)
-    try:
-        if value is not None:
-            os.environ[key] = value
-        elif old_value is not None:
-            del os.environ[key]
-        yield
-    finally:
-        if old_value is None:
-            del os.environ[key]
-        else:
-            os.environ[key] = old_value
