@@ -12,22 +12,24 @@ from tests.linux_kernel import (
     LinuxKernelTestCase,
     fork_and_sigwait,
     skip_unless_have_full_mm_support,
+    skip_unless_have_stack_tracing,
     skip_unless_have_test_kmod,
 )
 
 
 class TestIdentifyAddress(LinuxKernelTestCase):
+    @skip_unless_have_test_kmod
     def test_identify_symbol(self):
-        symbol = self.prog.symbol("__schedule")
+        symbol = self.prog.symbol("drgn_test_function")
 
-        self.assertIn(
+        self.assertEqual(
             identify_address(self.prog, symbol.address),
-            ("symbol: __sched_text_start+0x0", "function symbol: __schedule+0x0"),
+            "function symbol: drgn_test_function+0x0",
         )
 
         self.assertEqual(
             identify_address(self.prog, symbol.address + 1),
-            "function symbol: __schedule+0x1",
+            "function symbol: drgn_test_function+0x1",
         )
 
     @skip_unless_have_full_mm_support
@@ -53,12 +55,17 @@ class TestIdentifyAddress(LinuxKernelTestCase):
         start_addr = (pfn_to_virt(self.prog["min_low_pfn"])).value_()
         end_addr = (pfn_to_virt(self.prog["max_pfn"]) + self.prog["PAGE_SIZE"]).value_()
 
-        self.assertIsNone(identify_address(self.prog, start_addr - 1))
+        # On s390x, the start address is 0, and identify_address() doesn't
+        # allow a negative address.
+        if start_addr > 0:
+            self.assertIsNone(identify_address(self.prog, start_addr - 1))
         self.assertIsNone(identify_address(self.prog, end_addr))
         self.assertIsNone(identify_address(self.prog, self.prog["drgn_test_va"]))
 
 
 class TestPrintAnnotatedStack(LinuxKernelTestCase):
+    @skip_unless_have_stack_tracing
+    @skip_unless_have_test_kmod
     def test_print_annotated_stack(self):
         with fork_and_sigwait() as pid:
             trace = self.prog.stack_trace(pid)
