@@ -10,7 +10,7 @@ import sys
 import tempfile
 import unittest
 
-from drgn import FaultError
+from drgn import NULL, FaultError
 from drgn.helpers.linux.mm import (
     PFN_PHYS,
     PHYS_PFN,
@@ -30,6 +30,7 @@ from drgn.helpers.linux.mm import (
     follow_page,
     follow_pfn,
     follow_phys,
+    for_each_vma,
     page_size,
     page_to_pfn,
     page_to_phys,
@@ -42,12 +43,15 @@ from drgn.helpers.linux.mm import (
     virt_to_page,
     virt_to_pfn,
     virt_to_phys,
+    vma_find,
     vmalloc_to_page,
     vmalloc_to_pfn,
 )
 from drgn.helpers.linux.pid import find_task
 from tests.linux_kernel import (
     LinuxKernelTestCase,
+    fork_and_sigwait,
+    iter_maps,
     mlock,
     prng32,
     skip_unless_have_full_mm_support,
@@ -295,14 +299,21 @@ class TestMm(LinuxKernelTestCase):
     @skip_unless_have_full_mm_support
     def test_access_process_vm_big(self):
         task = find_task(self.prog, os.getpid())
-        with self._pages() as (map, address, _):
-            self.assertEqual(access_process_vm(task, address, len(map)), map[:])
-            self.assertEqual(
-                access_process_vm(task, address + 1, len(map) - 1), map[1:]
-            )
-            self.assertEqual(
-                access_process_vm(task, address + 1, len(map) - 2), map[1:-1]
-            )
+        # 32M = 2**(log2(16K) + log2(16K / 8)), so 32MB + 1 is enough so that
+        # even with 16KB pages on a 64-bit architecture, we're guaranteed to
+        # read across two second-level page table entries. (The same for 64KB
+        # pages is 512MB + 1, which is too big to test reliably in vmtest.)
+        max_size = 32 * 1024 * 1024 + 1
+        expected = os.getrandom(max_size)
+        while len(expected) < max_size:
+            expected += os.getrandom(max_size - len(expected))
+        address = ctypes.memmove(expected, expected, 0)
+        expected = memoryview(expected)
+        for size in (1, 65537, max_size):
+            with self.subTest(size=size):
+                self.assertEqual(
+                    access_process_vm(task, address, size), expected[:size]
+                )
 
     @skip_unless_have_full_mm_support
     def test_access_remote_vm_init_mm(self):
@@ -333,6 +344,9 @@ class TestMm(LinuxKernelTestCase):
         task = find_task(self.prog, os.getpid())
         self.assertEqual(cmdline(task), proc_cmdline)
 
+    def test_cmdline_kernel_thread(self):
+        self.assertIsNone(cmdline(find_task(self.prog, 2)))
+
     @skip_unless_have_full_mm_support
     def test_environ(self):
         with open("/proc/self/environ", "rb") as f:
@@ -340,6 +354,58 @@ class TestMm(LinuxKernelTestCase):
         task = find_task(self.prog, os.getpid())
         self.assertEqual(environ(task), proc_environ)
 
+    def test_environ_kernel_thread(self):
+        self.assertIsNone(environ(find_task(self.prog, 2)))
+
+    def test_vma_find(self):
+        with fork_and_sigwait() as pid:
+            mm = find_task(self.prog, pid).mm
+
+            prev_end = 0
+            for map in iter_maps(pid):
+                # Gate VMAs are not included in vma_find().
+                if map.is_gate():
+                    continue
+
+                if map.start != prev_end:
+                    self.assertIdentical(
+                        vma_find(mm, prev_end),
+                        NULL(self.prog, "struct vm_area_struct *"),
+                    )
+                    self.assertIdentical(
+                        vma_find(mm, map.start - 1),
+                        NULL(self.prog, "struct vm_area_struct *"),
+                    )
+                vma = vma_find(mm, map.start)
+                self.assertEqual((map.start, map.end), (vma.vm_start, vma.vm_end))
+
+                vma = vma_find(mm, (map.start + map.end) // 2)
+                self.assertEqual((map.start, map.end), (vma.vm_start, vma.vm_end))
+
+                vma = vma_find(mm, map.end - 1)
+                self.assertEqual((map.start, map.end), (vma.vm_start, vma.vm_end))
+
+                prev_end = map.end
+
+            self.assertIdentical(
+                vma_find(mm, prev_end), NULL(self.prog, "struct vm_area_struct *")
+            )
+
+    def test_for_each_vma(self):
+        with fork_and_sigwait() as pid:
+            self.assertEqual(
+                [
+                    (vma.vm_start, vma.vm_end)
+                    for vma in for_each_vma(find_task(self.prog, pid).mm)
+                ],
+                [
+                    (map.start, map.end)
+                    for map in iter_maps(pid)
+                    # Gate VMAs are not included in for_each_vma().
+                    if not map.is_gate()
+                ],
+            )
+
     def test_totalram_pages(self):
         with open("/proc/meminfo") as f:
             lines = f.read().splitlines()
