@@ -43,3 +43,16 @@ class TestVMCore(LinuxVMCoreTestCase):
         # This assumes that we crashed from vmtest.enter_kdump. I don't know
         # why anyone would run these tests from kdump otherwise.
         self.assertEqual(crashed_thread.object.comm.string_(), b"selfdestruct")
+
+    def test_crashed_thread_stack_trace(self):
+        self.assertIn("sysrq", str(self.prog.crashed_thread().stack_trace()))
+
+    def test_crashed_thread_stack_trace_by_tid(self):
+        self.assertIn(
+            "sysrq", str(self.prog.stack_trace(self.prog.crashed_thread().tid))
+        )
+
+    def test_crashed_thread_stack_trace_by_task_struct(self):
+        self.assertIn(
+            "sysrq", str(self.prog.stack_trace(self.prog.crashed_thread().object))
+        )
