@@ -10,12 +10,12 @@ from typing import Dict, Mapping, NamedTuple, Sequence
 
 from util import NORMALIZED_MACHINE_NAME
 
-# Kernel versions that we run tests on and therefore support. We always support
-# the current mainline, stable, and longterm versions from kernel.org (although
-# there may be some delay before a new mainline version is fully supported). We
-# keep support for EOL versions until it becomes too difficult to do so.
-# Anything else is not tested and therefore best-effort.
+# Kernel versions that we run tests on and therefore support. Keep this in sync
+# with docs/support_matrix.rst.
 SUPPORTED_KERNEL_VERSIONS = (
+    "6.7",
+    "6.6",
+    "6.5",
     "6.4",
     "6.3",
     "6.2",
@@ -38,7 +38,7 @@ SUPPORTED_KERNEL_VERSIONS = (
 )
 
 KERNEL_ORG_COMPILER_VERSION = "12.2.0"
-VMTEST_KERNEL_VERSION = 21
+VMTEST_KERNEL_VERSION = 23
 
 
 BASE_KCONFIG = """
@@ -170,28 +170,40 @@ KERNEL_FLAVORS = OrderedDict(
                 CONFIG_SLUB=y
                 # For slab tests.
                 CONFIG_SLUB_DEBUG=y
+                CONFIG_RANDOMIZE_BASE=y
             """,
         ),
         KernelFlavor(
             name="alternative",
-            description="SLAB allocator",
+            description="SLAB allocator, no KASLR",
             config="""
                 CONFIG_SMP=y
                 CONFIG_SLAB=y
+                # Linux kernel commit eb07c4f39c3e ("mm/slab: rename
+                # CONFIG_SLAB to CONFIG_SLAB_DEPRECATED") (in v6.5) renamed the
+                # option for SLAB.
+                CONFIG_SLAB_DEPRECATED=y
+                CONFIG_RANDOMIZE_BASE=n
             """,
         ),
         KernelFlavor(
             name="tiny",
-            description="!SMP, !PREEMPT, and SLOB allocator",
+            description="no SMP, no PREEMPT, no KASLR, and SLUB_TINY or SLOB allocator",
             config="""
                 CONFIG_SMP=n
                 CONFIG_SLOB=y
                 # Linux kernel commit 149b6fa228ed ("mm, slob: rename CONFIG_SLOB to
                 # CONFIG_SLOB_DEPRECATED") (in v6.2) renamed the option for SLOB.
                 CONFIG_SLOB_DEPRECATED=y
+                # Linux kernel commit c9929f0e344a ("mm/slob: remove
+                # CONFIG_SLOB") (in v6.4) removed SLOB. Use SLUB_TINY instead,
+                # which was introduced in Linux kernel commit e240e53ae0ab
+                # ("mm, slub: add CONFIG_SLUB_TINY") (in v6.2).
+                CONFIG_SLUB_TINY=y
                 # CONFIG_PREEMPT_DYNAMIC is not set
                 CONFIG_PREEMPT_NONE=y
                 # !PREEMPTION && !SMP will also select TINY_RCU.
+                CONFIG_RANDOMIZE_BASE=n
             """,
         ),
     )
@@ -206,6 +218,8 @@ class Architecture(NamedTuple):
     kernel_arch: str
     # Directory under arch/ in the Linux kernel source tree.
     kernel_srcarch: str
+    # Name of the architecture in Debian.
+    debian_arch: str
     # Linux kernel configuration options.
     kernel_config: str
     # Flavor-specific Linux kernel configuration options.
@@ -226,6 +240,7 @@ ARCHITECTURES = {
             name="aarch64",
             kernel_arch="arm64",
             kernel_srcarch="arm64",
+            debian_arch="arm64",
             kernel_config="""
                 CONFIG_PCI_HOST_GENERIC=y
                 CONFIG_RTC_CLASS=y
@@ -255,6 +270,7 @@ ARCHITECTURES = {
             name="arm",
             kernel_arch="arm",
             kernel_srcarch="arm",
+            debian_arch="armhf",
             kernel_config="""
                 CONFIG_NR_CPUS=8
                 CONFIG_HIGHMEM=y
@@ -281,13 +297,14 @@ ARCHITECTURES = {
                 """,
             },
             kernel_org_compiler_name="arm-linux-gnueabi",
-            qemu_options=("-M", "virt"),
+            qemu_options=("-M", "virt,highmem=off"),
             qemu_console="ttyAMA0",
         ),
         Architecture(
             name="ppc64",
             kernel_arch="powerpc",
             kernel_srcarch="powerpc",
+            debian_arch="ppc64el",
             kernel_config="""
                 CONFIG_PPC64=y
                 CONFIG_CPU_LITTLE_ENDIAN=y
@@ -297,6 +314,9 @@ ARCHITECTURES = {
                 CONFIG_RTC_CLASS=y
                 CONFIG_RTC_DRV_GENERIC=y
                 CONFIG_HVC_CONSOLE=y
+                # This has a missing dependency in v6.5-rc1 that causes a build
+                # failure, and we don't need it.
+                CONFIG_CRYPTO_AES_GCM_P10=n
             """,
             kernel_flavor_configs={},
             kernel_org_compiler_name="powerpc64-linux",
@@ -307,6 +327,7 @@ ARCHITECTURES = {
             name="s390x",
             kernel_arch="s390",
             kernel_srcarch="s390",
+            debian_arch="s390x",
             kernel_config="""
                 # Needed for CONFIG_KEXEC_FILE.
                 CONFIG_CRYPTO_SHA256_S390=y
@@ -320,6 +341,7 @@ ARCHITECTURES = {
             name="x86_64",
             kernel_arch="x86_64",
             kernel_srcarch="x86",
+            debian_arch="amd64",
             kernel_config="""
                 CONFIG_RTC_CLASS=y
                 CONFIG_RTC_DRV_CMOS=y
@@ -344,6 +366,14 @@ class Kernel(NamedTuple):
     path: Path
 
 
+def local_kernel(arch: Architecture, path: Path) -> Kernel:
+    return Kernel(
+        arch=arch,
+        release=(path / "build/include/config/kernel.release").read_text().strip(),
+        path=path,
+    )
+
+
 class Compiler(NamedTuple):
     target: Architecture
     bin: Path
