@@ -1,5 +1,5 @@
 # Copyright (c) Meta Platforms, Inc. and affiliates.
-# SPDX-License-Identifier: GPL-3.0-or-later
+# SPDX-License-Identifier: LGPL-2.1-or-later
 
 import json
 from pathlib import Path
@@ -154,7 +154,12 @@ class AioGitHubApi(_GitHubApiBase):
         data: Any = None,
     ) -> Any:
         return self._session.request(
-            method, url, params=params, headers=headers, data=data
+            method,
+            url,
+            params=params,
+            headers=headers,
+            data=data,
+            raise_for_status=True,
         )
 
     async def _cached_get_json(self, endpoint: str, cache: _CACHE) -> Any:
