@@ -102,6 +102,10 @@ _PATCHES = (
             (KernelVersion("4.5"), KernelVersion("4.9.303")),
         ),
     ),
+    _Patch(
+        name="sched-work-around-mystery-QEMU-hang.patch",
+        versions=((KernelVersion("5.11"), KernelVersion("5.14")),),
+    ),
 )
 
 
@@ -112,7 +116,7 @@ async def apply_patches(kernel_dir: Path) -> None:
         .decode()
         .strip()
     )
-    logging.info("applying patches for kernel version %s", version)
+    logger.info("applying patches for kernel version %s", version)
     any_applied = False
     for patch in _PATCHES:
         for min_version, max_version in patch.versions:
@@ -122,7 +126,7 @@ async def apply_patches(kernel_dir: Path) -> None:
                 break
         else:
             continue
-        logging.info("applying %s", patch.name)
+        logger.info("applying %s", patch.name)
         any_applied = True
         proc = await asyncio.create_subprocess_exec(
             "git",
@@ -147,9 +151,9 @@ async def apply_patches(kernel_dir: Path) -> None:
                 sys.stderr.buffer.write(stderr)
                 sys.stderr.buffer.flush()
                 raise
-            logging.info("already applied")
+            logger.info("already applied")
     if not any_applied:
-        logging.info("no patches")
+        logger.info("no patches")
 
 
 class KBuild:
@@ -491,15 +495,19 @@ MODULE_LICENSE("GPL");
                 env=self._env,
             )
 
-            # `make modules_install` creates these as symlinks to the absolute
-            # path of the source directory. Delete them, make build a
-            # directory, and make source a symlink to build.
+            # `make modules_install` creates build as a symlink to the absolute
+            # path of the build directory. Delete it and make it an empty
+            # directory for us to populate.
             modules_build_dir = modules_dir / "build"
-            modules_source_dir = modules_dir / "source"
             modules_build_dir.unlink()
             modules_build_dir.mkdir()
-            modules_source_dir.unlink()
-            modules_source_dir.symlink_to("build")
+            # Before Linux kernel commit d8131c2965d5 ("kbuild: remove
+            # $(MODLIB)/source symlink") (in v6.6), source is a symlink to the
+            # absolute path of the source directory. It's not needed.
+            try:
+                (modules_dir / "source").unlink()
+            except FileNotFoundError:
+                pass
 
             logger.info("copying vmlinux")
             vmlinux = modules_build_dir / "vmlinux"
